****************************************************************************************
*                                                                                      *
*  DragonsLair.Screen.s :  Library for Screens handling functions.                     *
*                                                                                      *
****************************************************************************************
*  Authors : Antoine VIGNAU / Olivier ZARDINI  *  Brutal Deluxe Software  *  Jan 2022  *
****************************************************************************************

                    ASC        0D,'This game is nipple free. You can Twitch it Kate !',0D

************************************************************************************************
*  StartupScreen() :  Display Text Startup Screen + Load Data 001,002,003 files (OUT:A=Error). *
************************************************************************************************
StartupScreen       JSR        Set40ColTextPage1    ; Display Text Page #1 / 40 Column
                    SEP        #$30
                    LDA        #$F0                 ; Color : Text = White Bakckgound = Black
                    STAL       TBCOLOR
                    LDA        #$00                 ; Color : Border = Black 
                    STAL       CLOCKCTL
                    REP        #$30
                    JSR        ClearTextPage1       ; Clear Page Text #1
*--
SS_TextScreen       LDA        #SS_TabLine          ; Display Startup Page
                    JSR        BuildTextScreen
*--
                    JSR        ResetTimerSec        ; Start Timer
*--
SS_Load_001         LDA        #$0001               ; Load Data001.bin        
                    LDX        BankData001
                    LDY        #$0000               ; Don't ask for floppy
                    JSR        LoadDataFile
                    BCC        SS_Load_002
                    LDA        #$0001               ; Can't load file 001
                    BRA        SS_End
*--
SS_Load_002         LDA        #$0002               ; Load Data002.bin        
                    LDX        BankData002
                    LDY        #$0000
                    JSR        LoadDataFile
                    BCC        SS_Load_003
                    LDA        #$0002               ; Can't load file 002
                    BRA        SS_End
*--
SS_Load_003         LDA        #$0003               ; Load Data003.bin        
                    LDX        BankData003
                    LDY        #$0000
                    JSR        LoadDataFile
                    BCC        SS_Text_2Sec
                    LDA        #$0003               ; Can't load file 003
                    BRA        SS_End
*--
SS_Text_2Sec        JSR        GetTimerSec          ; We need to wait at least 2 sec
                    CMP        #$0002
                    BMI        SS_Text_2Sec
*--
                    LDA        BankData002
                    JSL        CopySoundKBDToRAM    ; Copy Keyboard Sound Data to DOC Ram
*--
                    LDA        #$0000               ; No Error
SS_End              RTS
                    
*----------   StartupScreen Variables   ---------                    
SS_TabLine          DA         SS_Line1,SS_Line2,SS_Line3,SS_Line4,SS_Line5
                    HEX        0000

SS_Line1            HEX        0400,0000            ; Y=4, X=0
                    ASC        "        Sullivan Bluth presents         ",00
SS_Line2            HEX        0800,0000            ; Y=8, X=0
                    ASC        "        -=(  Dragon's Lair  )=-         ",00
SS_Line3            HEX        0A00,0000            ; Y=8, X=0
                    ASC        "                                        ",00
SS_Line4            HEX        0F00,0000            ; Y=15, X=0
                    ASC        " Home computers adaptation by ReadySoft ",00
SS_Line5            HEX        1600,0000            ; Y=22, X=0
                    ASC        "> Apple //gs version by Brutal Deluxe < ",00
                    
                    
**********************************************************************
*  ScoreLifeScreen() :  Display Score/Life Screen (IN:A=FinalScore). *
**********************************************************************
ScoreLifeScreen     STA        SLS_FinalScore+1     ; 1 if Final Score
                    JSR        FadeOut              ; FadeOut to Black
*--
                    LDA        #$0003               ; Scene 03 / Type 00 => Score + Lives Screen
                    LDX        #$0000               ; Type
                    LDY        #$0000               ; Frame      
                    JSR        GetSceneOffset       ; => X=Offset, Y=Length                  
                    LDA        BankData001          ; A=XX00
                    ORA        CurrentBank          ;   00YY
                    JSR        UnpackLZ4            ; A = BankSrc,BankDst / X = 1st Packed Byte offset / Y = Pack Data Size
*--
                    JSR        CopyPixelSCB         ; Copy Pixel + SCB                    
*--
SLS_ScoreNumber     STZ        SLS_SN_1_Dozen+1
                    LDA        SLS_Score1
SLS_ScoreNumber_1   CMP        #$000A
                    BGE        SLS_ScoreNumber_2
                    STA        SLS_SN_1_Unit+1
                    BRA        SLS_SN_1_Dozen
SLS_ScoreNumber_2   SEC
                    SBC        #$000A
                    INC        SLS_SN_1_Dozen+1
                    BRA        SLS_ScoreNumber_1
*--     
SLS_SN_1_Dozen      LDA        #$AAAA               ; Score 1 : Dozen
                    ASL
                    TAX
                    LDAL       NumberTable,X
                    TAX                             ; X = Src @                    
                    LDY        #$3F4F               ; Y = Dst @
                    LDA        #$0B20               ; A = Width4/Heigh
                    CLC                             ; 01/2000
                    JSL        CopyTransBitmap
*--
SLS_SN_1_Unit       LDA        #$AAAA               ; Score 1 : Unit
                    ASL
                    TAX
                    LDAL       NumberTable,X
                    TAX                             ; X = Src @                    
                    LDY        #$3F65               ; Y = Dst @
                    LDA        #$0B20               ; A = Width4/Heigh
                    CLC                             ; 01/2000
                    JSL        CopyTransBitmap
*--                    
                    LDA        SLS_Score2           ; Score 2
                    ASL
                    TAX
                    LDAL       NumberTable,X
                    TAX                             ; X = Src @                    
                    LDY        #$3F7B               ; Y = Dst @
                    LDA        #$0B20               ; A = Width4/Heigh
                    CLC                             ; 01/2000
                    JSL        CopyTransBitmap
*-- 
                    LDA        SLS_Score3           ; Score 3
                    ASL
                    TAX
                    LDAL       NumberTable,X
                    TAX                             ; X = Src @                    
                    LDY        #$3F91               ; Y = Dst @
                    LDA        #$0B20               ; A = Width4/Heigh
                    CLC                             ; 01/2000
                    JSL        CopyTransBitmap
*--
                    LDA        SLS_Score4           ; Score 4
                    ASL
                    TAX
                    LDAL       NumberTable,X
                    TAX                             ; X = Src @                    
                    LDY        #$3FA7               ; Y = Dst @
                    LDA        #$0B20               ; A = Width4/Heigh
                    CLC                             ; 01/2000
                    JSL        CopyTransBitmap
*--
                    LDA        SLS_Score5           ; Score 5
                    ASL
                    TAX
                    LDAL       NumberTable,X
                    TAX                             ; X = Src @                    
                    LDY        #$3FBD               ; Y = Dst @
                    LDA        #$0B20               ; A = Width4/Heigh
                    CLC                             ; 01/2000
                    JSL        CopyTransBitmap
*--
SLS_LifeNumber      LDA        SLS_Life             ; Life
                    ASL
                    TAX
                    LDAL       NumberTable,X
                    TAX                             ; X = Src @                    
                    LDY        #$7406               ; Y = Dst @
                    LDA        #$0B20               ; A = Width4/Heigh
                    CLC                             ; 01/2000
                    JSL        CopyTransBitmap
*--
SLS_FinalScore      LDA        #$AAAA               ; 1 if Final Score
                    BEQ        SLS_MissionIsOver
                    LDX        #FinalScore          ; X = Src @                    
                    LDY        #$2219               ; Y = Dst @
                    LDA        #$1709               ; A = Width4/Heigh
                    CLC                             ; 01/2000
                    JSL        CopyTransBitmap                                     
*--
SLS_MissionIsOver   LDA        SLS_Life             ; Mission Is Over
                    BNE        SLS_FadeIn
                    LDX        #MissionIsOver       ; X = Src @                    
                    LDY        #$2208               ; Y = Dst @
                    LDA        #$280B               ; A = Width4/Heigh
                    CLC                             ; 01/2000
                    JSL        CopyTransBitmap
*--
SLS_FadeIn          LDA        #BUFFER32KB          ; FadeIn to Credit Screen
                    LDY        #$FFFF               ; Don't Copy Pixel, Only Palette
                    JSR        FadeIN
*--
SLS_PlaySound       LDX        #$B3F2               ; X=Offset
                    LDY        #$47F4               ; Y=Length
                    LDA        BankData001          ; A=XX00
                    CLC                             ; Sound Data is NOT compressed
                    JSL        CopySoundDataToRAM   ; Install Sound data in RAM
                    LDA        #$0003               ; Scene ID
                    LDX        #$0000               ; Screen ID
                    CLC                             ; Game Sound
                    JSL        PlaySoundData        ; Play Sound                
*--
                    RTS

*----------   ScoreLifeScreen Variables   ---------
SLS_String1         ASC        'FINAL SCORE',00


***************************************************
*  ResetScoreLife() :  Reset Score & Life values. *
***************************************************
ResetScoreLife      STZ        SLS_Score1           ; Reset Score
                    STZ        SLS_Score2
                    STZ        SLS_Score3
                    STZ        SLS_Score4
                    STZ        SLS_Score5
*--
                    LDA        #$0003               ; Life = 3
                    STA        SLS_Life
*--
                    RTS


*************************************************************************************
*  SetScoreLife() :  Set Score & Life values (IN:X=Life|Score1,IN:Y=Score2|Score3). *
*************************************************************************************
SetScoreLife        TXA                             ; Set Life
                    XBA
                    AND        #$00FF
                    AND        #$0003               ; 0-3
                    STA        SLS_Life
*--
                    TXA                             ; Set Score
                    AND        #$00FF
                    STA        SLS_Score1
                    TYA
                    XBA
                    AND        #$00FF
                    STA        SLS_Score2
                    TYA
                    AND        #$00FF
                    STA        SLS_Score3
*--
                    RTS


***********************************************************************************************
*  GetScoreLife() :  Return Score & Life information (OUT:X=Life|Score1,OUT:Y=Score2|Score3). *
***********************************************************************************************
GetScoreLife        LDA        SLS_Life             ; Score + Life
                    XBA
                    ORA        SLS_Score1
                    TAX
                    LDA        SLS_Score2
                    XBA
                    ORA        SLS_Score3
                    TAY
                    RTS


***************************************************************
*  DecrementLife() :  Reset Score & Life values (OUT:A=Life). *
***************************************************************
DecrementLife       LDA        SLS_Life             ; Life--
                    BEQ        DL_End
                    DEC
                    STA        SLS_Life
*--
DL_End              RTS


******************************************************
*  IncrementScore() :  Increment Score value by 100. *
******************************************************
IncrementScore      LDA        SLS_Score3           ; Score += 100
                    INC
                    STA        SLS_Score3
                    CMP        #$000A
                    BNE        IS_End
                    STZ        SLS_Score3
*--
                    LDA        SLS_Score2
                    INC
                    STA        SLS_Score2
                    CMP        #$000A
                    BNE        IS_End
                    STZ        SLS_Score2
*--                   
                    INC        SLS_Score1
*--
IS_End              RTS

*----------   Score & Life Variables   ---------
SLS_Score1          HEX        0000                 ; Score 0-B
SLS_Score2          HEX        0000
SLS_Score3          HEX        0000
SLS_Score4          HEX        0000                 ; Always 0
SLS_Score5          HEX        0000                 ; Always 0
SLS_Life            HEX        0300                 ; Life 0-3

                    ASC        0D,'Good news Alex, you may now Load/Save your current game ! We hope you will enjoy the WaterMelon icon',0D

******************************************************************************
*  DifficultyScreen() :  Process Difficulty Screen (IN:A=Level,OUT:A=Level). *
******************************************************************************
DifficultyScreen    STA        DS_Level             ; Previous Difficulty Level
*--
                    JSR        FadeOut              ; FadeOut to Black
*--
                    JSR        ClearKDBStrobe       ; Reset Key Pressed
*--
                    LDX        #$7CFE               ; Clear Pixel
                    LDA        #$0000               ; Blue Color
DS_ClearPixel       STAL       $012000,X
                    DEX
                    DEX
                    BPL        DS_ClearPixel
*--
                    LDX        #$00FE               ; Clear SCB
                    LDA        #$0000               ; Palette #0
DS_ClearSCB         STAL       $019D00,X
                    DEX
                    DEX
                    BPL        DS_ClearSCB                 
*--
                    LDX        #$01FE               ; Clear 16 Palettes
                    LDA        #$0000
DS_ClearPalette     STA        BUFFERPALETTE,X
                    DEX
                    DEX
                    BPL        DS_ClearPalette
*--
                    LDX        #$01FE               ; Copy 16 colors Palette
DS_CopyPalette      LDA        DS_Palette,X
                    STA        BUFFERPALETTE,X
                    DEX
                    DEX
                    BPL        DS_CopyPalette
*--
DS_DisplayString    LDX        #DS_String1          ; Select Game Difficulty
                    LDY        #$2FC4
                    JSR        DisplayOneString
                    LDX        #DS_String2          ; None
                    LDY        #$4C8E
                    JSR        DisplayOneString                    
                    LDX        #DS_String3          ; Easy
                    LDY        #$590E
                    JSR        DisplayOneString            
                    LDX        #DS_String4          ; Medium
                    LDY        #$658E
                    JSR        DisplayOneString  
                    LDX        #DS_String5          ; Hard
                    LDY        #$720E
                    JSR        DisplayOneString
                    LDX        #DS_String6          ; Press ENTER to continue
                    LDY        #$8EBE
                    JSR        DisplayOneString  
*--
                    LDX        #Aplha_Left          ; <-
                    LDY        #$5892
                    LDA        #$030C
                    JSL        CopyOneBitmap
                    LDX        #Aplha_Right         ; ->
                    LDY        #$589A
                    LDA        #$030C
                    JSL        CopyOneBitmap                    
                    LDX        #Aplha_Down          ; v
                    LDY        #$58A2
                    LDA        #$030C
                    JSL        CopyOneBitmap                    
                    LDX        #Aplha_Up            ; ^
                    LDY        #$58AA
                    LDA        #$030C
                    JSL        CopyOneBitmap
                    LDX        #Aplha_Action        ; Action
                    LDY        #$58B2
                    LDA        #$030C
                    JSL        CopyOneBitmap
                    LDX        #Aplha_Event         ; Event
                    LDY        #$6512
                    LDA        #$030C
                    JSL        CopyOneBitmap                       
*--
DS_FadeIn           LDA        #BUFFER32KB          ; FadeIn to Credit Screen
                    LDY        #$FFFF               ; Don't Copy Pixel, Only Palette
                    JSR        FadeIN
*--
                    LDA        DS_Level
DS_EraseBullet      DEC                             ; Clear Bullet Point
                    ASL
                    TAX
                    LDA        DS_BulletPosition,X                    
                    CLC
                    ADC        #$0140               ; Bullet Point Pixels start at Line #2
                    TAX
                    LDA        #$0000               ; Blue Color
                    STAL       $012000,X
                    STAL       $012002,X
                    STAL       $012004,X
                    STAL       $0120A0,X
                    STAL       $0120A2,X
                    STAL       $0120A4,X                    
                    STAL       $012140,X
                    STAL       $012142,X
                    STAL       $012144,X
                    STAL       $0121E0,X
                    STAL       $0121E2,X
                    STAL       $0121E4,X
                    STAL       $012280,X
                    STAL       $012282,X
                    STAL       $012284,X
                    STAL       $012320,X
                    STAL       $012322,X
                    STAL       $012324,X
                    STAL       $0123C0,X
                    STAL       $0123C2,X
                    STAL       $0123C4,X
                    STAL       $012460,X
                    STAL       $012462,X
                    STAL       $012464,X                
*--
DS_DisplayBullet    LDX        #Aplha_Bullet        ; Display Bullet Point
                    LDA        DS_Level
                    DEC
                    ASL
                    TAY
                    LDA        DS_BulletPosition,Y
                    CLC
                    ADC        #$2000
                    TAY
                    LDA        #$030C
                    JSL        CopyOneBitmap    
*--
DS_ReadKBDBegin     SEP        #$20                 ; Read Keyboard Events
DS_ReadKBD          LDAL       KBDREAD
                    BPL        DS_ReadKBD
                    STAL       KBDSTRB
                    CMP        #$8B                 ; Up Arrow
                    BEQ        DS_GoUp
                    CMP        #$8A                 ; Down Arrow
                    BEQ        DS_GoDown
                    CMP        #$8D                 ; Enter
                    BEQ        DS_GoExit
                    CMP        #$A0                 ; Space                    
                    BEQ        DS_GoExit
                    CMP        #$B0                 ; 0  
                    BEQ        DS_GoExit
                    BRA        DS_ReadKBD
*--
DS_GoUp             REP        #$30                 ; Go Up
                    JSL        PlaySoundKBD         ; Notify Key Pressed
                    LDA        DS_Level
                    CMP        #$0001
                    BEQ        DS_ReadKBDBegin
                    DEC        DS_Level
                    JMP        DS_EraseBullet
*--
DS_GoDown           REP        #$30                 ; Go Down
                    JSL        PlaySoundKBD         ; Notify Key Pressed
                    LDA        DS_Level
                    CMP        #$0004
                    BEQ        DS_ReadKBDBegin
                    INC        DS_Level
                    JMP        DS_EraseBullet              
*--
DS_GoExit           REP        #$30
                    JSL        PlaySoundKBD         ; Notify Key Pressed
                    LDA        DS_Level             ; Send Back Chosen Difficulty Level
                    RTS


*************************************************************************************
*  GetDifficulty() :  Return Difficulty Level for the Game (OUT:A=DifficultyLevel). *
*************************************************************************************
GetDifficulty       LDA        DS_Level             ; Send Back Difficulty Level
                    RTS
                    
*----------   DifficultyScreen / GetDifficulty Variables   ---------
DS_Level            HEX        0000                 ; 1 (None) 2 (Easy) 3 (Medium) 4 (Hard)
DS_Palette          HEX        4A00,FF0F,0000,800F,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000
DS_String1          ASC        'Select Game Difficulty',00
DS_String2          ASC        'None     Movie',00
DS_String3          ASC        'Easy',00
DS_String4          ASC        'Medium',00
DS_String5          ASC        'Hard     Arcade',00
DS_String6          ASC        'Press ENTER to continue',00
DS_BulletPosition   HEX         E72B,6738,E744,6751


*************************************************************
*  InsertDiskScreen() :  Display Disk Screen (IN:A=DiskID). *
*************************************************************
InsertDiskScreen    PHA                             ; A = Disk Number : 1-11 (1-9 A B)
                    JSR        FadeOut              ; FadeOut to Black
*--
                    LDA        #$0004               ; Scene 04 / Type 00 => Disk Screen
                    LDX        #$0000               ; Type
                    LDY        #$0000               ; Frame      
                    JSR        GetSceneOffset       ; => X=Offset, Y=Length                  
                    LDA        BankData002          ; A=XX00
                    ORA        CurrentBank          ;   00YY
                    JSR        UnpackLZ4            ; A = BankSrc,BankDst / X = 1st Packed Byte offset / Y = Pack Data Size
*--
                    JSR        CopyPixelSCB         ; Copy Pixel + SCB
*--
                    LDA        #$00A0
                    STA        BUFFERPALETTE+12     ; Patch Color #6 = Green
*--
                    PLA                             ; Copy Disk Number : 1-11
                    DEC
                    ASL
                    TAX
                    LDAL       DiskTable,X          
                    TAX
                    CLC                             ; Src @
                    ADC        #$0004
                    STA        IDS_Src+1                    
IDS_BitmapBank_1    LDAL       $AA0000,X            ; $AA = Bitmap Bank, Width4 / Height
                    STA        IDS_WidthHeight+1
IDS_BitmapBank_2    LDAL       $AA0002,X            ; $AA = Bitmap Bank, Screen @
                    STA        IDS_Dst+1
*--                                        
IDS_Src             LDX        #$AAAA               ; X = Src @                    
IDS_Dst             LDY        #$BBBB               ; Y = Dst @
IDS_WidthHeight     LDA        #$CCCC               ; A = Width4/Heigh
                    JSL        CopyOneBitmap
*--
IDS_FadeIn          LDA        #BUFFER32KB          ; FadeIn to Insert Disk Screen
                    LDY        #$FFFF               ; Don't Copy Pixel, Only Palette
                    JSR        FadeIN
*--
                    JSR        ClearKDBStrobe       ; Reset Key Pressed
                    JSR        WaitForKey
*--
                    RTS

*****************************************************************************************
