****************************************************************************************
*                                                                                      *
*  DragonsLair.Shared.s :  Library for shared functions.                               *
*                                                                                      *
****************************************************************************************
*  Authors : Antoine VIGNAU / Olivier ZARDINI  *  Brutal Deluxe Software  *  Jan 2022  *
****************************************************************************************

                    ASC        0D,'Hello to the league of extraordinary gentlemen (Dagen, Jason, Jesse, John, Lane) who have discovered this project few month ago',0D

************************************************************
*  ToolInitialize() :  Initialize Tools layer (OUT:A=@DP). *
************************************************************
ToolInitialize      _TLStartUp                      ; Start Tools
                    PHA
                    _MMStartUp                      ; Start Memory Manager Tool Set
                    PLA
                    STA        myID                 ; Get current ID
*--
                    _MTStartUp                      ; Start Miscellaneous Tool Set
*--
                    PushLong   #0                   ; Allocate Page Direct in Bank 00
                    PushLong   #$000100
                    PushWord   myID
                    PushWord   #$C005               ; Fixed, Page Aligned, Locked, Unpurgeable
                    PushLong   #0
                    _NewHandle
                    PLX                             ; Handle Low  Address
                    PLA                             ; Handle High Address (00XX bank)
                    XBA
                    STA        TI_1+2
TI_1                LDAL       $000000,X            ; Get Low Address  A=??/XXXX
                    PHA
*--
                    _SoundStartUp                   ; Start Sound Tool Set
*--
                    PushLong   #0                   ; Compact Memory
                    PushLong   #$8FFFFF
                    PushWord   myID
                    PushWord   #%11000000_00000000
                    PushLong   #0
                    _NewHandle
                    _DisposeHandle
                    _CompactMem
*--
                    PushLong   #0                   ; Ask Shadowing Screen ($8000 bytes from $01/2000)
                    PushLong   #$8000
                    PushWord   myID
                    PushWord   #%11000000_00000011
                    PushLong   #$012000
                    _NewHandle
                    PLA
                    PLA
*--
                    RTS


************************************************
*  ToolTerminate() :  Free Memory, Stop Tools. *
************************************************
ToolTerminate       _SoundShutDown                  ; Stop Tools
*--
                    _MTShutDown
*--
                    PushWord   myID
                    _DisposeAll
*--               
                    PushWord   myID
                    _MMShutDown
*--               
                    _TLShutDown
*--
                    RTS
               
*----------   Tools Variables   ---------
myID                ds         2                    ; ID of this Program in memory


********************************************************
*  BackupEnvironment() :  Backup Environment Settings. *
********************************************************
BackupEnvironment   SEP        #$20                 ; Backup Environment values (color, border...)
*--
                    LDAL       TBCOLOR              ; Text + Background Color
                    STA        BE_C022
*--            
                    LDAL       NEWVIDEO             ; Apple IIgs Graphic Mode
                    STA        BE_C029
*--            
                    LDAL       CLOCKCTL             ; Border Color
                    STA        BE_C034
*--            
                    LDAL       SHADOW               ; Shadowing
                    STA        BE_C035
*--
                    LDAL       RD80VID              ; 80 Column status
                    STA        BE_C01F
*--
                    REP        #$30
                    RTS


**********************************************************
*  RestoreEnvironment() :  Restore Environment Settings. *
**********************************************************
RestoreEnvironment  SEP        #$20                 ; Restore Environment values (color, border...)
*--
                    LDA        BE_C035              ; Shadowing
                    STAL       SHADOW
*--            
                    LDA        BE_C034              ; Border Color
                    STAL       CLOCKCTL
*--            
                    LDA        BE_C029              ; Apple IIgs Graphic Mode
                    STAL       NEWVIDEO
*--            
                    LDA        BE_C022              ; Text + Background Color
                    STAL       TBCOLOR
*--
                    LDA        BE_C01F              ; Restaure 80 Column if needed
                    ASL                             ; Bit #7=1 if 80 column
                    BCC        RE_End
                    STAL       SET80VID             ; Set 80 column mode      
*--
RE_End              REP        #$30
                    RTS

*--------------   Environment Variables   -------------
BE_C01F             HEX        00                   ; 80 Column status
BE_C022             HEX        00                   ; Background + Text Color
BE_C029             HEX        00                   ; Linearization of the Graphic Page
BE_C034             HEX        00                   ; Border Color
BE_C035             HEX        00                   ; Shadowing


***********************************************************************************
*  AllocOneBank() :  Allocate 1 Bank of 64 KB (OUT:A=XX/00,OUT:Carry=SetOnError). *
***********************************************************************************
AllocOneBank        PushLong   #0
                    PushLong   #$10000              ; 64 KB
                    PushWord   myID                 ; ID of my current Program
                    PushWord   #%11000000_00011100
                    PushLong   #0
                    _NewHandle
                    PLX                             ; Handle Low Address
                    PLA                             ; Handle High Address (00XX)
                    XBA
                    STA        AOB_1+2
AOB_1               LDAL       $000001,X            ; Get Bank Address (A=XX/00)
                    RTS


***************************************************************************************************************
*  MemoryAllocation() :  Allocate all required Banks of 64 KB (IN:A=MaxBank,IN:X=@TableBank,OUT:A=AllocBank). *
***************************************************************************************************************
MemoryAllocation    ASL                             ; Maximum of # Bank required x 2
                    STA        MA_3+1
                    STX        MA_2+1
                    LDX        #$0000
*--                    
MA_1                PHX                             ; Allocate one Bank of 64 KB
                    JSR        AllocOneBank
                    PLX
                    BCS        MA_End               ; Error, no more memory available
*--
MA_2                STA        $AAAA,X              ; Store Bank Number in the Table
                    INX
                    INX
*--                    
MA_3                CPX        #$AAAA               ; Don't allocate more than required
                    BMI        MA_1                
*---                    
MA_End              TXA                             ; Number of allocated Bank x 2
                    LSR      
                    RTS


********************************************************
*  WaitForKey() :  Wait for a Key pressed (OUT:A=Key). *
********************************************************
WaitForKey          SEP        #$20                 ; Wait for a Key Press
WFK_1               LDAL       KBDREAD
                    BPL        WFK_1
                    STAL       KBDSTRB
                    REP        #$30
                    AND        #$007F               ; Clear Bit #7
                    RTS


***********************************************
*  ClearKDBStrobe() :  Clear Keyboard Strobe. *
***********************************************
ClearKDBStrobe      SEP        #$20                 ; Reset any Key Pressed
                    STAL       KBDSTRB
                    REP        #$30
                    RTS


*************************************************************************
*  HasKeyPressed() :  Check for a Key pressed (OUT:Carry=SetIfPressed). *
*************************************************************************
HasKeyPressed       SEP        #$20                 ; Wait for a Key Press
                    CLC                             ; Init as 'Not Pressed'
                    LDAL       KBDREAD
                    BPL        HKP_End
                    STAL       KBDSTRB
                    SEC
HKP_End             REP        #$30
                    RTS


*****************************************************
*  ResetTimerSec() :  Start the Timer (in Seconds). *
*****************************************************
ResetTimerSec       PHA                             ; Space for result
                    PHA
                    PHA
                    PHA
                    _ReadTimeHex                    ; Read current Time/Date in HEX Format
                    PLA
                    STA        RTS_SECOND_MINUTE
                    PLA
                    STA        RTS_HOUR_YEAR
                    PLA
                    STA        RTS_DAY_MONTH
                    PLA
                    STA        RTS_NULL_WEEKDAY                                                
                    RTS

*----------   ResetTimerSec Variables   ---------
RTS_SECOND_MINUTE   HEX        00,00                ; 0-59  0-59
RTS_HOUR_YEAR       HEX        00,00                ; 0-23  Year-1900  
RTS_DAY_MONTH       HEX        00,00                ; 0-11  0-30
RTS_NULL_WEEKDAY    HEX        00,00                ; 1(Sunday)-7  NULL


*********************************************************************************
*  GetTimerSec() :  Compute the number of seconds since the Reset (OUT:A=#Sec). *
*********************************************************************************
GetTimerSec         PHA                             ; Space for result
                    PHA
                    PHA
                    PHA
                    _ReadTimeHex                    ; Read current Time/Date in HEX Format        
                    PLA
                    STA        GTS_SECOND_MINUTE
                    PLA
                    PLA
                    PLA
*--
                    LDA        GTS_SECOND_MINUTE    ; Compute difference
                    CMP        RTS_SECOND_MINUTE
                    BEQ        GetTimerSec_Same
*--
                    SEP        #$20
                    LDA        GTS_SECOND_MINUTE
                    CMP        RTS_SECOND_MINUTE
                    BMI        GetTimerSec_Min
                    SEC
                    SBC        RTS_SECOND_MINUTE
                    BRA        GetTimerSec_End  
*--                    
GetTimerSec_Min     LDA        #$3C                 ; 60 sec
                    SEC
                    SBC        RTS_SECOND_MINUTE
                    CLC
                    ADC        GTS_SECOND_MINUTE
*--                    
GetTimerSec_End     REP        #$30
                    AND        #$00FF               ; 0-59 sec 
                    RTS
*--
GetTimerSec_Same    LDA        #$0000               ; In the Same second
                    RTS
                    
*----------   GetTimerSec Variables   ---------
GTS_SECOND_MINUTE   HEX        00,00                ; 0-59  0-59


**********************************************************************************************
*  LoadFile() :  Load one File into memory (IN:X=@FilePath,IN:A=BankXX/00,OUT:Carry=Status). *
**********************************************************************************************
LoadFile            STX        LF_gsosOPEN+4        ; X = File Path 
                    STA        LF_gsosREAD+5        ; A = Bank XX/00
*--
LF_Open             JSL        GSOS                 ; Open File
                    dw         $2010                ; Code for command 'Open'
                    adrl       LF_gsosOPEN          ; Parameters Table Address (Open in Read Mode)
                    BCS        LF_ErrorEnd
                    LDA        LF_gsosOPEN+2
                    STA        LF_gsosGETEOF+2
                    STA        LF_gsosREAD+2
                    STA        LF_gsosCLOSE+2
*--
LF_GetSize          JSL        GSOS                 ; Get File Size
                    dw         $2019                ; Code for command 'GetFileSize'
                    adrl       LF_gsosGETEOF        ; Parameters Table Address
                    LDA        LF_gsosGETEOF+4
                    STA        LF_gsosREAD+8
                    LDA        LF_gsosGETEOF+6
                    STA        LF_gsosREAD+10
*--
LF_Read             JSL        GSOS                 ; Read File Content
                    dw         $2012                ; Code for command 'Read'
                    adrl       LF_gsosREAD          ; Parameters Table Address
                    BCS        LF_Error
*--
LF_Close            JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       LF_gsosCLOSE         ; Parameters Table Address
*--
LF_End              CLC                             ; No Error
                    LDA        LF_gsosGETEOF+4      ; A = File Size
                    RTS
*----
LF_Error            JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       LF_gsosCLOSE         ; Parameters Table Address
*--
LF_ErrorEnd         SEC                             ; Error
                    RTS
                         
*----------   LoadFile Variables   ---------
LF_gsosOPEN         dw         2                    ; Number of parameters (Open in Read Mode)
                    ds         2                    ; +2 File Descriptor
                    adrl       LDF_FileData_HDrive  ; +4 File Pathname (init with an existing file path, so we only have to change 2 bytes)

LF_gsosGETEOF       dw         2                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor
                    ds         4                    ; +2 eof

LF_gsosREAD         dw         4                    ; Number of parameters
                    ds         2                    ; +2  File Descriptor
                    ds         4                    ; +4  Data Buffer Address
                    ds         4                    ; +8  requestCount
                    ds         4                    ; +12 transferCount
                    
LF_gsosCLOSE        dw         1                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor


*******************************************************************************************
*  IsFileExist() :  Check if a file already exist on disk (IN:A=FilePath,OUT:A=1IfExist). *
*******************************************************************************************
IsFileExist         STA        IFE_gsosOPEN+4       ; A = File Path 
*--
IFE_Open            JSL        GSOS                 ; Open File
                    dw         $2010                ; Code for command 'Open'
                    adrl       IFE_gsosOPEN         ; Parameters Table Address (Open in Read Mode)
                    BCS        IFE_ErrorEnd
                    LDA        IFE_gsosOPEN+2
                    STA        IFE_gsosCLOSE+2
*--
IFE_Close           JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       IFE_gsosCLOSE        ; Parameters Table Address
*--
IFE_End             LDA        #$0001               ; File exist
                    RTS
*--
IFE_ErrorEnd        LDA        #$0000               ; File doesn't exist
                    RTS

*----------   IsFileExist Variables   ---------
IFE_gsosOPEN        dw         2                    ; Number of parameters (Open in Read Mode)
                    ds         2                    ; +2 File Descriptor
                    adrl       LDF_FileData_HDrive  ; +4 File Pathname (init with an existing file path, so we only have to change 2 bytes)
                    
IFE_gsosCLOSE       dw         1                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor


***********************************************************************
*  ExitToLauncher() :  Quit program and go back to Finder / Launcher. *
***********************************************************************
ExitToLauncher      JSL        GSOS                 ; Quit Program
                    dw         $2029
                    adrl       gsosQUIT               
               
*----------   GS/OS Calls Variables   ---------
gsosQUIT            dw         2                    ; Number of parameters
                    ds         4                    ; +2 pathname
                    ds         2                    ; +6 flags
                    
                    
********************************************************************************************************************
*  UnpackLZ4() :  Unpack LZ4 data buffer (IN:A=BankSrcDst,IN:X=HeaderSize,IN:Y=PackDataSize,OUT:A=UnpackDataSize). *
********************************************************************************************************************
UnpackLZ4           STA        LZ4_Literal_3+1      ; Uncompress a LZ4 Packed Data buffer (64 KB max)
                    STX        LZ4_Init_1+1         ; X = LZ4 Data
                    STX        LZ4_Init_2+1         
                    SEP        #$20                 ; A = Bank Src,Bank Dst
                    STA        LZ4_Match_5+1        
                    STA        LZ4_Match_5+2
                    XBA                             ;  => Return in A the length of unpacked Data
                    STA        LZ4_ReadToken+3  
                    STA        LZ4_Match_1+3    
                    STA        LZ4_GetLength_1+3
                    REP        #$30
                    TYA                             ; Y = Pack Data Size
                    CLC
LZ4_Init_1          ADC        #$AAAA               ; LZ4 Data offset
                    STA        LZ4_Limit+1          ; Last packed byte offset
LZ4_Init_2          LDA        #$AAAA               ; LZ4 Data
                    CLC
                    ADC        #$0010               ; LZ4 Header Size = 16 bytes
                    TAX                             ; X = First packed byte
*--
                    LDY        #BUFFER32KB          ; Init Target unpacked Data offset (#$0000 if dedicated uncompress buffer)
LZ4_ReadToken       LDAL       $AA0000,X            ; Read Token Byte
                    INX
                    STA        LZ4_Match_2+1
*-----------
LZ4_Literal         AND        #$00F0               ; >>> Process Literal Bytes <<<
                    BEQ        LZ4_Limit            ; No Literal
                    CMP        #$00F0
                    BNE        LZ4_Literal_1
                    JSR        LZ4_GetLengthLiter   ; Compute Literal Length with next bytes
                    BRA        LZ4_Literal_2
LZ4_Literal_1       LSR                             ; Literal Length use the 4 bit
                    LSR
                    LSR
                    LSR
*--
LZ4_Literal_2       DEC                             ; Copy A+1 Bytes
LZ4_Literal_3       MVN        $AA,$BB              ; Copy Literal Bytes from packed data buffer
                    PHK                             ; X and Y are auto incremented
                    PLB
*-----------
LZ4_Limit           CPX        #$AAAA               ; End Of Packed Data buffer ?
                    BEQ        LZ4_End
*-----------
LZ4_Match           TYA                             ; >>> Process Match Bytes <<<
                    SEC
LZ4_Match_1         SBCL       $AA0000,X            ; Match Offset
                    INX
                    INX
                    STA        LZ4_Match_4+1
*--
LZ4_Match_2         LDA        #$0000               ; Current Token Value
                    AND        #$000F
                    CMP        #$000F
                    BNE        LZ4_Match_3
                    JSR        LZ4_GetLengthMatch   ; Compute Match Length with next bytes
LZ4_Match_3         CLC
                    ADC        #$0003               ; Minimum Match Length is 4 (-1 for the MVN)
*--
                    PHX
LZ4_Match_4         LDX        #$AAAA               ; Match Byte Offset
LZ4_Match_5         MVN        $BB,$BB              ; Copy Match Bytes from unpacked data buffer
                    PHK                             ; X and Y are auto incremented
                    PLB
                    PLX
*-----------
                    BRA        LZ4_ReadToken
*-----------
LZ4_GetLengthLiter  LDA        #$000F               ; Compute Variable Length (Literal or Match)
LZ4_GetLengthMatch  STA        LZ4_GetLength_2+1
LZ4_GetLength_1     LDAL       $AA0000,X            ; Read Length Byte
                    INX
                    AND        #$00FF
                    CMP        #$00FF
                    BNE        LZ4_GetLength_3
                    CLC
LZ4_GetLength_2     ADC        #$000F
                    STA        LZ4_GetLength_2+1
                    BRA        LZ4_GetLength_1
LZ4_GetLength_3     ADC        LZ4_GetLength_2+1
                    RTS
*-----------
LZ4_End             TYA
                    SEC
                    SBC        #BUFFER32KB          ; A = Length of Unpack Data
                    RTS
                 
                 
**********************************************************
*  FadeOut() :  Fade Out Screen to black + Erase Screen. *
**********************************************************
FadeOut             LDY        #$000F               ; 16 Tones
*--
FOUT_1              LDX        #$01FE               ; 16 Palettes of 16 colors
*--
FOUT_2              LDAL       $019E00,X            ; Blue
                    AND        #$000F
                    BEQ        FOUT_3
                    LDAL       $019E00,X
                    SEC
                    SBC        #$0001
                    STAL       $019E00,X
FOUT_3              LDAL       $019E00,X            ; Green
                    AND        #$00F0
                    BEQ        FOUT_4
                    LDAL       $019E00,X
                    SEC
                    SBC        #$0010
                    STAL       $019E00,X
FOUT_4              LDAL       $019E00,X            ; Red
                    AND        #$0F00
                    BEQ        FOUT_5
                    LDAL       $019E00,X
                    SEC
                    SBC        #$0100
                    STAL       $019E00,X
*--
FOUT_5              DEX
                    DEX
                    BPL        FOUT_2
*--                 
FOUT_6              JSR        WaitForNextVBL       ; Tempo (1/60 sec)
*--
                    DEY
                    BPL        FOUT_1
*-----------
                    LDX        #$7FFE               ; Erase Screen
                    LDA        #$0000
FOUT_7              STAL       $012000,X
                    DEX
                    DEX
                    BPL        FOUT_7
*--         
                    RTS


*************************************************************************************
*  FadeIN() :  Fade In Screen from Black to Picture (IN:A=@Picture,IN:Y=FFFF/0000). *
*************************************************************************************
FadeIN              STA        FIN_1+1              ; Initialization
                    CLC
                    ADC        #$7E00               ; Palette
                    STA        FIN_5+1
                    STA        FIN_8+1
                    STA        FIN_11+1
                    CPY        #$FFFF               ; If Y=FFFF, no Picture Data Copy
                    BEQ        FIN_2
*--
                    LDX        #$7DFE               ; Copy Pixel + SCB 
FIN_1               LDAL       $062000,X
                    STAL       $012000,X
                    DEX
                    DEX
                    BPL        FIN_1
*--
FIN_2               LDY        #$000F               ; 16 Tones
*--
FIN_3               LDX        #$01FE               ; 16 Palettes of 16 colors
*--
FIN_4               LDAL       $019E00,X            ; Blue
                    AND        #$000F
                    STA        FIN_6+1
FIN_5               LDAL       $069E00,X
                    AND        #$000F
FIN_6               CMP        #$AAAA
                    BEQ        FIN_7                    
                    LDAL       $019E00,X
                    CLC
                    ADC        #$0001
                    STAL       $019E00,X
*--
FIN_7               LDAL       $019E00,X            ; Green
                    AND        #$00F0
                    STA        FIN_9+1
FIN_8               LDAL       $069E00,X
                    AND        #$00F0
FIN_9               CMP        #$AAAA
                    BEQ        FIN_10
                    LDAL       $019E00,X
                    CLC
                    ADC        #$0010
                    STAL       $019E00,X
*--
FIN_10              LDAL       $019E00,X            ; Red
                    AND        #$0F00
                    STA        FIN_12+1
FIN_11              LDAL       $069E00,X
                    AND        #$0F00
FIN_12              CMP        #$AAAA
                    BEQ        FIN_13
                    LDAL       $019E00,X
                    CLC
                    ADC        #$0100
                    STAL       $019E00,X
*--
FIN_13              DEX
                    DEX
                    BPL        FIN_4
*--
FIN_14              JSR        WaitForNextVBL       ; Tempo (1/60 sec)
*-- 
                    DEY
                    BPL        FIN_3
                    RTS


**********************************************************
*  WaitForNextVBL() :  Wait for one VBL to be completed. *
**********************************************************
WaitForNextVBL      LDA        #$0096               ; Wait for the Spot at Line #150
                    JSR        WaitForSpot
WFNVBL_1            LDAL       RDVBLBAR-1           ; Wait For VBL
                    BPL        WFNVBL_1
                    RTS
                    
                    
********************************************************************************************
*  WaitForSpot() :  Wait for the Video Spot to cross a screen line (IN:A=Line[0000-00C7]). *
********************************************************************************************
WaitForSpot         LSR                             ; SHR Line # / 2 : 00-63
                    CLC
                    ADC        #$0080               ; C02E Line Code : 80-E3
                    SEP        #$20
                    STA        WFS_2+1
*--
WFS_1               LDAL       VERTCNT              ; Current Code [7D-FF]
WFS_2               CMP        #$AA                 ; Expected Line Code
                    BNE        WFS_1
*--
                    REP        #$30
                    RTS
                  

****************************************************************
*  IncBorderColor() :  Increment Border Color (Debug Purpose). *
****************************************************************
IncBorderColor      SEP        #$20
                    LDAL       CLOCKCTL             ; Current Border Color
                    STA        IncBorderColor_1+1
                    INC
                    AND        #$0F                 ; New Color
                    STA        IncBorderColor_2+1
IncBorderColor_1    LDA        #$AA
                    AND        #$F0
IncBorderColor_2    ORA        #$AA
                    STAL       CLOCKCTL             ; New Border Color
                    REP	       #$30
                    RTS


*******************************************************************
*  SetBorderColor() :  Increment Border Color (IN:A=BorderColor). *
*******************************************************************
SetBorderColor      SEP        #$20
                    AND        #$0F
                    STA        SetBorderColor_1+1
                    LDAL       CLOCKCTL             ; Current Border Color
                    AND        #$F0
SetBorderColor_1    ORA        #$AA
                    STAL       CLOCKCTL             ; New Border Color
                    REP	       #$30
                    RTS


******************************************************************
*  Set40ColTextPage1() :  Display Text Page 1 in 40 column mode. *
******************************************************************
Set40ColTextPage1  	SEP        #$20
                    LDA        #$01
                    STAL       NEWVIDEO             ; Display Text Page
                    STAL       CLR80VID             ; Enforce 40 column mode
                    REP	       #$30
                    RTS


*********************************************************
*  ClearTextPage1() :  Clear the Screen of Text Page 1. *
*********************************************************
ClearTextPage1      LDY        #$002E               ; 24 lines
CTP1_1              LDA        TEXTPAGELINEADDR,Y   ; Address of the Line
                    STA        CTP1_2+1
*--
                    LDX        #$0026               ; 40 characters wide
                    LDA        #$A0A0               ; Space x 2
CTP1_2              STAL       $E0AAAA,X            ; Page Text 1 is located in $E0/0400
                    DEX
                    DEX
                    BPL        CTP1_2
*--                    
                    DEY
                    DEY
                    BPL        CTP1_1
                    RTS


***************************************************************************
*  BuildTextScreen() :  Display Text information on Screen (IN:A=@Table). *
***************************************************************************
BuildTextScreen     STA        BTS_1+1              ; Table of Line Information Address
                    LDY        #$0000
*--
BTS_1               LDA        $AAAA,Y              ; @ Line Information
                    BEQ        BTS_End
                    STA        BTS_2+1              ; @ Y
                    INC
                    INC
                    STA        BTS_3+1              ; @ X
                    INC
                    INC
                    STA        BTS_4+1              ; @ String
*--                    
BTS_2               LDA        $AAAA                ; @ Y
                    ASL
                    TAX
                    LDA        TEXTPAGELINEADDR,X 
                    CLC
BTS_3               ADC        $AAAA                ; @ X
                    STA        BTS_5+1              ; Address in Text Page where to display the String
*--
                    SEP        #$30                 ; Display Line High-Ascii value
                    LDX        #$00
BTS_4               LDA        $AAAA,X              ; @ String
                    BEQ        BTS_6
BTS_5               STAL       $E0AAAA,X            ; Page Text 1 is located in $E0/0400
                    INX
                    BRA        BTS_4
BTS_6               REP        #$30
*--
                    INY                             ; Next Line
                    INY
                    BRA        BTS_1
*--                    
BTS_End             RTS


*-------------------------------- Misc Tables ----------------------------------
                    
TEXTPAGELINEADDR    DA         $0400,$0480,$0500,$0580,$0600,$0680,$0700,$0780
                    DA         $0428,$04A8,$0528,$05A8,$0628,$06A8,$0728,$07A8      
                    DA         $0450,$04D0,$0550,$05D0,$0650,$06D0,$0750,$07D0


**************************************************************************************
*  DisplayOneString() :  Display a String on the Screen (IN:X=@String,IN:Y=@Screen). *
**************************************************************************************
DisplayOneString    STX        DOS_String+1         ; String Address
                    STY        DOS_Screen+1         ; Screen Address
                    STZ        DOS_Index+1
*--
DOS_Index           LDX        #$0000                     
DOS_String          LDA        $AAAA,X              ; String Data
                    AND        #$00FF
                    STA        DOS_Char_2+1
                    BEQ        DOS_End              ; End of String
                    INC        DOS_Index+1
*--
                    LDX        #$0000               ; Search character into 
DOS_Char_1          LDAL       Alpha_Char,X         ; authorized characters list
                    AND        #$00FF
                    BEQ        DOS_Index            ; Unknown character
DOS_Char_2          CMP        #$AAAA
                    BEQ        DOS_Display
                    INX
                    BRA        DOS_Char_1
*--                    
DOS_Display         TXA                             ; Character Bitmap Address
                    ASL
                    TAX
                    LDAL       Alpha_Tab,X
                    TAX                             ; Bitmap Address
DOS_Screen          LDY        #$AAAA               ; Screen Address
                    LDA        #$030A               ; Width4 / Heigh
                    JSL        CopyOneBitmap
*--
                    LDA        DOS_Screen+1         ; New Screen Address
                    CLC
                    ADC        #$0004
                    STA        DOS_Screen+1
                    BRA        DOS_Index
*--   
DOS_End             RTS


****************************************************************************************************
*  DisplayTransString() :  Display a Transparent String on the Screen (IN:X=@String,IN:Y=@Screen). *
****************************************************************************************************
DisplayTransString  STX        DTS_String+1         ; String Address
                    STY        DTS_Screen+1         ; Screen Address
                    STZ        DTS_Index+1
*--
DTS_Index           LDX        #$0000                     
DTS_String          LDA        $AAAA,X              ; String Data
                    AND        #$00FF
                    STA        DTS_Char_2+1
                    BEQ        DTS_End              ; End of String
                    INC        DTS_Index+1
*--
                    LDX        #$0000               ; Search character into 
DTS_Char_1          LDAL       Alpha_Char,X         ; authorized characters list
                    AND        #$00FF
                    BEQ        DTS_Index            ; Unknown character
DTS_Char_2          CMP        #$AAAA
                    BEQ        DTS_Display
                    INX
                    BRA        DTS_Char_1
*--                    
DTS_Display         TXA                             ; Character Bitmap Address
                    ASL
                    TAX
                    LDAL       Alpha_Tab,X
                    TAX                             ; Bitmap Address
DTS_Screen          LDY        #$AAAA               ; Screen Address
                    LDA        #$030A               ; Width4 / Heigh
                    SEC                             ; E1/2000
                    JSL        CopyTransBitmap
*--
                    LDA        DTS_Screen+1         ; New Screen Address
                    CLC
                    ADC        #$0004
                    STA        DTS_Screen+1
                    BRA        DTS_Index
*--   
DTS_End             RTS


****************************************************************
*  PauseGame() :  Pause Audio + Video waiting for a Key press. *
****************************************************************
PauseGame           
                    JSL        EnableDisableAudio  ; Pause Audio
*--
                    LDX        #$1FE               ; Backup Palette
PG_BP               LDAL       $019E00,X
                    STA        BUFFERPALETTE,X
                    DEX
                    DEX
                    BPL        PG_BP
*--
                    LDA        #$0000              ; For TAY 8 bits -> 16 bits
                    SEP        #$20
                    LDX        #$1FF               ; Dark Palette
PG_DP               LDAL       $019E00,X
                    TAY
                    LDA        PG_DarkPalette,Y    ; Divide by 2 RGB components
                    STAL       $019E00,X
                    DEX
                    BPL        PG_DP                    
                    REP        #$30
*--
                    JSR        WaitForKey          ; Wait for Key press
*--
                    LDX        #$1FE               ; Restaure Palette
PG_RP               LDA        BUFFERPALETTE,X
                    STAL       $019E00,X
                    DEX
                    DEX
                    BPL        PG_RP
*--
                    JSL        EnableDisableAudio  ; Restart Audio
*--
                    RTS
                    
*----------   PauseGame Variables   ---------

PG_DarkPalette      HEX        00,00,01,01,02,02,03,03,04,04,05,05,06,06,07,07
                    HEX        00,00,01,01,02,02,03,03,04,04,05,05,06,06,07,07
                    HEX        10,10,11,11,12,12,13,13,14,14,15,15,16,16,17,17
                    HEX        10,10,11,11,12,12,13,13,14,14,15,15,16,16,17,17
                    HEX        20,20,21,21,22,22,23,23,24,24,25,25,26,26,27,27
                    HEX        20,20,21,21,22,22,23,23,24,24,25,25,26,26,27,27
                    HEX        30,30,31,31,32,32,33,33,34,34,35,35,36,36,37,37
                    HEX        30,30,31,31,32,32,33,33,34,34,35,35,36,36,37,37
                    HEX        40,40,41,41,42,42,43,43,44,44,45,45,46,46,47,47
                    HEX        40,40,41,41,42,42,43,43,44,44,45,45,46,46,47,47
                    HEX        50,50,51,51,52,52,53,53,54,54,55,55,56,56,57,57
                    HEX        50,50,51,51,52,52,53,53,54,54,55,55,56,56,57,57
                    HEX        60,60,61,61,62,62,63,63,64,64,65,65,66,66,67,67
                    HEX        60,60,61,61,62,62,63,63,64,64,65,65,66,66,67,67
                    HEX        70,70,71,71,72,72,73,73,74,74,75,75,76,76,77,77
                    HEX        70,70,71,71,72,72,73,73,74,74,75,75,76,76,77,77



******************************************************************************************
*  HexDec2Hex() :  Convert a Decimal number into Hexadecimal (IN:A=NumDec,OUT:A=NumHex). *
******************************************************************************************
HexDec2Hex          STA        HD2H_H+1             ; Convert 032 as 0*100 + 3*10 + 2 = 000 + 01E + 002 = 020
                    STA        HD2H_D+1
                    AND        #$000F
                    STA        HD2H_U+1
*--
HD2H_H              LDA        #$AAAA
                    AND        #$0F00
                    XBA
                    ASL
                    TAX
                    LDA        HD2H_Tab100,X 
                    STA        HD2H_Hundred+1
*--
HD2H_D              LDA        #$AAAA
                    AND        #$00F0
                    LSR
                    LSR
                    LSR
                    LSR
                    TAX
                    LDA        HD2H_Tab10,X         ; Dozen
                    AND        #$00FF
*--
                    CLC
HD2H_U              ADC        #$AAAA               ; Unit
HD2H_Hundred        ADC        #$AAAA               ; Hundreds
*--
                    RTS                    

*----------   HexDec2Hex Variables   ---------
HD2H_Tab10          HEX        00,0A,14,1E,28,32,3C,46,50,5A,64,6E,78,82,8C,96          
HD2H_Tab100         HEX        0000,6400,C800,2C01,9001,F401,5802,BC02,2003,8403      


******************************************************************************************
*  Hex2HexDec() :  Convert a Hexadecimal number into Decimal (IN:A=NumHex,OUT:A=NumDec). *
******************************************************************************************
Hex2HexDec          AND        #$00FF               ; Convert $2A as 2*16+10 = 42 = $42
                    TAX
                    LDA        H2HD_Tab,X
                    AND        #$00FF
                    RTS                    

*----------   HexDec2Hex Variables   ---------
H2HD_Tab            HEX        00,01,02,03,04,05,06,07,08,09,10,11,12,13,14,15
                    HEX        16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31
                    HEX        32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47
                    HEX        48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63
                    HEX        64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79
                    HEX        80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95
                    HEX        96,97,98,99          


*******************************************************************
*  HBEntryPoint() :  HeartBeat Interruption Entry Point function. *
*******************************************************************
HBEntryPoint        ds         4
                    dw         1                    ; 1/60 seconds
                    dw         $A55A                ; Signature
*--
                    CLC
                    XCE
                    REP        #$30
*--
                    LDA        #$0001               ; Reset Timer
                    STAL       HBEntryPoint+4
*--
                    LDAL       HBCounter            ; Increment Counter
                    INC
                    STAL       HBCounter
*--
                    SEP        #$30
                    CLC
                    RTL
                    mx         %00
                    
                     
*****************************************************************
*  HBResetCounter() :  Reset Counter of HeartBeat Interruption. *
*****************************************************************
HBResetCounter      STZ        HBCounter            ; Set VBL counter to 0
                    STZ        HBCounter
                    STZ        HBCounter 
                    STZ        HBCounter 
                    RTS


********************************************************************************
*  HBGetCounter() :  Get HeartBeat Interruption Counter value (OUT:A=Counter). *
********************************************************************************
HBGetCounter        LDA        HBCounter            ; Get VBL counter
                    RTS

*----------   HBEntryPoint / HBResetCounter / HBGetCounter Variables   ---------
HBCounter           HEX        0000


**********************************************************************
*  SwitchTwilightOff/On() :  Turn Off / On Twilight II screen saver. *
**********************************************************************
SwitchTwilightOff   LDAL       $E11600
                    STA        $00
                    LDAL       $E11602
                    STA        $02
*--
STOff_1             LDY        #$0008
                    LDA        [$00],y
                    LDX        #$117A               ; offV1
                    CMP        #$49BF               ; lenV1
                    BEQ        STOff_2
                    LDX        #$154C               ; offV2
                    CMP        #$539A               ; lenV2
                    BNE        STOff_3
*----
STOff_2             STX        TWILIGHT_off
                    LDA        [$00]
                    STA        $04
                    STA        TWILIGHT_ptr
                    LDY        #$0002
                    LDA        [$00],y
                    STA        $06
                    STA        TWILIGHT_ptr+2
                    TXY
                    LDA        [$04],y
                    CMP        #$0EF0
                    BNE        STOff_3
                    LDA        #$0E80
                    STA        [$04],y
                    INC        TWILIGHT_fg
*--
                    RTS
*----
STOff_3             LDY        #$0010
                    LDA        [$00],y
                    TAX
                    INY
                    INY
                    LDA        [$00],y
                    STA        $02
                    TXA
                    STA        $00
                    LDA        $00
                    ORA        $02
                    BNE        STOff_1
*--
                    RTS
*--------------------------------------------------------
SwitchTwilightOn    LDA        TWILIGHT_fg
                    BNE        STon_1
                    RTS
*--
STon_1              LDA        TWILIGHT_ptr
                    STA        $04
                    LDA        TWILIGHT_ptr+2
                    STA        $06
                    LDY        TWILIGHT_off
                    LDA        #$0EF0
                    STA        [$04],y
*--
                    RTS

*----------   TWILIGHToff / TWILIGHTon Variables   ---------
TWILIGHT_ptr        DS         4
TWILIGHT_fg         DS         2
TWILIGHT_off        DS         2

********************************************************************************