****************************************************************************************
*                                                                                      *
*  DragonsLair.Sound.s :  Library for Sound Handling functions.                        *
*                                                                                      *
****************************************************************************************
*  Authors : Antoine VIGNAU / Olivier ZARDINI  *  Brutal Deluxe Software  *  Jan 2022  *
****************************************************************************************

*----------   Global Equates : Firmware / Hardware / GS/OS   ---------
CLOCKCTL            =          $00C034              ; Border Color + Clock
SOUNDCTL            =          $00C03C              ; Ensoniq 5503 Control Register
SOUNDDATA           =          $00C03D              ; Ensoniq 5503 Data Register
SOUNDADRL           =          $00C03E              ; Ensoniq 5503 Address Low
SOUNDADRH           =          $00C03F              ; Ensoniq 5503 Address High
CTRLPNLSNDVOL       =          $E100CA              ; Control Panel Sound Volume

                    ASC        0D,'See you next month, on the 30th to discover your next 30th anniversary special gift !',0D

********************************************************************************************************************************************
*  CopySoundDataToRAM() :  Copy/Uncompress Sound Data to Sound RAM (IN:A=Sound@Bank,IN:X=Sound@,IN:Y=SoundSize,IN:Carry=SoundCompression). *
********************************************************************************************************************************************
CopySoundDataToRAM  ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    BCS        CSDTR_CopyCompress   ; This sound is compressed
                    JMP        CSDTR_CopyRawData    ; This sound is raw data
*----                    
CSDTR_CopyCompress  STA        CSDTR_CC_1+2         ; Sound Bank Address A=XX00
                    STX        CSDTR_CC_1+1         ; Sound Address
                    CPY        #$7FFA
                    BLT        CSDTR_CC_Init        ; Make sure the Sound Length < 64 KB
                    LDY        #$7FFA
CSDTR_CC_Init       STY        CSDTR_CC_2+1         ; Sound Length
*--
                    JSL        StopSound            ; Stop Current Sound
*--
                    SEI                             ; Disable Interrupts
                    PHD                             ; Save Direct Page Address
                    LDA        #$C000
                    TCD                             ; Define $C000 as the new Direct Page Address
*--
                    SEP        #$20
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F
                    ORA        #$60                 ; Access Sound Ram + Auto Increment Enabled
                    STAL       SOUNDCTL
                    LDA        #$00
                    STAL       SOUNDADRL            ; Sound Ram Address = $0000
                    STAL       SOUNDADRH
                    XBA                             ; Make sure B is #$00 for the TAY               
*--
                    LDA        #$00                 ; Make sure a Sound starting at $8000 will stop immediately
                    STAL       SOUNDADRL            ; Sound Ram Address = $8000
                    LDA        #$80
                    STAL       SOUNDADRH
                    LDA        #$00
                    STA        $3D                  ; SOUNDDATA = $C03D
                    STA        $3D                  
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
*--
                    LDA        #$00                 ; Copy Sound Data into Sound RAM at $0000
                    STAL       SOUNDADRL            ; Sound Ram Address = $0000
                    STAL       SOUNDADRH
                    XBA                             ; Make sure B is #$00 for the TAY               
                    LDX        #$0000
CSDTR_CC_1          LDAL       $BBAAAA,X            ; Copy/Uncompress Sound Data  
                    TAY
                    LDA        CSDTR_UPPER_NIBBLE,Y
                    STA        $3D                  ; SOUNDDATA = $C03D
                    LDA        CSDTR_LOWER_NIBBLE,Y
                    STA        $3D                  ; SOUNDDATA = $C03D
                    INX
CSDTR_CC_2          CPX        #$AAAA               ; Sound Size
                    BNE        CSDTR_CC_1
*--
                    LDA        #$00                 ; Add few #$00 to stop the Sound
                    STA        $3D
                    STA        $3D
                    STA        $3D
*--
                    REP        #$30
                    PLD                             ; Restore Direct Page Address
                    CLI                             ; Enable Interrupts
                    PLB
                    RTL                    
*--------
CSDTR_CopyRawData   STA        CSDTR_CRD_1+2        ; Sound Bank Address A=XX00
                    STX        CSDTR_CRD_1+1        ; Sound Address
                    CPY        #$FFFA
                    BLT        CSDTR_CRD_Init       ; Make sure the Sound Length < 64 KB
                    LDY        #$FFFA
CSDTR_CRD_Init      STY        CSDTR_CRD_3+1        ; Sound Length
*--
                    JSL        StopSound            ; Stop Current Sound
*--
                    SEI                             ; Disable Interrupts
                    PHD                             ; Save Direct Page Address
                    LDA        #$C000
                    TCD                             ; Define $C000 as the new Direct Page Address
*--
                    SEP        #$20
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F
                    ORA        #$60                 ; Access Sound Ram + Auto Increment Enabled
                    STAL       SOUNDCTL
                    LDA        #$00
                    STAL       SOUNDADRL            ; Sound Ram Address = $0000
                    STAL       SOUNDADRH
                    XBA                             ; Make sure B is #$00 for the TAY               
*--
                    LDA        #$00                 ; Make sure a Sound starting at $8000 will stop immediately
                    STAL       SOUNDADRL            ; Sound Ram Address = $8000
                    LDA        #$80
                    STAL       SOUNDADRH
                    LDA        #$00
                    STA        $3D                  ; SOUNDDATA = $C03D
                    STA        $3D                  
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
*--
                    LDA        #$00                 ; Copy Sound Data into Sound RAM at $0000
                    STAL       SOUNDADRL            ; Sound Ram Address = $0000
                    STAL       SOUNDADRH
                    XBA                             ; Make sure B is #$00 for the TAY               
                    LDX        #$0000
CSDTR_CRD_1         LDAL       $BBAAAA,X            ; Copy Raw Sound Data
                    BNE        CSDTR_CRD_2
                    LDA        #$01                 ; Change #00 into #01  
CSDTR_CRD_2         STA        $3D                  ; SOUNDDATA = $C03D
                    INX
CSDTR_CRD_3         CPX        #$AAAA               ; Sound Size
                    BNE        CSDTR_CRD_1
*--
                    LDA        #$00                 ; Add few #$00 to stop the Sound
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
*--
                    REP        #$30
                    PLD                             ; Restore Direct Page Address
                    CLI                             ; Enable Interrupts
                    PLB
                    RTL                    

*----------   CopySoundDataToRAM Variables   ---------
CSDTR_UPPER_NIBBLE  HEX        01,01,01,01,01,01,01,01,01,01,01,01,01,01,01,01
                    HEX        11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11
                    HEX        22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33
                    HEX        44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66
                    HEX        77,77,77,77,77,77,77,77,77,77,77,77,77,77,77,77
                    HEX        88,88,88,88,88,88,88,88,88,88,88,88,88,88,88,88
                    HEX        99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99
                    HEX        AA,AA,AA,AA,AA,AA,AA,AA,AA,AA,AA,AA,AA,AA,AA,AA
                    HEX        BB,BB,BB,BB,BB,BB,BB,BB,BB,BB,BB,BB,BB,BB,BB,BB
                    HEX        CC,CC,CC,CC,CC,CC,CC,CC,CC,CC,CC,CC,CC,CC,CC,CC
                    HEX        DD,DD,DD,DD,DD,DD,DD,DD,DD,DD,DD,DD,DD,DD,DD,DD
                    HEX        EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF

CSDTR_LOWER_NIBBLE  HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    HEX        01,11,22,33,44,55,66,77,88,99,AA,BB,CC,DD,EE,FF
                    

********************************************************************************************************
*  CopyLongSoundData08() :  Copy Long Sound Data to Sound RAM for Scene #08 (IN:A=Bank16,IN:X:Bank18). *
********************************************************************************************************
CopyLongSoundData08 ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    STA        CLSD08_1+2           ; Sound #1 Bank Address A=XX00
                    LDA        #$0000               ; Sound #1 Address
                    STA        CLSD08_1+1
                    STX        CLSD08_3+2           ; Sound #2 Bank Address X=XX00
                    LDX        #$0000               ; Sound #2 Address
                    STX        CLSD08_3+1
*--
                    JSL        StopSound            ; Stop Current Sound
*--
                    SEI                             ; Disable Interrupts
                    PHD                             ; Save Direct Page Address
                    LDA        #$C000
                    TCD                             ; Define $C000 as the new Direct Page Address
*--
                    SEP        #$20
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F
                    ORA        #$60                 ; Access Sound Ram + Auto Increment Enabled
                    STAL       SOUNDCTL
*----
                    LDA        #$00                 ; Copy Sound #1 Data into Sound RAM at $0000
                    STAL       SOUNDADRL            ; Sound Ram Address = $0000
                    STAL       SOUNDADRH               
                    LDX        #$0000
CLSD08_1            LDAL       $BBAAAA,X            ; Copy Raw Sound Data
                    BNE        CLSD08_2
                    LDA        #$01                 ; Change #00 into #01  
CLSD08_2            STA        $3D                  ; SOUNDDATA = $C03D
                    INX
                    CPX        #$7800               ; Sound Size
                    BNE        CLSD08_1
*--
                    LDA        #$00                 ; Add few #$00 to stop the Sound
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
*----
                    LDA        #$00                 ; Copy Sound #2 Data into Sound RAM at $0000
                    STAL       SOUNDADRL            ; Sound Ram Address = $8000
                    LDA        #$80
                    STAL       SOUNDADRH               
                    LDX        #$0000
CLSD08_3            LDAL       $BBAAAA,X            ; Copy Raw Sound Data
                    BNE        CLSD08_4
                    LDA        #$01                 ; Change #00 into #01  
CLSD08_4            STA        $3D                  ; SOUNDDATA = $C03D
                    INX
                    CPX        #$7700               ; Sound Size
                    BNE        CLSD08_3
*--
                    LDA        #$00                 ; Add few #$00 to stop the Sound
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
*----
                    REP        #$30
                    PLD                             ; Restore Direct Page Address
                    CLI                             ; Enable Interrupts
                    PLB
                    RTL                    


********************************************************************************************************
*  CopyLongSoundData09() :  Copy Long Sound Data to Sound RAM for Scene #09 (IN:A=Bank27,IN:X:Bank28). *
********************************************************************************************************
CopyLongSoundData09 ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    STA        CLSD09_1+2           ; Sound #1 Bank Address A=XX00
                    LDA        #$0000               ; Sound #1 Address
                    STA        CLSD09_1+1
                    STX        CLSD09_3+2           ; Sound #2 Bank Address X=XX00
                    LDX        #$0000               ; Sound #2 Address
                    STX        CLSD09_3+1
*--
                    JSL        StopSound            ; Stop Current Sound
*--
                    SEI                             ; Disable Interrupts
                    PHD                             ; Save Direct Page Address
                    LDA        #$C000
                    TCD                             ; Define $C000 as the new Direct Page Address
*--
                    SEP        #$20
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F
                    ORA        #$60                 ; Access Sound Ram + Auto Increment Enabled
                    STAL       SOUNDCTL
*----
                    LDA        #$00                 ; Copy Sound #1 Data into Sound RAM at $0000
                    STAL       SOUNDADRL            ; Sound Ram Address = $0000
                    STAL       SOUNDADRH               
                    LDX        #$0000
CLSD09_1            LDAL       $BBAAAA,X            ; Copy Raw Sound Data
                    BNE        CLSD09_2
                    LDA        #$01                 ; Change #00 into #01  
CLSD09_2            STA        $3D                  ; SOUNDDATA = $C03D
                    INX
                    CPX        #$7800               ; Sound Size
                    BNE        CLSD09_1
*--
                    LDA        #$00                 ; Add few #$00 to stop the Sound
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
*----
                    LDA        #$00                 ; Copy Sound #2 Data into Sound RAM at $0000
                    STAL       SOUNDADRL            ; Sound Ram Address = $8000
                    LDA        #$80
                    STAL       SOUNDADRH               
                    LDX        #$0000
CLSD09_3            LDAL       $BBAAAA,X            ; Copy Raw Sound Data
                    BNE        CLSD09_4
                    LDA        #$01                 ; Change #00 into #01  
CLSD09_4            STA        $3D                  ; SOUNDDATA = $C03D
                    INX
                    CPX        #$7700               ; Sound Size
                    BNE        CLSD09_3
*--
                    LDA        #$00                 ; Add few #$00 to stop the Sound
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
*----
                    REP        #$30
                    PLD                             ; Restore Direct Page Address
                    CLI                             ; Enable Interrupts
                    PLB
                    RTL                    

                    
************************************************************************************************************************************************
*  PlaySoundData() :  Start ENSONIQ Oscillators 0 -> 3 to play Sound Sample located at $0000 (IN:A=SceneID,IN:X=ScreenID,IN:Carry=SetOnDeath). *
************************************************************************************************************************************************
PlaySoundData       ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    STA        PSD_SceneScreenID+1  ; Search for Frequency
                    TXA                             ; Screen ID
                    XBA
PSD_SceneScreenID   ORA        #$AAAA               ; Scene ID / Death ID
                    BCS        PSD_Death_Init_0     ; Game or Death Sound
*----
PSD_Game_Init_0     STA        PSD_Game_Init_2+1    ; Game Sound
                    STA        PSD_Control_1+1
                    LDX        #$0000
PSD_Game_Init_1     LDA        PSD_TabGameSceneID,X
PSD_Game_Init_2     CMP        #$AAAA
                    BEQ        PSD_Game_Init_3
                    INX
                    INX        
                    BRA        PSD_Game_Init_1
PSD_Game_Init_3     LDA        PSD_TabGameFreq,X
                    BRA        PSD_Init
*----
PSD_Death_Init_0    STA        PSD_Death_Init_2+1   ; Death Sound
                    LDX        #$0000
PSD_Death_Init_1    LDA        PSD_TabDeathSceneID,X
PSD_Death_Init_2    CMP        #$AAAA
                    BEQ        PSD_Death_Init_3
                    INX
                    INX        
                    BRA        PSD_Death_Init_1
PSD_Death_Init_3    LDA        PSD_TabDeathFreq,X
*----
PSD_Init            SEP        #$20
                    STA        PSD_Frequency_Lo_0+1
                    XBA
                    STA        PSD_Frequency_Hi_0+1
*----
                    REP        #$30
                    JSL        StopSound            ; Stop Current sound
                    SEP        #$20
*----
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F                 ; Access Sound Register + Auto Increment Disabled           
                    STAL       SOUNDCTL
*----
PSD_Enable          LDA        #$E1                 ; Oscillators Enable Register
                    STAL       SOUNDADRL
                    LDA        #$3E                 ; # Oscillators x 2 => We want all the 32 oscillators running
                    STAL       SOUNDDATA
*----
PSD_Frequency       LDA        #$00                 ; Frequency Low Register for Oscillator #00 -> #03 ($#00 -> $#03)
                    STA        PSD_Frequency_1+1
                    LDA        #$20
                    STA        PSD_Frequency_2+1
                    LDX        #$0003
*--
PSD_Frequency_1     LDA        #$00
                    STAL       SOUNDADRL
PSD_Frequency_Lo_0  LDA        #$D6                 ; Frequency Low value (Hz*32/1645), 11025 hz => 214 = D6
                    STAL       SOUNDDATA
PSD_Frequency_2     LDA        #$20                 ; Frequency High Register for Oscillator #00 -> #$03 (#$20 -> #$23)
                    STAL       SOUNDADRL
PSD_Frequency_Hi_0  LDA        #$00                 ; Frequency High value (Hz*32/1645)
                    STAL       SOUNDDATA
*--
                    INC        PSD_Frequency_1+1
                    INC        PSD_Frequency_2+1
                    DEX
                    BPL        PSD_Frequency_1                  
*----                    
PSD_Volume          LDA        #$40                 ; Volume Register for Oscillator #00 -> #03 (#$40 -> #$43)
                    STA        PSD_Volume_1+1
                    LDX        #$0003
*--
PSD_Volume_1        LDA        #$40
                    STAL       SOUNDADRL
                    JSL        GetMuteStatus        ; Volume : 00 (Min) / FF (Max)
                    STAL       SOUNDDATA
*--
                    INC        PSD_Volume_1+1
                    DEX
                    BPL        PSD_Volume_1
*----
PSD_Address         LDA        #$80                 ; Sound Address Register for Oscillator #00
                    STAL       SOUNDADRL
                    LDA        #$00                 ; Sound Address = $0000 => $00
                    STAL       SOUNDDATA         
*--
                    LDA        #$81                 ; Sound Address Register for Oscillator #01
                    STAL       SOUNDADRL
                    LDA        #$80                 ; Sound Address = $8000 => $80
                    STAL       SOUNDDATA
*--
                    LDA        #$82                 ; Sound Address Register for Oscillator #02
                    STAL       SOUNDADRL
                    LDA        #$00                 ; Sound Address = $0000 => $00
                    STAL       SOUNDDATA
*--
                    LDA        #$83                 ; Sound Address Register for Oscillator #03
                    STAL       SOUNDADRL
                    LDA        #$80                 ; Sound Address = $8000 => $80
                    STAL       SOUNDDATA
*----
PSD_Size            LDA        #$C0                 ; Size Register for Oscillator #00 -> #03 (#$C0 -> #$C3)
                    STA        PSD_Size_1+1
                    LDX        #$0003
*--
PSD_Size_1          LDA        #$C0
                    STAL       SOUNDADRL
                    LDA        #$3F                 ; Sound Size = 32 KB
                    STAL       SOUNDDATA
*--
                    INC        PSD_Size_1+1
                    DEX
                    BPL        PSD_Size_1
*----
                    REP        #$30
                    LDX        #$0000               ; Play Sound in Swap Mode (32 KB + 32 KB) or Loop Mode (32 KB)
PSD_Control         LDA        PSD_TabLoopSceneID,X
                    BEQ        PSD_Control_Swap
PSD_Control_1       CMP        #$AAAA               ; SceneID | ScreenID
                    BEQ        PSD_Control_Loop
                    INX
                    INX
                    BRA        PSD_Control
*----
PSD_Control_Swap    SEP        #$20
                    LDA        #$A0                 ; Control Register for Oscillator #00
                    STAL       SOUNDADRL
                    LDA        #$06                 ; Canal #0=0000 / No Interrupt=0 / Swap=11 / Start Sound=0
                    STAL       SOUNDDATA
*--
                    LDA        #$A1                 ; Control Register for Oscillator #01
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
*--
                    LDA        #$A2                 ; Control Register for Oscillator #02
                    STAL       SOUNDADRL
                    LDA        #$16                 ; Canal #1=0001 / No Interrupt=0 / Swap=11 / Start Sound=0
                    STAL       SOUNDDATA
*--
                    LDA        #$A3                 ; Control Register for Oscillator #03
                    STAL       SOUNDADRL
                    LDA        #$13                 ; Canal #1=0001 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
                    BRA        PSD_End
*----
PSD_Control_Loop    SEP        #$20
                    LDA        #$A0                 ; Control Register for Oscillator #00
                    STAL       SOUNDADRL
                    LDA        #$06                 ; Canal #0=0000 / No Interrupt=0 / Swap=11 / Start Sound=0
                    STAL       SOUNDDATA
*--
                    LDA        #$A1                 ; Control Register for Oscillator #01
                    STAL       SOUNDADRL
                    LDA        #$07                 ; Canal #0=0000 / No Interrupt=0 / Swap=11 / Stop Sound=1
                    STAL       SOUNDDATA
*--
                    LDA        #$A2                 ; Control Register for Oscillator #02
                    STAL       SOUNDADRL
                    LDA        #$16                 ; Canal #1=0001 / No Interrupt=0 / Swap=11 / Start Sound=0
                    STAL       SOUNDDATA
*--
                    LDA        #$A3                 ; Control Register for Oscillator #03
                    STAL       SOUNDADRL
                    LDA        #$17                 ; Canal #1=0001 / No Interrupt=0 / Swap=11 / Stop Sound=1
                    STAL       SOUNDDATA
*----
PSD_End             REP        #$30
                    PLB
                    RTL                 

*----------   PlaySoundData Variables   ---------                     
PSD_TabGameSceneID  HEX        0100,0200,0300,0400,0600,0601,0700,0701,0800,0801,0802  ; Game SceneID | ScreenID
                    HEX        0900,0901,0A00,0A01,0A02,0A03,0A04,0A05,0A06,0A07,0B00
                    HEX        0B01,0B02,0C00,0C01,0D00,0D01,0D02,0D03,0D04,0D05,0D06
                    HEX        0E00,0F00,0F01,0F02,0F03,0F04,0F05,1000,1001,1002,1003
                    HEX        1004,1005,1100
PSD_TabGameFreq     HEX        D600,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00  ; FreqOsc=FreqHz*32/1645 (D6=11Khz, 9B=8Khz)           
                    HEX        9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00
                    HEX        9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00
                    HEX        9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00
                    HEX        9B00,9B00,9B00
*--
PSD_TabDeathSceneID HEX        0200,0300,0400,0500,0600,0700,0800,0900,0A00,0B00,0F00  ; Death ID | ScreenID
                    HEX        1000,1100,1200
PSD_TabDeathFreq    HEX        9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00,9B00  ; FreqOsc=FreqHz*32/1645 (D6=11Khz, 9B=8Khz)           
                    HEX        9B00,9B00,9B00
*--
PSD_TabLoopSceneID  HEX        0601,0900,0901,0D00,1100,0000                           ; Loop Sound if SceneID|ScreenID is here


************************************************************************************************************
*  CheckForPlaySound() :  Check if we need to play a Sound (IN:A=SceneID,IN:X=FrameID,OUT:Carry=SetIfPlay) *
************************************************************************************************************
CheckForPlaySound   ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    STX        CFPS_FrameID+1       ; Check if we need to play a Sound for this Frame
                    ASL
                    TAX
                    LDA        CFPS_TabGameSceneID,X
                    STA        CFPS_FrameList+1
*--
                    LDX        #$0000
CFPS_FrameList      LDA        $AAAA,X
                    BEQ        CFPS_NoSound         ; No More Frame
CFPS_FrameID        CMP        #$BBBB
                    BEQ        CFPS_PlaySound
                    INX
                    INX
                    BRA        CFPS_FrameList
*--
CFPS_PlaySound      SEC
                    PLB
                    RTL
CFPS_NoSound        CLC
                    PLB
                    RTL

*----------   CheckForPlaySound Variables   ---------
CFPS_TabGameSceneID DA         CFPS_Scene00,CFPS_Scene01,CFPS_Scene02
                    DA         CFPS_Scene03,CFPS_Scene04,CFPS_Scene05
                    DA         CFPS_Scene06,CFPS_Scene07,CFPS_Scene08
                    DA         CFPS_Scene09,CFPS_Scene0A,CFPS_Scene0B
                    DA         CFPS_Scene0C,CFPS_Scene0D,CFPS_Scene0E
                    DA         CFPS_Scene0F,CFPS_Scene10,CFPS_Scene11
                    HEX        0000
CFPS_Scene00        HEX        0000
CFPS_Scene01        HEX        0000
CFPS_Scene02        HEX        0000
CFPS_Scene03        HEX        0000
CFPS_Scene04        HEX        0000
CFPS_Scene05        HEX        0000            
CFPS_Scene06        HEX        0F00,2700,0000           
CFPS_Scene07        HEX        0200,1400,2D00,0000           
CFPS_Scene08        HEX        0100,3400,4700,0000           
CFPS_Scene09        HEX        0100,8900,0000           
CFPS_Scene0A        HEX        0100,0E00,1300,3800,5E00,6900,7800,8C00,9E00,B800,0000            
CFPS_Scene0B        HEX        0100,2300,2F00,0000            
CFPS_Scene0C        HEX        0700,2700,0000            
CFPS_Scene0D        HEX        0100,4000,5700,6E00,8500,9C00,B300,0000            
CFPS_Scene0E        HEX        0B00,0000
CFPS_Scene0F        HEX        0100,1900,3400,5200,7B00,9500,0000
CFPS_Scene10        HEX        0100,1E00,2C00,4700,6200,7800,0000                     
CFPS_Scene11        HEX        0600,0000


*************************************************************************************
*  CopySoundKBDToRAM() :  Copy Keyboard Sound Data to Sound RAM (IN:A=BankData002). *
*************************************************************************************
CopySoundKBDToRAM   ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    STA        CSKBDTR_1+2          ; Sound Bank Address A=XX00
                    LDA        #$1E71               ; Sound Address
                    STA        CSKBDTR_1+1        
*--
                    SEI                             ; Disable Interrupts
                    PHD                             ; Save Direct Page Address
                    LDA        #$C000
                    TCD                             ; Define $C000 as the new Direct Page Address
*--
                    SEP        #$20
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F
                    ORA        #$60                 ; Access Sound Ram + Auto Increment Enabled
                    STAL       SOUNDCTL
                    LDA        #$F8                 ; Sound DOC Ram Address = $F800
                    STAL       SOUNDADRH
                    LDA        #$00            
                    STAL       SOUNDADRL
                    XBA                             ; Make sure B is #$00 for the TAY
*--           
                    LDX        #$0000
CSKBDTR_1           LDAL       $BBAAAA,X            ; Copy Raw Sound Data
                    BNE        CSKBDTR_2
                    LDA        #$01                 ; Change #00 into #01  
CSKBDTR_2           STA        $3D                  ; SOUNDDATA = $C03D
                    INX
                    CPX        #$04F0               ; Sound Size
                    BNE        CSKBDTR_1
*--
                    LDA        #$00                 ; Add few #$00 to stop the Sound
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
*--
                    REP        #$30
                    PLD                             ; Restore Direct Page Address
                    CLI                             ; Enable Interrupts
                    PLB
                    RTL  


******************************************************************************************************
*  PlaySoundKBD() :  Start ENSONIQ Oscillators 4 & 5 to play Keyboard Sound Sample located at $F800. *
******************************************************************************************************
PlaySoundKBD        ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    SEP        #$20
*------
                    LDA        #$A4                 ; Control Register for Oscillator #04                 
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Stop Oscillators #04 : Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
                    LDA        #$A5                 ; Control Register for Oscillator #05                
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Stop Oscillators #05 : Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
*------
                    LDA        #$04                 ; Frequency Low Register for Oscillator #04
                    STAL       SOUNDADRL
                    LDA        #$9B                 ; Frequency Low value (Hz*32/1645), 8000 hz => 9B
                    STAL       SOUNDDATA
                    LDA        #$24                 ; Frequency High Register for Oscillator #04
                    STAL       SOUNDADRL
                    LDA        #$00                 ; Frequency High value (Hz*32/1645)
                    STAL       SOUNDDATA
*--
                    LDA        #$05                 ; Frequency Low Register for Oscillator #05
                    STAL       SOUNDADRL
                    LDA        #$9B                 ; Frequency Low value (Hz*32/1645), 8000 hz => 9B
                    STAL       SOUNDDATA
                    LDA        #$25                 ; Frequency High Register for Oscillator #05
                    STAL       SOUNDADRL
                    LDA        #$00                 ; Frequency High value (Hz*32/1645)
                    STAL       SOUNDDATA
*------
                    LDA        #$44                 ; Sound Volume Register for Oscillator #04
                    STAL       SOUNDADRL
                    JSL        GetMuteStatus        ; Volume : 00 (Min) / FF (Max)
                    STAL       SOUNDDATA
*--
                    LDA        #$45                 ; Sound Volume Register for Oscillator #05
                    STAL       SOUNDADRL
                    JSL        GetMuteStatus        ; Volume : 00 (Min) / FF (Max)
                    STAL       SOUNDDATA
*------
                    LDA        #$84                 ; Sound Address Register for Oscillator #04
                    STAL       SOUNDADRL
                    LDA        #$F8                 ; Sound Address = $F800 => $F8
                    STAL       SOUNDDATA
*--
                    LDA        #$85                 ; Sound Address Register for Oscillator #05
                    STAL       SOUNDADRL
                    LDA        #$F8                 ; Sound Address = $F800 => $F8
                    STAL       SOUNDDATA
*------
                    LDA        #$C4                 ; Oscillator #4
                    STAL       SOUNDADRL
                    LDA        #$1B                 ; Sound Size = 2 KB
                    STAL       SOUNDDATA
                    LDA        #$C5                 ; Oscillator #5
                    STAL       SOUNDADRL
                    LDA        #$1B                 ; Sound Size = 2 KB
                    STAL       SOUNDDATA
*------
                    LDA        #$A4                 ; Control Register for Oscillator #04
                    STAL       SOUNDADRL
                    LDA        #$02                 ; Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Start Sound=0
                    STAL       SOUNDDATA
*--
                    LDA        #$A5                 ; Control Register for Oscillator #05
                    STAL       SOUNDADRL
                    LDA        #$12                 ; Canal #1=0001 / No Interrupt=0 / One-Shot=01 / Start Sound=0
                    STAL       SOUNDDATA
*------
                    REP        #$30
                    PLB
                    RTL  


***********************************************************************************
*  WaitEndOfSound() :  Wait until the Sound stop playing from Oscillators 0 -> 5. *
***********************************************************************************
WaitEndOfSound      ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*-- 
                    SEP        #$20
*--
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F                 ; Access Sound Register + Auto Increment Disabled           
                    STAL       SOUNDCTL                    
*--                                     
                    LDA        #$A0                 ; Control Register for Oscillator #00
                    STAL       SOUNDADRL
                    LDX        #$0000
WEOS_OSC0           LDAL       SOUNDDATA            ; Oscillator #0 information
                    LSR
                    BCC        WEOS_OSC0            ; Bit #0 = 0 => Still Active
                    INX
                    CPX        #$8000
                    BNE        WEOS_OSC0            ; Wait for #$8000 non active sequences
*--
                    LDA        #$A1                 ; Control Register for Oscillator #01
                    STAL       SOUNDADRL
                    LDX        #$0000
WEOS_OSC_1          LDAL       SOUNDDATA            ; Oscillator #1 information
                    LSR
                    BCC        WEOS_OSC_1           ; Bit #0 = 0 => Still Active
                    INX
                    CPX        #$8000
                    BNE        WEOS_OSC_1           ; Wait for #$8000 non active sequences
*--
                    LDA        #$A2                 ; Control Register for Oscillator #02
                    STAL       SOUNDADRL
                    LDX        #$0000
WEOS_OSC_2          LDAL       SOUNDDATA            ; Oscillator #2 information
                    LSR
                    BCC        WEOS_OSC_2           ; Bit #0 = 0 => Still Active
                    INX
                    CPX        #$8000
                    BNE        WEOS_OSC_2           ; Wait for #$8000 non active sequences
*--
                    LDA        #$A3                 ; Control Register for Oscillator #03
                    STAL       SOUNDADRL
                    LDX        #$0000
WEOS_OSC_3          LDAL       SOUNDDATA            ; Oscillator #3 information
                    LSR
                    BCC        WEOS_OSC_3           ; Bit #0 = 0 => Still Active
                    INX
                    CPX        #$8000
                    BNE        WEOS_OSC_3           ; Wait for #$8000 non active sequences
*--
                    LDA        #$A4                 ; Control Register for Oscillator #04
                    STAL       SOUNDADRL
                    LDX        #$0000
WEOS_OSC_4          LDAL       SOUNDDATA            ; Oscillator #4 information
                    LSR
                    BCC        WEOS_OSC_4           ; Bit #0 = 0 => Still Active
                    INX
                    CPX        #$8000
                    BNE        WEOS_OSC_4           ; Wait for #$8000 non active sequences
*--
                    LDA        #$A5                 ; Control Register for Oscillator #05
                    STAL       SOUNDADRL
                    LDX        #$0000
WEOS_OSC_5          LDAL       SOUNDDATA            ; Oscillator #5 information
                    LSR
                    BCC        WEOS_OSC_5           ; Bit #0 = 0 => Still Active
                    INX
                    CPX        #$8000
                    BNE        WEOS_OSC_5           ; Wait for #$8000 non active sequences
*--
                    REP        #$30
                    PLB
                    RTL  

**********************************************************************
*  EnableDisableAudio() :  Enable / Disable Music and Sound effects. *
**********************************************************************
EnableDisableAudio  ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*-- 
                    SEP        #$30
                    SEI                             ; Stop Interruption
*--
                    LDA        EDA_STATUS
                    BEQ        EDA_DisableAudio
*--------
EDA_EnableAudio     STZ        EDA_STATUS           ; Enable Audio
*--
EDA_EA_Ready        LDAL       SOUNDCTL
                    BMI        EDA_EA_Ready
*--
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound volume
                    AND        #$0F
                    STAL       SOUNDCTL
*--
                    LDX        #$00
EDA_Restaure        TXA
                    CLC
                    ADC        #$A0
                    STAL       SOUNDADRL
                    LDAL       EDA_SoundRegister,X  ; Restaure Sound Registers
                    STAL       SOUNDDATA
                    INX
                    CPX        #$20
                    BNE        EDA_Restaure
*--
                    BRA        EDA_End
*--------
EDA_DisableAudio    INC        EDA_STATUS           ; Disable Audio
*--
EDA_DA_Ready        LDAL       SOUNDCTL
                    BMI        EDA_DA_Ready
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound volume
                    AND        #$0F
                    STAL       SOUNDCTL
*----
                    LDX        #$00
EDA_DA_Backup       TXA
                    CLC
                    ADC        #$A0
                    STAL       SOUNDADRL
                    LDAL       SOUNDDATA
                    LDAL       SOUNDDATA
                    STAL       EDA_SoundRegister,X  ; Backup Sound Registers
                    TXA
                    CLC
                    ADC        #$A1
                    STAL       SOUNDADRL
                    LDAL       SOUNDDATA
                    LDAL       SOUNDDATA
                    STAL       EDA_SoundRegister+1,X
*--
                    LDY        #$02                 ; x2 for the Loop Songs
EDA_DA_Stop         TXA
                    CLC
                    ADC        #$A0
                    STAL       SOUNDADRL
                    LDA        #%0000_0001          ; Stop Oscillator
                    STAL       SOUNDDATA
                    TXA
                    CLC
                    ADC        #$A1
                    STAL       SOUNDADRL
                    LDAL       #%0000_0001
                    STAL       SOUNDDATA
                    DEY
                    BNE        EDA_DA_Stop
*--
                    INX
                    INX
                    CPX        #$20
                    BNE        EDA_DA_Backup
*----
EDA_End             CLI                             ; Enable Interruption
                    REP        #$30
                    PLB
                    RTL

*----------   EnableDisableAudio Variables   ---------
EDA_STATUS          HEX        00                   ; Enable (00) / Disable (01)
EDA_SoundRegister   ds         32                   ; Sound Register values
         
         
******************************************************         
*  MuteUnmuteAudio() :  Mute / Unmute Sound effects. *          
******************************************************
MuteUnmuteAudio     ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*-- 
                    SEP        #$30                 ; Inverse Sound Status
                    LDA        MUA_STATUS
                    BEQ        MUA_Unmute
*--
MUA_Mute            INC        MUA_STATUS           ; Set Sound Level to Min : FF -> 00
                    BRA        MUA_Begin
MUA_Unmute          DEC        MUA_STATUS           ; Set Sound Level to Max : 00 -> FF
*--
MUA_Begin           LDX        #$40      
MUA_Loop            TXA
                    STAL       SOUNDADRL
                    LDA        MUA_STATUS           ; 00 or FF
                    STAL       SOUNDDATA
                    INX
                    CPX        #$60
                    BNE        MUA_Loop
*--
                    REP        #$30 
                    PLB
                    RTL             


***************************************************************         
*  GetMuteStatus() :  Get current Mute status (OUT:A=Status). *          
***************************************************************
GetMuteStatus       ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*-- 
                    LDA        MUA_STATUS           ; Work in 8 and 16 bit
*--
                    PLB
                    RTL

*----------   MuteUnmuteAudio / GetMuteStatus Variables   ---------
MUA_STATUS          HEX        FF,00                ; Unmute (FF) / Mute (00)


*************************************************
*  StopSound() :  Stop the first 4 oscillators. *
*************************************************
StopSound           ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*-- 
                    SEP        #$20
*----
                    LDY        #$0002
StopSound_1         LDA        #$A0                 ; Control Register for Oscillator #00
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
*--
                    LDA        #$A1                 ; Control Register for Oscillator #01                 
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
*--
                    LDA        #$A2                 ; Control Register for Oscillator #02                 
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
*--
                    LDA        #$A3                 ; Control Register for Oscillator #03                 
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
                    DEY
                    BPL        StopSound_1
*----
                    REP        #$30
                    PLB
                    RTL

                    ASC        0D,'We know that the game is pretty short to be completed (let',27,'s say 4 minutes...) but hopefully, more is coming... Stay Tunes',0D

****************************************************************************************
