****************************************************************************************
*                                                                                      *
*  DragonsLair.Bitmap.s :  Library for Bitmap Handling (Numbers + Character Set).      *
*                                                                                      *
****************************************************************************************
*  Authors : Antoine VIGNAU / Olivier ZARDINI  *  Brutal Deluxe Software  *  Jan 2022  *
****************************************************************************************

*----------   Merlin 16+ Directives   ---------

                    mx         %00                  ; Assemble in 16 bit

                    rel                             ; Build a relocated S16 file
                    dsk        Bitmap.l             ; Name of the Segment

                    ASC        0D,'Just in case you ask yourself which tools were involved in the creation of this game : Merlin32, Cadius, WindowCapture, Studio and a customized version of KEGS !',0D

*********************************************************************************************
*  EntryPointBitmap() :  Entry Point of Segment 'Bitmap' (IN:A=MainBankXX,IN:X=Buffer32KB). *
*********************************************************************************************
EntryPointBitmap    ENT                             ; First Byte of this Segment
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    STA        EPB_MainBank         ; A = 8 bit
                    STX        EPB_Buffer32KB
*--
                    PLB
                    RTL

*----------   EntryPointBitmap Variables   ---------                    
EPB_MainBank        HEX        0000                 ; XX00   
EPB_Buffer32KB      HEX        0000                 ; YYYY                    

                    
******************************************************************************************
*  CopyOneBitmap() :  Copy one Bitmap to Screen (IN:A=Width4|Heigh,IN:X=@Src,IN:Y=@Dst). *
******************************************************************************************
CopyOneBitmap       ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    STX        COB_Data+1           ; X = Src @
                    STY        COB_Screen+1         ; Y = Dst @
                    TAX                             ; A = Width4/Heigh
                    AND        #$00FF               ; Heigh
                    STA        COB_Heigh+1
                    TXA
                    XBA
                    AND        #$00FF               ; Width4
                    STA        COB_Width+1
                    LDA        #$00A0
                    SEC
                    SBC        COB_Width+1
                    SEC
                    SBC        COB_Width+1
                    STA        COB_Offset+1
*--                                                    
                    LDY        #$0000               ; Copy Bitmap Data
                    LDX        #$0000
COB_Data            LDA        $AAAA                ; Number Pixel
COB_Screen          STAL       $012000              ; Screen Address
                    INC        COB_Data+1
                    INC        COB_Data+1
                    INC        COB_Screen+1
                    INC        COB_Screen+1                    
                    INX
COB_Width           CPX        #$AAAA
                    BNE        COB_Data
                    LDA        COB_Screen+1
                    CLC
COB_Offset          ADC        #$00A0               ; #$A0 - Width
                    STA        COB_Screen+1
                    LDX        #$0000
                    INY
COB_Heigh           CPY        #$AAAA
                    BNE        COB_Data
*--
                    PLB
                    RTL


***************************************************************************************************************************
*  CopyTransBitmap() :  Copy one Transparent Bitmap to Screen (IN:A=Width4|Heigh,IN:X=@Src,IN:Y=@Dst,IN:Carry=0/1:01:E1). *
***************************************************************************************************************************
CopyTransBitmap     ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    PHA
                    BCC        CTB_01               ; Target Screen is 01/ or E1/
CTB_E1              LDA        #$E120
                    BRA        CTB_Screen
CTB_01              LDA        #$0120
CTB_Screen          STA        CTB_Screen_1+2
                    STA        CTB_Screen_2+2
                    PLA
*--
                    STX        CTB_Data+1           ; X = Src @
                    STY        CTB_Screen_1+1       ; Y = Dst @
                    STY        CTB_Screen_2+1
                    TAX                             ; A = Width4/Heigh
                    AND        #$00FF               ; Heigh
                    STA        CTB_Heigh+1
                    TXA
                    XBA
                    AND        #$00FF               ; Width4
                    STA        CTB_Width+1
                    LDA        #$00A0
                    SEC
                    SBC        CTB_Width+1
                    SEC
                    SBC        CTB_Width+1
                    STA        CTB_Offset+1
*--                                                    
                    LDY        #$0000               ; Copy Transparent Bitmap
                    LDX        #$0000
CTB_Data            LDA        $AAAA                ; Bitmap Data
                    STA        CTB_DataBitmap
*--
CTB_Mask_1          LDA        #$FFFF               ; Prepare Mask
                    STA        CTB_MaskBitmap
                    LDA        CTB_DataBitmap
                    AND        #$000F
                    BEQ        CTB_Mask_2
                    LDA        CTB_MaskBitmap
                    AND        #$FFF0
                    STA        CTB_MaskBitmap
CTB_Mask_2          LDA        CTB_DataBitmap
                    AND        #$00F0
                    BEQ        CTB_Mask_3
                    LDA        CTB_MaskBitmap
                    AND        #$FF0F
                    STA        CTB_MaskBitmap
CTB_Mask_3          LDA        CTB_DataBitmap
                    AND        #$0F00
                    BEQ        CTB_Mask_4
                    LDA        CTB_MaskBitmap
                    AND        #$F0FF
                    STA        CTB_MaskBitmap
CTB_Mask_4          LDA        CTB_DataBitmap
                    AND        #$F000
                    BEQ        CTB_Screen_1
                    LDA        CTB_MaskBitmap
                    AND        #$0FFF
                    STA        CTB_MaskBitmap                                                              
*--
CTB_Screen_1        LDAL       $012000              ; Background Data
                    AND        CTB_MaskBitmap       ; AND Mask
                    ORA        CTB_DataBitmap       ; ORA Data             
CTB_Screen_2        STAL       $012000
*--
                    INC        CTB_Data+1
                    INC        CTB_Data+1
                    INC        CTB_Screen_1+1
                    INC        CTB_Screen_1+1
                    INC        CTB_Screen_2+1
                    INC        CTB_Screen_2+1
                    INX
CTB_Width           CPX        #$AAAA
                    BNE        CTB_Data
                    LDA        CTB_Screen_1+1
                    CLC
CTB_Offset          ADC        #$00A0               ; #$A0 - Width
                    STA        CTB_Screen_1+1
                    STA        CTB_Screen_2+1
                    LDX        #$0000
                    INY
CTB_Heigh           CPY        #$AAAA
                    BEQ        CTB_End
                    JMP        CTB_Data
*--
CTB_End             PLB
                    RTL

*----------   CopyTransBitmap Variables   ---------
CTB_MaskBitmap      HEX        0000                 ; Mask 0-F
CTB_DataBitmap      HEX        0000                 ; Data 


*****************************************************************************************************************************
*  CopyTransBitmapFF() :  Copy one Transparent Bitmap to Screen (IN:A=Width4|Heigh,IN:X=@Src,IN:Y=@Dst,IN:Carry=0/1:01:E1). *
*****************************************************************************************************************************
CopyTransBitmapFF   ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
                    PHA
*--
                    BCC        CTBFF_01             ; Target Screen is 01/ or E1/
CTBFF_E1            LDA        #$E120
                    BRA        CTBFF_Screen
CTBFF_01            LDA        #$0120
CTBFF_Screen        STA        CTBFF_Screen_2+2
*--
                    LDA        EPB_MainBank         ; Background Screen is Buffer32KB
                    XBA
                    STA        CTBFF_Screen_1+2 
*--
                    STX        CTBFF_Data+1         ; X = Src @
                    STY        CTBFF_Screen_2+1     ; Y = Dst @
                    TYA                             ; Background Address in Buffer32KB
                    SEC
                    SBC        #$2000
                    CLC
                    ADC        EPB_Buffer32KB
                    STA        CTBFF_Screen_1+1     
*--            
                    PLA
                    TAX                             ; A = Width4/Heigh
                    AND        #$00FF               ; Heigh
                    STA        CTBFF_Heigh+1
                    TXA
                    XBA
                    AND        #$00FF               ; Width4
                    STA        CTBFF_Width+1
                    LDA        #$00A0
                    SEC
                    SBC        CTBFF_Width+1
                    SEC
                    SBC        CTBFF_Width+1
                    STA        CTBFF_Offset+1
*--                                                    
                    LDY        #$0000               ; Copy Transparent Bitmap
                    LDX        #$0000
CTBFF_Data          LDA        $AAAA                ; Bitmap Data
                    CMP        #$FFFF
                    BNE        CTBFF_Data_1
                    STA        CTBFF_MaskBitmap     ; AND Mask FFFF
                    STZ        CTBFF_DataBitmap     ; ORA Data 0000 
                    JMP        CTBFF_Screen_1       ; Copy Background Data to Screen             
CTBFF_Data_1        STA        CTBFF_DataBitmap
*----
CTBFF_Mask_1        LDA        #$FFFF               ; Prepare Mask
                    STA        CTBFF_MaskBitmap
*--
                    LDA        CTBFF_DataBitmap
                    AND        #$000F
                    CMP        #$000F
                    BEQ        CTBFF_Mask_1F
                    LDA        CTBFF_MaskBitmap
                    AND        #$FFF0
                    STA        CTBFF_MaskBitmap
                    BRA        CTBFF_Mask_2
CTBFF_Mask_1F       LDA        CTBFF_DataBitmap
                    AND        #$FFF0
                    STA        CTBFF_DataBitmap
*--
CTBFF_Mask_2        LDA        CTBFF_DataBitmap
                    AND        #$00F0
                    CMP        #$00F0
                    BEQ        CTBFF_Mask_2F
                    LDA        CTBFF_MaskBitmap
                    AND        #$FF0F
                    STA        CTBFF_MaskBitmap
                    BRA        CTBFF_Mask_3
CTBFF_Mask_2F       LDA        CTBFF_DataBitmap
                    AND        #$FF0F
                    STA        CTBFF_DataBitmap
*--
CTBFF_Mask_3        LDA        CTBFF_DataBitmap
                    AND        #$0F00
                    CMP        #$0F00
                    BEQ        CTBFF_Mask_3F
                    LDA        CTBFF_MaskBitmap
                    AND        #$F0FF
                    STA        CTBFF_MaskBitmap
                    BRA        CTBFF_Mask_4
CTBFF_Mask_3F       LDA        CTBFF_DataBitmap
                    AND        #$F0FF
                    STA        CTBFF_DataBitmap
*--
CTBFF_Mask_4        LDA        CTBFF_DataBitmap
                    AND        #$F000
                    CMP        #$F000
                    BEQ        CTBFF_Mask_4F
                    LDA        CTBFF_MaskBitmap
                    AND        #$0FFF
                    STA        CTBFF_MaskBitmap
                    BRA        CTBFF_Screen_1
CTBFF_Mask_4F       LDA        CTBFF_DataBitmap
                    AND        #$0FFF
                    STA        CTBFF_DataBitmap
*----
CTBFF_Screen_1      LDAL       $AABBBB              ; Background is Buffer32KB
                    AND        CTBFF_MaskBitmap     ; AND Mask
                    ORA        CTBFF_DataBitmap     ; ORA Data             
CTBFF_Screen_2      STAL       $012000              ; Screen is 01/2000 or E1/2000
*--
CTBFF_Next          INC        CTBFF_Data+1
                    INC        CTBFF_Data+1
                    INC        CTBFF_Screen_1+1
                    INC        CTBFF_Screen_1+1
                    INC        CTBFF_Screen_2+1
                    INC        CTBFF_Screen_2+1
                    INX
CTBFF_Width         CPX        #$AAAA
                    BEQ        CTBFF_Width_1
                    JMP        CTBFF_Data
CTBFF_Width_1       LDA        CTBFF_Screen_1+1
                    CLC
CTBFF_Offset        ADC        #$00A0               ; #$A0 - Width
                    STA        CTBFF_Screen_1+1
                    LDA        CTBFF_Screen_2+1
                    CLC
                    ADC        CTBFF_Offset+1       ; #$A0 - Width
                    STA        CTBFF_Screen_2+1
                    LDX        #$0000
                    INY
CTBFF_Heigh         CPY        #$AAAA
                    BEQ        CTBFF_End
                    JMP        CTBFF_Data
*--
CTBFF_End           PLB
                    RTL

*----------   CopyTransBitmapFF Variables   ---------
CTBFF_MaskBitmap    HEX        0000                 ; Mask 0-F
CTBFF_DataBitmap    HEX        0000                 ; Data 


********************************************************************************************
*  RemapBitmapColor() :  Remap Bitmap Color (IN:A=ColorSrcColorDst,IN:X=@Src,IN:Y=Length). *
********************************************************************************************
RemapBitmapColor    ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    STX        RBC_Data_1+1         ; @Data
                    STX        RBC_Data_4+1
                    STY        RBC_Length+1         ; Data Length
                    SEP        #$20
                    PHA                             ; Color Dst
                    AND        #$0F
                    STA        RBC_ColorDst1
                    PLA 
                    AND        #$F0
                    STA        RBC_ColorDst2
                    XBA
                    PHA                             ; Color Src
                    AND        #$0F
                    STA        RBC_ColorSrc1
                    PLA 
                    AND        #$F0
                    STA        RBC_ColorSrc2
*--                    
                    LDX        #$0000
RBC_Data_1          LDA        $AAAA,X
                    STA        RBC_Data
                    AND        #$0F                 ; Color 0c
                    CMP        RBC_ColorSrc1
                    BNE        RBC_Data_2
                    LDA        RBC_Data
                    AND        #$F0
                    ORA        RBC_ColorDst1
                    STA        RBC_Data
*--                   
RBC_Data_2          LDA        RBC_Data             ; Color c0
                    AND        #$F0
                    CMP        RBC_ColorSrc2
                    BNE        RBC_Data_3
                    LDA        RBC_Data
                    AND        #$0F
                    ORA        RBC_ColorDst2
                    STA        RBC_Data
*--
RBC_Data_3          LDA        RBC_Data
RBC_Data_4          STA        $AAAA,X
                    INX
RBC_Length          CPX        #$AAAA
                    BNE        RBC_Data_1                    
*--
                    REP        #$30
                    PLB
                    RTL

*----------   RemapBitmapColor Variables   ---------
RBC_Data            HEX        00
RBC_ColorSrc1       HEX        00                   ; 0s
RBC_ColorSrc2       HEX        00                   ; s0
RBC_ColorDst1       HEX        00                   ; 0d
RBC_ColorDst2       HEX        00                   ; d0


************************************************************************************
*  DisplayHintBitmap() :  Remap & Display Hint Bitmap (IN:A=SceneID,IN:X=@Bitmap). *
************************************************************************************
DisplayHintBitmap   ENT
                    PHB
                    PHK
                    PLB
*--
                    LDY       #$220D                ; @ Screen
                    JSR       DisplayRemapBitmap
*--
                    PLB
                    RTL


************************************************************************************
*  DisplayDiskBitmap() :  Remap & Display Disk Bitmap (IN:A=SceneID,IN:X=@Bitmap). *
************************************************************************************
DisplayDiskBitmap   ENT
                    PHB
                    PHK
                    PLB
*--
                    LDY       #$01E1                ; @ Screen
                    JSR       DisplayRemapBitmap
*--
                    PLB
                    RTL


****************************************************************************************
*  DisplayFrameBitmap() :  Display Frame ID Bitmap on screen (IN:A=FrameID,X=SceneID). *
****************************************************************************************
DisplayFrameBitmap  ENT
                    PHB
                    PHK
                    PLB
                    STA       DFB_FrameID
*----
                    LDA       DFB_FrameID           ; Bitmap #0
                    AND       #$00F0
                    LSR
                    LSR
                    LSR
                    LSR
                    ASL
                    TAX
                    LDA       DFB_Bitmap_Tab,X
                    STA       DFB_Draw_Bitmap_0+1
*--
                    LDA       DFB_FrameID           ; Bitmap #1 
                    AND       #$000F
                    ASL
                    TAX
                    LDA       DFB_Bitmap_Tab,X
                    STA       DFB_Draw_Bitmap_1+1
*----
                    LDY       #$000B
                    LDX       #$0000
DFB_Draw_Erase      LDA       #$0000
                    STAL      $012000,X
                    STAL      $012002,X
                    STAL      $012004,X
                    STAL      $012006,X
                    STAL      $012008,X
                    STAL      $01200A,X     
                    TXA
                    CLC
                    ADC       #$00A0
                    TAX
                    DEY
                    BPL       DFB_Draw_Erase
*----
                    LDA       #$0001
DFB_Draw_Bitmap_0   LDX       #$AAAA
                    LDY       #$0000
                    JSR       DisplayRemapBitmap
*--
                    LDA       #$0001
DFB_Draw_Bitmap_1   LDX       #$AAAA
                    LDY       #$0006
                    JSR       DisplayRemapBitmap
*----
                    PLB
                    RTL
*----------   DisplayFrameBitmap Variables   ---------
DFB_Bitmap_Tab      DA         Aplha_0,Aplha_1,Aplha_2,Aplha_3,Aplha_4,Aplha_5,Aplha_6,Aplha_7
                    DA         Aplha_8,Aplha_9,Aplha_A,Aplha_B,Aplha_C,Aplha_D,Aplha_E,Aplha_F
DFB_FrameID         HEX        0000


*******************************************************************************************************
*  DisplayRemapBitmap() :  Remap & Display Bitmap on Screen (IN:A=SceneID,IN:X=@Bitmap,IN:Y=@Screen). *
*******************************************************************************************************
DisplayRemapBitmap  STY        DRB_ScreenAddress+1  ; @ Screen
                    STA        DRB_SceneID+1        ; Scene ID
                    STX        DRB_ColorRemap+1     ; @ Bitmap
                    TXA                             ; @ Mask
                    CLC
                    ADC        #$0048
                    STA        DRB_Mask_0+1
                    INC
                    INC
                    STA        DRB_Mask_1+1
                    INC
                    INC
                    STA        DRB_Mask_2+1
*----
                    LDX        #$0000               ; Remap Color Scheme
DRB_ColorScheme     LDA        ColorSceneTab,X
DRB_SceneID         CMP        #$AAAA               ; Scene ID
                    BEQ        DRB_ColorSet
                    INX
                    INX
                    BRA        DRB_ColorScheme
DRB_ColorSet        LDA        ColorBWTab,X         ; Black White
*--
                    SEP        #$20
                    PHA                             ; New White
                    AND        #$F0
                    STA        DRB_RemapWhite_1+1
                    PLA
                    AND        #$0F
                    STA        DRB_RemapWhite_2+1
                    XBA
                    PHA                             ; New Black
                    AND        #$F0
                    STA        DRB_RemapBlack_1+1
                    PLA
                    AND        #$0F
                    STA        DRB_RemapBlack_2+1
*--                                        
                    LDX        #$0047
DRB_ColorRemap      LDA        $AAAA,X              ; Bitmap
                    STA        DRB_ColorRemap_2+1
*--                    
DRB_ColorRemap_1    AND        #$F0                 ; Remap Color #1
                    CMP        #$10                 ; 1 = White
                    BEQ        DRB_RemapWhite_1
                    CMP        #$20                 ; 2 = Black
                    BEQ        DRB_RemapBlack_1
                    BRA        DRB_ColorRemap_10
DRB_RemapWhite_1    LDA        #$C0                 ; New White
                    BRA        DRB_ColorRemap_10
DRB_RemapBlack_1    LDA        #$C0                 ; New Black
DRB_ColorRemap_10   STA        DRB_Color_1+1
*--
DRB_ColorRemap_2    LDA        #$AA                 ; Remap Color #2
                    AND        #$0F
                    CMP        #$01                 ; 1 = White
                    BEQ        DRB_RemapWhite_2
                    CMP        #$02                 ; 2 = Black
                    BEQ        DRB_RemapBlack_2
                    BRA        DRB_ColorRemap_20
DRB_RemapWhite_2    LDA        #$0C                 ; New White
                    BRA        DRB_ColorRemap_20
DRB_RemapBlack_2    LDA        #$0C                 ; New Black
DRB_ColorRemap_20   STA        DRB_Color_2+1    
*--
DRB_Color_1         LDA        #$C0
DRB_Color_2         ORA        #$0C
                    STA        DRB_Buffer,X         ; Remap Buffer
                    DEX
                    BPL        DRB_ColorRemap   
                    REP        #$30
*----
DRB_ScreenAddress   LDX        #$AAAA               ; Screen Address
                    LDY        #$0000               ; 12 lines * 6 pixels
*--
DRB_Display         LDAL       $E12000,X            ; Background
DRB_Mask_0          AND        Aplha_Event_Mask+0,Y ; Mask
                    ORA        DRB_Buffer+0,Y       ; Data
                    STAL       $E12000,X            ; Background
                    LDAL       $E12002,X            ; Background
DRB_Mask_1          AND        Aplha_Event_Mask+2,Y ; Mask
                    ORA        DRB_Buffer+2,Y       ; Data
                    STAL       $E12002,X            ; Background
                    LDAL       $E12004,X            ; Background
DRB_Mask_2          AND        Aplha_Event_Mask+4,Y ; Mask
                    ORA        DRB_Buffer+4,Y       ; Data
                    STAL       $E12004,X            ; Background
*--
                    TXA                             ; Next Screen Line
                    CLC
                    ADC        #$00A0
                    TAX
                    TYA                             ; Next Bitmap Line
                    CLC
                    ADC        #$0006
                    TAY
                    CPY        #$0048
                    BNE        DRB_Display
*----
                    RTS

*----------   DisplayRemapBitmap Variables   ---------
DRB_Buffer          HEX        00,00,00,00,00,00    ; Buffer for Color Remap
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00


************************************
*  Disk Screen 1-8 Numbers Pixels  *
************************************

DiskTable           ENT
                    DA         Disk1,Disk2,Disk3,Disk4,Disk5,Disk6,Disk7,Disk8
                                  
Disk1               HEX        20,0B,D972           ; 1 : Height, Width4, Screen @
                    HEX        55,55,55,55,55,55,55,55,66,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,56,66,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,66,66,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,56,66,66,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,66,66,66,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,56,66,66,66,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,66,66,66,65,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,56,66,66,66,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,66,66,66,65,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,56,66,66,66,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,66,66,66,65,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,56,66,66,66,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55
                    HEX        55,55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55
                    HEX        55,55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55
                    HEX        55,55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55

Disk2               HEX        20,0B,D972           ; 2 : Height, Width4, Screen @
                    HEX        55,55,55,55,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55,55,55,55
                    HEX        55,55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55,55
                    HEX        55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55
                    HEX        55,66,66,66,66,65,55,55,55,55,55,55,55,55,55,66,66,66,66,65,55,55
                    HEX        56,66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,66,66,66,66,55,55
                    HEX        56,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,65,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,65,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,65,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,66,55,55
                    HEX        55,55,55,55,55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55
                    HEX        55,55,55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55
                    HEX        55,55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55,55
                    HEX        55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55,55,55
                    HEX        55,66,66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        56,66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        56,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55
                    HEX        66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55
                    HEX        66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55
                    HEX        66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55

Disk3               HEX        20,0B,D972           ; 3 : Height, Width4, Screen @
                    HEX        55,55,55,55,56,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55,55,55
                    HEX        55,55,55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55,55
                    HEX        55,55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55
                    HEX        55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55
                    HEX        55,66,66,66,66,66,55,55,55,55,55,55,55,55,55,56,66,66,66,66,65,55
                    HEX        55,66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        56,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55
                    HEX        56,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55
                    HEX        56,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55
                    HEX        56,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,66,65,55
                    HEX        55,55,55,55,55,55,55,55,55,56,66,66,66,66,66,66,66,66,66,66,55,55
                    HEX        55,55,55,55,55,55,55,55,55,56,66,66,66,66,66,66,66,66,66,65,55,55
                    HEX        55,55,55,55,55,55,55,55,55,56,66,66,66,66,66,66,66,66,66,65,55,55
                    HEX        55,55,55,55,55,55,55,55,55,56,66,66,66,66,66,66,66,66,66,66,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,66,65,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55
                    HEX        56,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55
                    HEX        56,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55
                    HEX        56,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55
                    HEX        56,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55
                    HEX        55,66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        55,66,66,66,66,66,55,55,55,55,55,55,55,55,55,56,66,66,66,66,65,55
                    HEX        55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55
                    HEX        55,55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55
                    HEX        55,55,55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55,55
                    HEX        55,55,55,55,56,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55,55,55

Disk4               HEX        20,0B,D972           ; 4 : Height, Width4, Screen @
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,66,66,66,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,66,66,66,66,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,56,66,66,66,66,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,56,66,66,66,66,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,66,66,66,66,65,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,56,66,66,66,65,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,56,66,66,66,66,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,66,66,66,66,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,66,66,66,66,65,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,56,66,66,66,65,55,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,56,66,66,66,66,55,55,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,66,66,66,66,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,66,66,66,66,65,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,56,66,66,66,65,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55
                    HEX        66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55
                    HEX        66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55
                    HEX        66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55,55,55,55,55

Disk5               HEX        20,0B,D972           ; 5 : Height, Width4, Screen @
                    HEX        56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55
                    HEX        56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55
                    HEX        56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55
                    HEX        56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55
                    HEX        56,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        56,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        56,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        56,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        56,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        56,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        56,66,66,65,56,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55,55,55
                    HEX        56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55,55
                    HEX        56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55
                    HEX        56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55
                    HEX        56,66,66,66,66,65,55,55,55,55,55,55,55,55,55,56,66,66,66,66,65,55
                    HEX        56,66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        56,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55
                    HEX        56,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55
                    HEX        56,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55
                    HEX        56,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,55
                    HEX        56,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55
                    HEX        56,66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55
                    HEX        55,66,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        55,66,66,66,66,66,55,55,55,55,55,55,55,55,55,56,66,66,66,66,65,55
                    HEX        55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55
                    HEX        55,55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55
                    HEX        55,55,55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55,55
                    HEX        55,55,55,55,56,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55,55,55


Disk6               HEX        20,0B,D972           ; 6 : Height, Width4, Screen @
                    HEX        55,55,55,55,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55,55,55,55
                    HEX        55,55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55,55
                    HEX        55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55
                    HEX        55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55
                    HEX        56,66,66,66,66,65,55,55,55,55,55,55,55,55,55,66,66,66,66,66,55,55
                    HEX        56,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,65,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        66,66,66,55,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55,55,55,55
                    HEX        66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55,55
                    HEX        66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55
                    HEX        66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55
                    HEX        66,66,66,66,66,65,55,55,55,55,55,55,55,55,55,66,66,66,66,66,55,55
                    HEX        66,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,65,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,65,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,65,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,65,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        56,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55
                    HEX        56,66,66,66,66,65,55,55,55,55,55,55,55,55,55,66,66,66,66,66,55,55
                    HEX        55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55
                    HEX        55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55
                    HEX        55,55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55,55
                    HEX        55,55,55,55,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55,55,55,55

Disk7               HEX        20,0B,D972           ; 7 : Height, Width4, Screen @
                    HEX        56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55
                    HEX        56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55
                    HEX        56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55
                    HEX        56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,66,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,65,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,65,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,66,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,66,66,66,65,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,55,66,66,66,65,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,56,66,66,66,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,55,66,66,66,65,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,56,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,56,66,66,66,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,55,66,66,66,65,55,55,55,55,55,55,55,55,55,55
                    HEX        55,55,55,55,55,55,55,56,66,66,66,55,55,55,55,55,55,55,55,55,55,55

Disk8               HEX        20,0B,D972           ; 8 : Height, Width4, Screen @
                    HEX        55,55,55,55,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55,55,55,55
                    HEX        55,55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55,55
                    HEX        55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55
                    HEX        55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55
                    HEX        56,66,66,66,66,65,55,55,55,55,55,55,55,55,55,66,66,66,66,66,55,55
                    HEX        56,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,65,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,65,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        56,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55
                    HEX        56,66,66,66,66,65,55,55,55,55,55,55,55,55,55,66,66,66,66,66,55,55
                    HEX        55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55
                    HEX        55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55
                    HEX        55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55
                    HEX        55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55
                    HEX        56,66,66,66,66,65,55,55,55,55,55,55,55,55,55,66,66,66,66,66,55,55
                    HEX        56,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,65,55
                    HEX        66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,65,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        66,66,66,65,55,55,55,55,55,55,55,55,55,55,55,55,55,66,66,66,65,55
                    HEX        56,66,66,66,55,55,55,55,55,55,55,55,55,55,55,55,56,66,66,66,55,55
                    HEX        56,66,66,66,66,65,55,55,55,55,55,55,55,55,55,66,66,66,66,66,55,55
                    HEX        55,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55
                    HEX        55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55
                    HEX        55,55,56,66,66,66,66,66,66,66,66,66,66,66,66,66,66,66,55,55,55,55
                    HEX        55,55,55,55,66,66,66,66,66,66,66,66,66,66,66,66,65,55,55,55,55,55

                    
*****************************************
*  Score/Life Screen 0-9 Number Pixels  *
*****************************************

NumberTable         ENT
                    DA         Number0,Number1,Number2,Number3,Number4,Number5,Number6,Number7,Number8,Number9

Number0             HEX        00,00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00,00,00  ; 0
                    HEX        00,00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00,00 
                    HEX        00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00 
                    HEX        00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        0E,EE,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,00,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00 
                    HEX        0E,EE,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,00,00 
                    HEX        00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00 
                    HEX        00,00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00,00 
                    HEX        00,00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00,00,00 

Number1             HEX        00,00,00,00,00,00,00,00,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,00  ; 1
                    HEX        00,00,00,00,00,00,00,0E,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,EE,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,0E,EE,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,EE,EE,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,0E,EE,EE,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,EE,EE,EE,E0,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,0E,EE,EE,EE,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,EE,EE,EE,E0,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,0E,EE,EE,EE,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,EE,EE,EE,E0,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,0E,EE,EE,EE,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00 
                    HEX        00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00 
                    HEX        00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00 
                    HEX        00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00 

Number2             HEX        00,00,00,00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00,00,00  ; 2
                    HEX        00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00 
                    HEX        00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        00,0E,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,EE,00,00 
                    HEX        00,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,E0,00 
                    HEX        00,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,E0,00 
                    HEX        00,00,00,00,00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00 
                    HEX        00,00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00 
                    HEX        00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00,00 
                    HEX        00,0E,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00 

Number3             HEX        00,00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00,00,00  ; 3
                    HEX        00,00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00,00 
                    HEX        00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00 
                    HEX        00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        0E,EE,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,00,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00 
                    HEX        0E,EE,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,00,00 
                    HEX        00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00 
                    HEX        00,00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00,00 
                    HEX        00,00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00,00,00 

Number4             HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,E0,00,00,00,00  ; 4
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,EE,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,E0,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,0E,EE,EE,EE,EE,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,EE,EE,EE,EE,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,EE,EE,EE,EE,E0,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,0E,EE,EE,EE,E0,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,0E,EE,EE,EE,EE,00,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,EE,EE,EE,EE,00,00,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,EE,EE,EE,EE,E0,00,00,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,0E,EE,EE,EE,E0,00,00,00,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,0E,EE,EE,EE,EE,00,00,00,00,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,EE,EE,EE,EE,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00,00,00,00 

Number5             HEX        EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00  ; 5
                    HEX        EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        EE,EE,EE,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00,00,00 
                    HEX        EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00,00 
                    HEX        EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00 
                    HEX        EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        EE,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,00,00 
                    HEX        EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        0E,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00 
                    HEX        0E,EE,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,00,00 
                    HEX        00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00 
                    HEX        00,00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00,00 
                    HEX        00,00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00,00,00 

Number6             HEX        00,00,00,00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00,00,00  ; 6
                    HEX        00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00 
                    HEX        00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00 
                    HEX        00,EE,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,EE,E0,00 
                    HEX        00,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        0E,EE,EE,E0,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00,00,00 
                    HEX        0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00 
                    HEX        0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00 
                    HEX        0E,EE,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,EE,E0,00 
                    HEX        0E,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        00,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        00,EE,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,EE,E0,00 
                    HEX        00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00 
                    HEX        00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00 
                    HEX        00,00,00,00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00,00,00 

Number7             HEX        EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00  ; 7
                    HEX        EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,E0,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,E0,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,00,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00 
                    HEX        00,00,00,00,00,00,00,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00 

Number8             HEX        00,00,00,00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00,00,00  ; 8
                    HEX        00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00 
                    HEX        00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00 
                    HEX        00,EE,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,EE,E0,00 
                    HEX        00,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        00,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        00,EE,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,EE,E0,00 
                    HEX        00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00 
                    HEX        00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00 
                    HEX        00,EE,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,EE,E0,00 
                    HEX        00,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,00 
                    HEX        0E,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00 
                    HEX        00,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        00,EE,EE,EE,EE,EE,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,EE,E0,00 
                    HEX        00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00 
                    HEX        00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00 
                    HEX        00,00,00,00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00,00,00 

Number9             HEX        00,00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00,00,00  ; 9
                    HEX        00,00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00,00 
                    HEX        00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00 
                    HEX        00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        0E,EE,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,00,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,E0,00 
                    HEX        0E,EE,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,E0,00 
                    HEX        00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00 
                    HEX        00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00 
                    HEX        00,00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00 
                    HEX        00,00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,0E,EE,EE,E0,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,E0,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,00,00,00,00,EE,EE,EE,E0,00 
                    HEX        0E,EE,EE,EE,00,00,00,00,00,00,00,00,00,00,00,00,0E,EE,EE,EE,00,00 
                    HEX        0E,EE,EE,EE,EE,E0,00,00,00,00,00,00,00,00,00,EE,EE,EE,EE,EE,00,00 
                    HEX        00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00 
                    HEX        00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00 
                    HEX        00,00,0E,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,00,00,00,00 
                    HEX        00,00,00,00,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,EE,E0,00,00,00,00,00 

***********************************************
*  Score/Life Screen :  Your Mission is Over. *
***********************************************

MissionIsOver       ENT                              ; Global Reference
                    HEX        FF,F0,00,0F,FF,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,FF,00,00,0F,F0,00,FF,F0,00,00,00,00,00,00,00,00,00,00,FF,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,FF,00,00,00,00,00,00,00,00,00,00,FF,F0,00,00,00,00,00,00,00,00,00,00,00,00,00
                    HEX        FE,EF,00,FE,EF,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,FE,F0,00,FE,F0,0F,EE,F0,00,00,00,00,00,00,00,00,00,0F,EE,F0,00,00,00,00,00,00,00,00,00,00,00,00,00,0F,EE,F0,00,00,00,00,00,00,00,00,0F,EE,EF,00,00,00,00,00,00,00,00,00,00,00,00,00
                    HEX        FE,EF,00,FE,EF,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,FE,F0,00,FE,F0,0F,EE,F0,00,00,00,00,00,00,00,00,00,0F,EE,F0,00,00,00,00,00,00,00,00,00,00,00,00,00,0F,EE,F0,00,00,00,00,00,00,00,00,FE,EF,EE,F0,00,00,00,00,00,00,00,00,00,00,00,00
                    HEX        FE,EF,00,FE,EF,0F,FF,F0,00,FF,00,FF,0F,FF,0F,F0,00,00,00,00,FE,EF,0F,EE,F0,0F,FF,F0,00,0F,FF,FF,00,0F,FF,FF,00,0F,FF,F0,00,0F,FF,F0,00,FF,FF,F0,00,00,00,00,00,0F,FF,F0,00,0F,FF,FF,00,00,00,0F,EE,FF,FE,EF,0F,F0,0F,F0,00,FF,FF,00,FF,F0,FF,00
                    HEX        FF,EE,FF,EE,FF,FE,EE,EF,0F,EE,FF,EE,FE,EE,FE,EF,00,00,00,00,FE,EF,0F,EE,F0,FE,EE,F0,00,FE,EE,EE,F0,FE,EE,EE,F0,FE,EE,F0,00,FE,EE,EF,0F,EE,EE,EF,00,00,00,00,00,FE,EE,F0,00,FE,EE,EE,F0,00,00,0F,EE,F0,FE,EF,FE,EF,FE,EF,0F,EE,EE,FF,EE,EF,EE,F0
                    HEX        0F,FE,EE,EF,FF,EE,FF,EE,FF,EE,FF,EE,FF,EE,EF,EE,F0,00,00,00,FE,EE,FE,EE,F0,FF,EE,F0,0F,EE,EF,FF,FF,EE,EF,FF,F0,FF,EE,F0,0F,EE,FF,EE,FF,EE,FF,EE,F0,00,00,00,00,FF,EE,F0,0F,EE,EF,FF,F0,00,00,0F,EE,F0,FE,EF,FE,EF,FE,EF,FE,EF,FE,EF,FE,EE,FE,EF
                    HEX        00,FF,EE,FF,0F,EE,FF,EE,FF,EE,FF,EE,FF,EE,FF,EE,F0,00,00,00,FE,EE,EE,EE,F0,0F,EE,F0,0F,FE,EE,F0,0F,FE,EE,F0,00,0F,EE,F0,0F,EE,FF,EE,FF,EE,FF,EE,F0,00,00,00,00,0F,EE,F0,0F,FE,EE,F0,00,00,00,0F,EE,F0,FE,EF,FE,EF,FE,EF,FE,EF,FE,EF,FE,EF,FE,EF
                    HEX        00,0F,EE,F0,0F,EE,FF,EE,FF,EE,FF,EE,FF,EE,FF,FF,F0,00,00,00,FE,EF,EF,EE,F0,0F,EE,F0,00,FF,EE,EF,00,FF,EE,EF,00,0F,EE,F0,0F,EE,FF,EE,FF,EE,FF,EE,F0,00,00,00,00,0F,EE,F0,00,FF,EE,EF,00,00,00,0F,EE,F0,FE,EF,FF,EE,EE,FF,FE,EE,EE,EF,FE,EF,FF,FF
                    HEX        00,0F,EE,F0,0F,EE,FF,EE,FF,EE,FF,EE,FF,EE,F0,00,00,00,00,00,FE,EF,EF,EE,F0,0F,EE,F0,00,FF,FE,EE,F0,FF,FE,EE,F0,0F,EE,F0,0F,EE,FF,EE,FF,EE,FF,EE,F0,00,00,00,00,0F,EE,F0,00,FF,FE,EE,F0,00,00,0F,FE,EF,EE,FF,0F,EE,EE,F0,FE,EF,FF,FF,FE,EF,00,00
                    HEX        00,FE,EE,EF,0F,FE,EE,EF,FF,FE,EE,FE,EE,EE,EF,00,00,00,00,00,FE,EF,FF,EE,F0,FE,EE,EF,0F,EE,EE,EF,FF,EE,EE,EF,F0,FE,EE,EF,0F,FE,EE,EF,FF,EE,FF,EE,F0,00,00,00,00,FE,EE,EF,0F,EE,EE,EF,F0,00,00,00,FF,EE,EF,F0,0F,FE,EF,F0,FF,EE,EE,FF,EE,EE,F0,00
                    HEX        00,FF,FF,FF,00,FF,FF,FF,00,FF,FF,FF,FF,FF,FF,00,00,00,00,00,FF,FF,0F,FF,F0,FF,FF,FF,0F,FF,FF,FF,FF,FF,FF,FF,00,FF,FF,FF,00,FF,FF,FF,0F,FF,FF,FF,F0,00,00,00,00,FF,FF,FF,0F,FF,FF,FF,00,00,00,00,0F,FF,FF,00,00,FF,FF,00,0F,FF,FF,FF,FF,FF,F0,00


**************************************************
*  Attract Screen :  Press Fire Button to Start. *
**************************************************

PressFireButton     ENT                             ; Global Reference
                    HEX        0F,FF,FF,FF,FF,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,FF,FF,FF,FF,FF,FF,FF,00,00,00,00,00,00,00,00,00,00,00,00,FF,FF,FF,FF,F0,00,00,00,00,00,FF,F0,00,0F,FF,00,00,00,00,00,00,00,00,00,00,00,00,00,00,FF,F0,00,00,00,00,00,00,00,00,0F,FF,FF,FF,00,FF,F0,00,00,00,00,00,00,00,00,00,00,0F,FF,00,00
                    HEX        0F,11,11,11,1F,F0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,F1,11,11,11,1F,F1,1F,00,00,00,00,00,00,00,00,00,00,00,00,F1,11,11,11,FF,00,00,00,00,0F,F1,F0,00,FF,1F,00,00,00,00,00,00,00,00,00,00,00,00,00,0F,F1,F0,00,00,00,00,00,00,00,00,FF,11,11,1F,FF,F1,F0,00,00,00,00,00,00,00,00,00,00,FF,1F,00,00
                    HEX        0F,F1,1F,FF,11,FF,FF,FF,FF,F0,FF,FF,FF,F0,FF,FF,FF,FF,FF,FF,FF,FF,00,00,00,FF,11,FF,F1,1F,FF,FF,FF,FF,FF,FF,F0,FF,FF,FF,F0,00,00,00,FF,11,FF,F1,1F,FF,F0,FF,FF,FF,11,FF,FF,F1,1F,FF,0F,FF,FF,FF,FF,FF,FF,FF,00,00,00,00,FF,11,FF,F0,FF,FF,FF,F0,00,00,00,F1,1F,FF,11,FF,11,FF,F0,FF,FF,FF,F0,FF,FF,FF,FF,FF,F1,1F,FF,00
                    HEX        00,F1,1F,FF,11,F1,11,F1,11,FF,F1,11,11,FF,F1,11,11,1F,F1,11,11,1F,00,00,00,0F,11,FF,FF,FF,11,1F,F1,11,F1,11,FF,F1,11,11,FF,00,00,00,0F,11,FF,F1,1F,11,F0,F1,1F,F1,11,11,FF,11,11,1F,FF,11,11,1F,F1,1F,11,1F,F0,00,00,00,F1,11,11,FF,F1,11,11,FF,00,00,00,F1,11,FF,FF,F1,11,11,F0,F1,11,11,FF,F1,11,F1,11,FF,11,11,1F,00
                    HEX        00,F1,11,11,1F,FF,11,1F,F1,1F,11,FF,F1,1F,11,FF,FF,FF,11,FF,FF,FF,00,00,00,0F,11,11,1F,0F,F1,1F,FF,11,1F,F1,1F,11,FF,F1,1F,00,00,00,0F,11,11,11,FF,11,F0,F1,1F,FF,11,FF,FF,F1,1F,FF,F1,1F,FF,11,F1,11,FF,11,F0,00,00,00,FF,11,FF,FF,11,FF,F1,1F,00,00,00,FF,F1,11,FF,FF,11,FF,F0,FF,FF,F1,1F,FF,11,1F,F1,1F,F1,1F,FF,00
                    HEX        00,F1,1F,FF,FF,0F,11,FF,F1,1F,11,11,11,1F,F1,11,11,FF,F1,11,11,FF,00,00,00,0F,11,FF,FF,00,F1,1F,0F,11,FF,F1,1F,11,11,11,1F,00,00,00,0F,11,FF,F1,1F,11,F0,F1,1F,0F,11,FF,FF,F1,1F,FF,F1,1F,0F,11,F1,1F,FF,11,F0,00,00,00,0F,11,FF,FF,11,F0,F1,1F,00,00,00,FF,FF,F1,11,FF,11,FF,FF,F1,11,11,1F,0F,11,FF,F1,1F,F1,1F,FF,F0
                    HEX        0F,F1,1F,FF,00,FF,11,FF,FF,FF,11,FF,FF,FF,FF,FF,F1,1F,FF,FF,F1,1F,00,00,00,FF,11,FF,F0,0F,F1,1F,FF,11,FF,FF,FF,11,FF,FF,FF,00,00,00,FF,11,FF,F1,1F,11,FF,F1,1F,FF,11,FF,1F,F1,1F,F1,F1,1F,FF,11,F1,1F,0F,11,F0,00,00,00,0F,11,FF,1F,11,FF,F1,1F,00,00,00,F1,1F,FF,11,FF,11,FF,1F,11,FF,F1,1F,FF,11,FF,FF,FF,F1,1F,F1,F0
                    HEX        0F,11,11,1F,00,F1,11,1F,00,0F,F1,11,11,FF,11,11,11,FF,11,11,11,FF,00,00,00,F1,11,11,F0,0F,11,11,F1,11,1F,00,0F,F1,11,11,F0,00,00,00,F1,11,11,11,FF,F1,11,1F,11,FF,F1,11,FF,FF,11,1F,FF,11,11,1F,F1,1F,0F,11,F0,00,00,00,0F,F1,11,FF,F1,11,11,FF,00,00,00,FF,11,11,1F,FF,F1,11,FF,F1,11,1F,11,F1,11,1F,00,00,FF,11,1F,F0
                    HEX        0F,FF,FF,FF,00,FF,FF,FF,00,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,00,00,00,FF,FF,FF,F0,0F,FF,FF,FF,FF,FF,00,00,FF,FF,FF,F0,00,00,00,FF,FF,FF,FF,F0,FF,FF,FF,FF,F0,FF,FF,F0,0F,FF,FF,0F,FF,FF,FF,FF,FF,0F,FF,F0,00,00,00,00,FF,FF,F0,FF,FF,FF,F0,00,00,00,0F,FF,FF,FF,00,FF,FF,F0,FF,FF,FF,FF,FF,FF,FF,00,00,0F,FF,FF,00


********************************
*  Final Score :  Final Score. *
********************************

FinalScore          ENT                             ; Global Reference
                    HEX        0F,FF,FF,FF,FF,0F,FF,FF,FF,FF,F0,FF,FF,00,FF,FF,0F,FF,FF,F0,00,00,00,00,00,0F,FF,FF,F0,0F,FF,FF,F0,0F,FF,FF,0F,FF,FF,FF,FF,FF,FF,FF,FF,00
                    HEX        0F,EE,EE,EE,EF,0F,EE,EE,FF,EE,FF,FE,EF,0F,FE,EF,FF,EE,EE,F0,00,00,00,00,00,FF,EE,EE,FF,FF,EE,EE,FF,FF,EE,EF,FF,EE,EE,EE,FF,EE,EE,EE,EF,00
                    HEX        0F,FE,EF,FF,EF,0F,FE,EF,FF,EE,EF,FE,EF,FF,EE,EE,FF,FE,EF,F0,00,00,00,00,00,FE,EF,FE,EF,FE,EF,FE,EF,FE,EF,EE,FF,FE,EF,FE,EF,FE,EF,FF,EF,00
                    HEX        00,FE,EF,EF,FF,00,FE,EF,FF,EE,EE,FE,EF,FE,EF,FE,EF,FE,EF,00,00,00,00,00,00,FE,EE,FF,FF,EE,FF,FF,FF,EE,FF,FE,EF,FE,EF,FE,EF,FE,EF,EF,FF,00
                    HEX        00,FE,EE,EF,00,00,FE,EF,0F,EE,FE,EE,EF,FE,EF,FE,EF,FE,EF,0F,FF,00,00,00,00,FF,EE,EF,FF,EE,F0,00,0F,EE,F0,FE,EF,FE,EE,EE,FF,FE,EE,EF,00,00
                    HEX        00,FE,EF,EF,00,00,FE,EF,0F,EE,FF,EE,EF,FE,EE,EE,EF,FE,EF,FF,EF,00,00,00,00,FF,FF,EE,EF,EE,FF,FF,FF,EE,FF,FE,EF,FE,EF,EE,FF,FE,EF,EF,FF,00
                    HEX        0F,FE,EF,FF,00,0F,FE,EF,FF,EE,FF,FE,EF,FE,EF,FE,EF,FE,EF,FE,EF,00,00,00,00,FE,EF,FE,EF,FE,EF,FE,EF,FE,EF,EE,FF,FE,EF,FE,EF,FE,EF,FF,EF,00
                    HEX        0F,EE,EE,F0,00,0F,EE,EE,FF,EE,F0,FE,EF,FE,EF,FE,EF,EE,EE,EE,EF,00,00,00,00,FF,EE,EE,FF,FF,EE,EE,FF,FF,EE,EF,FF,EE,EF,FE,EF,EE,EE,EE,EF,00
                    HEX        0F,FF,FF,F0,00,0F,FF,FF,FF,FF,F0,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,00,00,00,0F,FF,FF,F0,0F,FF,FF,F0,0F,FF,FF,0F,FF,FF,FF,FF,FF,FF,FF,FF,00
 
 
********************************************
*  Alphabet :  A-Z a-z 0-9 Actions Arrows. *
********************************************
 
Alpha_Char          ENT
                    ASC        'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789abcdefghijklmnopqrstuvwxyz ."/[]',00 

Alpha_Tab           ENT
                    DA         Aplha_A,Aplha_B,Aplha_C,Aplha_D,Aplha_E,Aplha_F,Aplha_G,Aplha_H,Aplha_I,Aplha_J
                    DA         Aplha_K,Aplha_L,Aplha_M,Aplha_N,Aplha_O,Aplha_P,Aplha_Q,Aplha_R,Aplha_S,Aplha_T
                    DA         Aplha_U,Aplha_V,Aplha_W,Aplha_X,Aplha_Y,Aplha_Z,Aplha_0,Aplha_1,Aplha_2,Aplha_3
                    DA         Aplha_4,Aplha_5,Aplha_6,Aplha_7,Aplha_8,Aplha_9,Aplha_a,Aplha_b,Aplha_c,Aplha_d
                    DA         Aplha_e,Aplha_f,Aplha_g,Aplha_h,Aplha_i,Aplha_j,Aplha_k,Aplha_l,Aplha_m,Aplha_n
                    DA         Aplha_o,Aplha_p,Aplha_q,Aplha_r,Aplha_s,Aplha_t,Aplha_u,Aplha_v,Aplha_w,Aplha_x
                    DA         Aplha_y,Aplha_z,Aplha_Space,Aplha_Dot,Aplha_Quote,Aplha_Slash,Aplha_OpenBracket,
                    DA         Aplha_CloseBracket
 
Aplha_A             HEX        00,02,22,20,00,00    ; A
                    HEX        00,22,11,22,00,00
                    HEX        02,21,11,12,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,11,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_B             HEX        22,22,22,22,00,00    ; B
                    HEX        21,11,11,12,20,00
                    HEX        22,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,11,12,20,00
                    HEX        02,11,22,11,20,00
                    HEX        22,11,22,11,20,00
                    HEX        21,11,11,12,20,00
                    HEX        22,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_C             HEX        00,22,22,22,00,00    ; C
                    HEX        02,21,11,12,20,00
                    HEX        22,11,22,11,20,00
                    HEX        21,12,22,22,20,00
                    HEX        21,12,00,00,00,00
                    HEX        21,12,22,22,20,00
                    HEX        22,11,22,11,20,00
                    HEX        02,21,11,12,20,00
                    HEX        00,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_D             HEX        22,22,22,20,00,00    ; D
                    HEX        21,11,11,22,00,00
                    HEX        22,11,21,12,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        22,11,21,12,20,00
                    HEX        21,11,11,22,00,00
                    HEX        22,22,22,20,00,00
                    HEX        00,00,00,00,00,00

Aplha_E             HEX        22,22,22,22,20,00    ; E
                    HEX        21,11,11,11,20,00
                    HEX        22,11,22,21,20,00
                    HEX        02,11,21,22,20,00
                    HEX        02,11,11,20,00,00
                    HEX        02,11,21,22,20,00
                    HEX        22,11,22,21,20,00
                    HEX        21,11,11,11,20,00
                    HEX        22,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_F             HEX        22,22,22,22,20,00    ; F
                    HEX        21,11,11,11,20,00
                    HEX        22,11,22,21,20,00
                    HEX        02,11,21,22,20,00
                    HEX        02,11,11,20,00,00
                    HEX        02,11,21,20,00,00
                    HEX        22,11,22,20,00,00
                    HEX        21,11,12,00,00,00
                    HEX        22,22,22,00,00,00
                    HEX        00,00,00,00,00,00

Aplha_G             HEX        00,22,22,22,00,00    ; G
                    HEX        02,21,11,12,20,00
                    HEX        22,11,22,11,20,00
                    HEX        21,12,22,22,20,00
                    HEX        21,12,22,22,20,00
                    HEX        21,12,21,11,20,00
                    HEX        22,11,22,11,20,00
                    HEX        02,21,11,11,20,00
                    HEX        00,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_H             HEX        02,22,22,22,20,00    ; H
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,11,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_I             HEX        00,22,22,22,00,00    ; I
                    HEX        00,21,11,12,00,00
                    HEX        00,22,11,22,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,22,11,22,00,00
                    HEX        00,21,11,12,00,00
                    HEX        00,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_J             HEX        00,02,22,22,20,00    ; J
                    HEX        00,02,11,11,20,00
                    HEX        00,02,21,12,20,00
                    HEX        00,00,21,12,00,00
                    HEX        22,22,21,12,00,00
                    HEX        21,12,21,12,00,00
                    HEX        21,12,21,12,00,00
                    HEX        22,11,11,22,00,00
                    HEX        02,22,22,20,00,00
                    HEX        00,00,00,00,00,00

Aplha_K             HEX        22,22,22,22,20,00    ; K
                    HEX        21,11,22,11,20,00
                    HEX        22,11,22,11,20,00
                    HEX        02,11,21,12,20,00
                    HEX        02,11,11,22,00,00
                    HEX        02,11,21,12,20,00
                    HEX        22,11,22,11,20,00
                    HEX        21,11,22,11,20,00
                    HEX        22,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_L             HEX        22,22,22,00,00,00    ; L
                    HEX        21,11,12,00,00,00
                    HEX        22,11,22,00,00,00
                    HEX        02,11,20,00,00,00
                    HEX        02,11,20,22,20,00
                    HEX        02,11,22,21,20,00
                    HEX        22,11,22,11,20,00
                    HEX        21,11,11,11,20,00
                    HEX        22,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_M             HEX        22,22,02,22,20,00    ; M
                    HEX        21,12,22,11,20,00
                    HEX        21,11,21,11,20,00
                    HEX        21,11,11,11,20,00
                    HEX        21,11,11,11,20,00
                    HEX        21,12,12,11,20,00
                    HEX        21,12,22,11,20,00
                    HEX        21,12,02,11,20,00
                    HEX        22,22,02,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_N             HEX        22,22,02,22,20,00    ; N
                    HEX        21,12,22,11,20,00
                    HEX        21,11,22,11,20,00
                    HEX        21,11,12,11,20,00
                    HEX        21,12,11,11,20,00
                    HEX        21,12,21,11,20,00
                    HEX        21,12,22,11,20,00
                    HEX        21,12,02,11,20,00
                    HEX        22,22,02,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_O             HEX        00,22,22,20,00,00    ; O
                    HEX        02,21,11,22,00,00
                    HEX        22,11,21,12,20,00
                    HEX        21,12,22,11,20,00
                    HEX        21,12,02,11,20,00
                    HEX        21,12,22,11,20,00
                    HEX        22,11,21,12,20,00
                    HEX        02,21,11,22,00,00
                    HEX        00,22,22,20,00,00
                    HEX        00,00,00,00,00,00

Aplha_P             HEX        22,22,22,22,00,00    ; P
                    HEX        21,11,11,12,20,00
                    HEX        22,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,11,12,20,00
                    HEX        02,11,22,22,00,00
                    HEX        22,11,22,00,00,00
                    HEX        21,11,12,00,00,00
                    HEX        22,22,22,00,00,00
                    HEX        00,00,00,00,00,00

Aplha_Q             HEX        00,22,22,22,00,00    ; Q
                    HEX        02,21,11,12,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,21,11,20,00
                    HEX        02,21,11,12,20,00
                    HEX        00,22,21,11,20,00
                    HEX        00,00,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_R             HEX        22,22,22,22,00,00    ; R
                    HEX        21,11,11,12,20,00
                    HEX        22,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,11,12,20,00
                    HEX        02,11,21,12,20,00
                    HEX        22,11,22,11,20,00
                    HEX        21,11,22,11,20,00
                    HEX        22,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_S             HEX        00,22,22,22,00,00    ; S
                    HEX        02,21,11,12,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,12,22,20,00
                    HEX        02,21,11,22,20,00
                    HEX        02,22,21,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,21,11,12,20,00
                    HEX        00,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_T             HEX        02,22,22,22,20,00    ; T
                    HEX        02,11,11,11,20,00
                    HEX        02,12,11,21,20,00
                    HEX        02,22,11,22,20,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,22,11,22,00,00
                    HEX        00,21,11,12,00,00
                    HEX        00,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_U             HEX        02,22,22,22,20,00    ; U
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,21,11,12,20,00
                    HEX        00,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_V             HEX        02,22,22,22,20,00    ; V
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,21,11,12,20,00
                    HEX        00,22,11,22,00,00
                    HEX        00,02,22,20,00,00
                    HEX        00,00,00,00,00,00

Aplha_W             HEX        22,22,02,22,20,00    ; W
                    HEX        21,12,02,11,20,00
                    HEX        21,12,02,11,20,00
                    HEX        21,12,22,11,20,00
                    HEX        21,12,12,11,20,00
                    HEX        21,11,11,11,20,00
                    HEX        21,11,21,11,20,00
                    HEX        21,12,22,11,20,00
                    HEX        22,22,02,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_X             HEX        22,22,02,22,20,00    ; X
                    HEX        21,12,02,11,20,00
                    HEX        21,12,22,11,20,00
                    HEX        22,11,21,12,20,00
                    HEX        02,21,11,22,00,00
                    HEX        02,21,11,22,00,00
                    HEX        22,11,21,12,20,00
                    HEX        21,12,22,11,20,00
                    HEX        22,22,02,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_Y             HEX        02,22,22,22,20,00    ; Y
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,21,11,12,20,00
                    HEX        00,22,11,22,00,00
                    HEX        00,22,11,22,00,00
                    HEX        00,21,11,12,00,00
                    HEX        00,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_Z             HEX        22,22,22,22,20,00    ; Z
                    HEX        21,11,11,11,20,00
                    HEX        21,12,22,11,20,00
                    HEX        21,22,21,12,20,00
                    HEX        22,22,11,22,20,00
                    HEX        02,21,12,21,20,00
                    HEX        22,11,22,11,20,00
                    HEX        21,11,11,11,20,00
                    HEX        22,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_0             HEX        02,22,22,22,00,00    ; 0
                    HEX        22,11,11,12,20,00
                    HEX        21,12,22,11,20,00
                    HEX        21,12,21,11,20,00
                    HEX        21,12,11,11,20,00
                    HEX        21,11,12,11,20,00
                    HEX        21,11,22,11,20,00
                    HEX        22,11,11,12,20,00
                    HEX        02,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_1             HEX        00,02,22,20,00,00    ; 1
                    HEX        00,22,11,20,00,00
                    HEX        00,21,11,20,00,00
                    HEX        00,22,11,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        02,22,11,22,20,00
                    HEX        02,11,11,11,20,00
                    HEX        02,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_2             HEX        00,22,22,22,00,00    ; 2
                    HEX        02,21,11,12,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,22,22,11,20,00
                    HEX        00,22,11,12,20,00
                    HEX        02,21,12,22,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,11,11,20,00
                    HEX        02,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_3             HEX        00,22,22,22,00,00    ; 3
                    HEX        02,21,11,12,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,22,22,11,20,00
                    HEX        00,02,11,12,20,00
                    HEX        02,22,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,21,11,12,20,00
                    HEX        00,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_4             HEX        00,02,22,22,00,00    ; 4
                    HEX        00,22,11,12,00,00
                    HEX        02,21,11,12,00,00
                    HEX        22,11,21,12,00,00
                    HEX        21,12,21,12,20,00
                    HEX        21,11,11,11,20,00
                    HEX        22,22,21,12,20,00
                    HEX        00,02,11,11,20,00
                    HEX        00,02,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_5             HEX        02,22,22,22,20,00    ; 5
                    HEX        02,11,11,11,20,00
                    HEX        02,11,22,22,20,00
                    HEX        02,11,11,12,20,00
                    HEX        02,22,22,11,20,00
                    HEX        02,22,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,21,11,12,20,00
                    HEX        00,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_6             HEX        00,02,22,22,00,00    ; 6
                    HEX        00,22,11,12,00,00
                    HEX        02,21,12,22,00,00
                    HEX        02,11,22,22,00,00
                    HEX        02,11,11,12,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,21,11,12,20,00
                    HEX        00,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_7             HEX        02,22,22,22,20,00    ; 7
                    HEX        02,11,11,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,22,22,11,20,00
                    HEX        00,02,21,12,20,00
                    HEX        00,02,11,22,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,22,20,00,00
                    HEX        00,00,00,00,00,00

Aplha_8             HEX        00,22,22,22,00,00    ; 8
                    HEX        02,21,11,12,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,21,11,12,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,21,11,12,20,00
                    HEX        00,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_9             HEX        00,22,22,22,00,00    ; 9
                    HEX        02,21,11,12,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,21,11,11,20,00
                    HEX        00,22,22,11,20,00
                    HEX        00,22,22,12,20,00
                    HEX        00,21,11,22,00,00
                    HEX        00,22,22,20,00,00
                    HEX        00,00,00,00,00,00

Aplha_a             HEX        00,00,00,00,00,00    ; a
                    HEX        00,00,00,00,00,00
                    HEX        00,22,22,00,00,00
                    HEX        02,11,11,20,00,00
                    HEX        00,22,21,12,00,00
                    HEX        02,11,11,12,00,00
                    HEX        21,12,21,12,20,00
                    HEX        22,11,12,11,20,00
                    HEX        02,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_b             HEX        02,22,00,00,00,00    ; b
                    HEX        21,11,20,00,00,00
                    HEX        02,11,20,00,00,00
                    HEX        02,11,22,22,00,00
                    HEX        02,11,11,12,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        21,12,11,12,20,00
                    HEX        22,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_c             HEX        00,00,00,00,00,00    ; c
                    HEX        00,00,00,00,00,00
                    HEX        00,02,22,20,00,00
                    HEX        00,21,11,12,00,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,22,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,21,11,12,20,00
                    HEX        00,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_d             HEX        00,00,22,20,00,00    ; d
                    HEX        00,02,11,12,00,00
                    HEX        00,00,21,12,00,00
                    HEX        00,22,21,12,00,00
                    HEX        02,11,11,12,00,00
                    HEX        21,12,21,12,00,00
                    HEX        21,12,21,12,20,00
                    HEX        22,11,12,11,20,00
                    HEX        02,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_e             HEX        00,00,00,00,00,00    ; e
                    HEX        00,00,00,00,00,00
                    HEX        00,02,22,20,00,00
                    HEX        00,21,11,12,00,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,11,11,20,00
                    HEX        02,11,22,22,20,00
                    HEX        02,21,11,12,00,00
                    HEX        00,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_f             HEX        00,00,22,20,00,00    ; f
                    HEX        00,02,11,12,00,00
                    HEX        00,21,12,11,20,00
                    HEX        00,21,12,22,20,00
                    HEX        02,11,11,20,00,00
                    HEX        02,21,12,20,00,00
                    HEX        02,21,12,20,00,00
                    HEX        02,11,11,20,00,00
                    HEX        02,22,22,20,00,00
                    HEX        00,00,00,00,00,00

Aplha_g             HEX        00,00,00,00,00,00    ; g
                    HEX        00,00,00,00,00,00
                    HEX        00,22,20,22,00,00
                    HEX        02,11,12,11,20,00
                    HEX        21,12,21,12,20,00
                    HEX        21,12,21,12,00,00
                    HEX        02,11,11,12,00,00
                    HEX        02,22,21,12,00,00
                    HEX        21,11,11,22,00,00
                    HEX        22,22,22,20,00,00

Aplha_h             HEX        02,22,00,00,00,00    ; h
                    HEX        21,11,20,00,00,00
                    HEX        02,11,22,20,00,00
                    HEX        02,11,21,12,00,00
                    HEX        02,11,12,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        22,11,22,11,20,00
                    HEX        21,11,22,11,20,00
                    HEX        22,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_i             HEX        00,00,22,00,00,00    ; i
                    HEX        00,02,11,20,00,00
                    HEX        00,02,22,20,00,00
                    HEX        00,21,11,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,11,22,00,00
                    HEX        00,21,11,12,00,00
                    HEX        00,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_j             HEX        00,00,00,22,00,00    ; j
                    HEX        00,00,02,11,20,00
                    HEX        00,00,00,22,20,00
                    HEX        00,00,02,11,20,00
                    HEX        00,00,02,11,20,00
                    HEX        00,22,02,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,21,11,12,20,00
                    HEX        00,22,22,22,00,00

Aplha_k             HEX        02,22,00,00,00,00    ; k
                    HEX        21,11,20,00,00,00
                    HEX        02,11,20,22,00,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,21,12,00,00
                    HEX        02,11,11,22,00,00
                    HEX        22,11,21,12,20,00
                    HEX        21,11,22,11,20,00
                    HEX        22,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_l             HEX        00,02,22,00,00,00    ; l
                    HEX        00,21,11,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,22,11,20,00,00
                    HEX        00,21,11,12,00,00
                    HEX        00,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_m             HEX        00,00,00,00,00,00    ; m
                    HEX        00,00,00,00,00,00
                    HEX        02,20,02,20,00,00
                    HEX        21,12,21,12,00,00
                    HEX        21,11,11,11,20,00
                    HEX        21,11,11,11,20,00
                    HEX        21,12,12,11,20,00
                    HEX        21,12,22,11,20,00
                    HEX        22,22,02,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_n             HEX        00,00,00,00,00,00    ; n
                    HEX        00,00,00,00,00,00
                    HEX        00,22,22,20,00,00
                    HEX        02,11,11,12,00,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_o             HEX        00,00,00,00,00,00    ; o
                    HEX        00,00,00,00,00,00
                    HEX        00,02,22,20,00,00
                    HEX        00,21,11,12,00,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,21,11,12,20,00
                    HEX        00,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_p             HEX        00,00,00,00,00,00    ; p
                    HEX        00,00,00,00,00,00
                    HEX        02,20,22,20,00,00
                    HEX        21,12,11,12,00,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,11,12,20,00
                    HEX        02,11,22,22,00,00
                    HEX        21,11,12,00,00,00
                    HEX        22,22,22,00,00,00

Aplha_q             HEX        00,00,00,00,00,00    ; q
                    HEX        00,00,00,00,00,00
                    HEX        00,22,20,22,00,00
                    HEX        02,11,12,11,20,00
                    HEX        21,12,21,12,20,00
                    HEX        21,12,21,12,00,00
                    HEX        22,11,11,12,00,00
                    HEX        02,22,21,12,20,00
                    HEX        00,02,11,11,20,00
                    HEX        00,02,22,22,20,00

Aplha_r             HEX        00,00,00,00,00,00    ; r
                    HEX        00,00,00,00,00,00
                    HEX        02,20,22,20,00,00
                    HEX        21,12,11,12,00,00
                    HEX        02,11,12,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,22,20,00
                    HEX        21,11,12,00,00,00
                    HEX        22,22,22,00,00,00
                    HEX        00,00,00,00,00,00

Aplha_s             HEX        00,00,00,00,00,00    ; s
                    HEX        00,00,00,00,00,00
                    HEX        00,02,22,22,00,00
                    HEX        00,21,11,11,20,00
                    HEX        02,11,22,22,00,00
                    HEX        00,21,11,12,20,00
                    HEX        00,02,22,11,20,00
                    HEX        02,11,11,12,20,00
                    HEX        02,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_t             HEX        00,00,20,00,00,00    ; t
                    HEX        00,02,12,00,00,00
                    HEX        00,21,12,00,00,00
                    HEX        02,11,11,12,00,00
                    HEX        02,21,12,22,00,00
                    HEX        00,21,12,22,00,00
                    HEX        00,21,12,12,00,00
                    HEX        00,22,11,22,00,00
                    HEX        00,02,22,20,00,00
                    HEX        00,00,00,00,00,00

Aplha_u             HEX        00,00,00,00,00,00    ; u
                    HEX        00,00,00,00,00,00
                    HEX        02,20,02,20,00,00
                    HEX        21,12,21,12,00,00
                    HEX        21,12,21,12,00,00
                    HEX        21,12,21,12,00,00
                    HEX        21,12,21,12,20,00
                    HEX        22,11,12,11,20,00
                    HEX        02,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_v             HEX        00,00,00,00,00,00    ; v
                    HEX        00,00,00,00,00,00
                    HEX        00,22,00,22,00,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,21,11,12,20,00
                    HEX        00,22,11,22,00,00
                    HEX        00,02,22,20,00,00
                    HEX        00,00,00,00,00,00

Aplha_w             HEX        00,00,00,00,00,00    ; w
                    HEX        00,00,00,00,00,00
                    HEX        02,20,00,22,00,00
                    HEX        21,12,22,11,20,00
                    HEX        21,12,12,11,20,00
                    HEX        21,11,11,11,20,00
                    HEX        21,11,11,11,20,00
                    HEX        22,11,21,12,20,00
                    HEX        02,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_x             HEX        00,00,00,00,00,00    ; x
                    HEX        00,00,00,00,00,00
                    HEX        02,20,00,22,00,00
                    HEX        21,12,22,11,20,00
                    HEX        22,11,21,12,20,00
                    HEX        02,21,11,22,00,00
                    HEX        02,11,21,12,20,00
                    HEX        21,12,22,11,20,00
                    HEX        22,20,00,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_y             HEX        00,00,00,00,00,00    ; y
                    HEX        00,00,00,00,00,00
                    HEX        00,22,00,22,00,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,11,22,11,20,00
                    HEX        02,21,11,11,20,00
                    HEX        00,22,22,11,20,00
                    HEX        02,11,11,12,20,00
                    HEX        02,22,22,22,00,00

Aplha_z             HEX        00,00,00,00,00,00    ; z
                    HEX        00,00,00,00,00,00
                    HEX        00,22,22,22,00,00
                    HEX        02,11,11,11,20,00
                    HEX        02,12,21,12,00,00
                    HEX        02,22,11,22,20,00
                    HEX        00,21,12,21,20,00
                    HEX        02,11,11,11,20,00
                    HEX        02,22,22,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_Space         HEX        00,00,00,00,00,00    ; Space
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00

Aplha_Dot           HEX        00,00,00,00,00,00    ; .
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,22,00,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,22,20,00,00
                    HEX        00,00,00,00,00,00

Aplha_Quote         HEX        00,00,00,00,00,00    ; '
                    HEX        00,02,22,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,22,20,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    
Aplha_Slash         HEX        00,00,02,22,20,00    ; /
                    HEX        00,00,02,11,20,00
                    HEX        00,00,02,11,20,00
                    HEX        00,00,21,12,00,00
                    HEX        00,00,21,12,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,02,11,20,00,00
                    HEX        00,21,12,00,00,00
                    HEX        00,22,22,00,00,00
                    HEX        00,00,00,00,00,00
                    
Aplha_OpenBracket   HEX        00,22,22,22,20,00    ; [
                    HEX        00,21,11,11,20,00
                    HEX        00,21,12,22,20,00
                    HEX        00,21,12,00,00,00
                    HEX        00,21,12,00,00,00
                    HEX        00,21,12,00,00,00
                    HEX        00,21,12,22,20,00
                    HEX        00,21,11,11,20,00
                    HEX        00,22,22,22,20,00
                    HEX        00,00,00,00,00,00
                    
Aplha_CloseBracket  HEX        02,22,22,22,00,00    ; ]
                    HEX        02,11,11,12,00,00
                    HEX        02,22,21,12,00,00
                    HEX        00,00,21,12,00,00
                    HEX        00,00,21,12,00,00
                    HEX        00,00,21,12,00,00
                    HEX        02,22,21,12,00,00
                    HEX        02,11,11,12,00,00
                    HEX        02,22,22,22,00,00
                    HEX        00,00,00,00,00,00

Aplha_Action        ENT
                    HEX        22,22,22,22,22,22    ; Action [-]
                    HEX        21,11,11,11,11,12
                    HEX        21,11,11,11,11,12
                    HEX        21,12,22,22,21,12
                    HEX        21,12,22,22,21,12
                    HEX        22,22,21,12,22,22
                    HEX        22,22,21,12,22,22
                    HEX        21,12,22,22,21,12
                    HEX        21,12,22,22,21,12
                    HEX        21,11,11,11,11,12
                    HEX        21,11,11,11,11,12
                    HEX        22,22,22,22,22,22

Aplha_Action_Mask   HEX        00,00,00,00,00,00    ; Action [-]
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00

Aplha_Event         ENT
                    HEX        00,00,00,00,00,00    ; Event []
                    HEX        02,22,22,22,22,20
                    HEX        02,11,21,11,11,20
                    HEX        02,11,21,11,11,20
                    HEX        02,11,22,22,22,20
                    HEX        02,11,20,02,11,20
                    HEX        02,11,20,02,11,20
                    HEX        02,22,22,22,11,20
                    HEX        02,11,11,12,11,20
                    HEX        02,11,11,12,11,20
                    HEX        02,22,22,22,22,20
                    HEX        00,00,00,00,00,00

Aplha_Event_Mask    HEX        FF,FF,FF,FF,FF,FF    ; Event []
                    HEX        F0,00,00,00,00,0F
                    HEX        F0,00,00,00,00,0F
                    HEX        F0,00,00,00,00,0F
                    HEX        F0,00,00,00,00,0F
                    HEX        F0,00,0F,F0,00,0F
                    HEX        F0,00,0F,F0,00,0F
                    HEX        F0,00,00,00,00,0F
                    HEX        F0,00,00,00,00,0F
                    HEX        F0,00,00,00,00,0F
                    HEX        F0,00,00,00,00,0F
                    HEX        FF,FF,FF,FF,FF,FF

Aplha_Up            ENT
                    HEX        00,00,22,22,00,00    ; ^
                    HEX        00,02,21,12,20,00
                    HEX        00,22,11,11,22,00
                    HEX        02,21,11,11,12,20
                    HEX        02,11,21,12,11,20
                    HEX        02,12,21,12,21,20
                    HEX        02,22,21,12,22,20
                    HEX        00,00,21,12,00,00
                    HEX        00,00,21,12,00,00
                    HEX        00,00,21,12,00,00
                    HEX        00,00,21,12,00,00
                    HEX        00,00,22,22,00,00

Aplha_Up_Mask       HEX        FF,FF,00,00,FF,FF    ; ^
                    HEX        FF,F0,00,00,0F,FF
                    HEX        FF,00,00,00,00,FF
                    HEX        F0,00,00,00,00,0F
                    HEX        F0,00,00,00,00,0F
                    HEX        F0,00,00,00,00,0F
                    HEX        F0,00,00,00,00,0F
                    HEX        FF,FF,00,00,FF,FF
                    HEX        FF,FF,00,00,FF,FF
                    HEX        FF,FF,00,00,FF,FF
                    HEX        FF,FF,00,00,FF,FF
                    HEX        FF,FF,00,00,FF,FF

Aplha_Down          ENT
                    HEX        00,00,22,22,00,00    ; v
                    HEX        00,00,21,12,00,00
                    HEX        00,00,21,12,00,00
                    HEX        00,00,21,12,00,00
                    HEX        00,00,21,12,00,00
                    HEX        02,22,21,12,22,20
                    HEX        02,12,21,12,21,20
                    HEX        02,11,21,12,11,20
                    HEX        02,21,11,11,12,20
                    HEX        00,22,11,11,22,00
                    HEX        00,02,21,12,20,00
                    HEX        00,00,22,22,00,00

Aplha_Down_Mask     HEX        FF,FF,00,00,FF,FF    ; v
                    HEX        FF,FF,00,00,FF,FF
                    HEX        FF,FF,00,00,FF,FF
                    HEX        FF,FF,00,00,FF,FF
                    HEX        FF,FF,00,00,FF,FF
                    HEX        F0,00,00,00,00,0F
                    HEX        F0,00,00,00,00,0F
                    HEX        F0,00,00,00,00,0F
                    HEX        F0,00,00,00,00,0F
                    HEX        FF,00,00,00,00,FF
                    HEX        FF,F0,00,00,0F,FF
                    HEX        FF,FF,00,00,FF,FF

Aplha_Left          ENT
                    HEX        00,00,00,00,00,00    ; <-
                    HEX        00,02,22,20,00,00
                    HEX        00,22,11,20,00,00
                    HEX        02,21,12,20,00,00
                    HEX        22,11,22,22,22,22
                    HEX        21,11,11,11,11,12
                    HEX        21,11,11,11,11,12
                    HEX        22,11,22,22,22,22
                    HEX        02,21,12,20,00,00
                    HEX        00,22,11,20,00,00
                    HEX        00,02,22,20,00,00
                    HEX        00,00,00,00,00,00

Aplha_Left_Mask     HEX        FF,FF,FF,FF,FF,FF    ; <-
                    HEX        FF,F0,00,0F,FF,FF
                    HEX        FF,00,00,0F,FF,FF
                    HEX        F0,00,00,0F,FF,FF
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        F0,00,00,0F,FF,FF
                    HEX        FF,00,00,0F,FF,FF
                    HEX        FF,F0,00,0F,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF

Aplha_Right         ENT
                    HEX        00,00,00,00,00,00    ; ->
                    HEX        00,00,02,22,20,00
                    HEX        00,00,02,11,22,00
                    HEX        00,00,02,21,12,20
                    HEX        22,22,22,22,11,22
                    HEX        21,11,11,11,11,12
                    HEX        21,11,11,11,11,12
                    HEX        22,22,22,22,11,22
                    HEX        00,00,02,21,12,20
                    HEX        00,00,02,11,22,00
                    HEX        00,00,02,22,20,00
                    HEX        00,00,00,00,00,00

Aplha_Right_Mask    HEX        FF,FF,FF,FF,FF,FF    ; ->
                    HEX        FF,FF,F0,00,0F,FF
                    HEX        FF,FF,F0,00,00,FF
                    HEX        FF,FF,F0,00,00,0F
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        FF,FF,F0,00,00,0F
                    HEX        FF,FF,F0,00,00,FF
                    HEX        FF,FF,F0,00,0F,FF
                    HEX        FF,FF,FF,FF,FF,FF

Aplha_Bullet        ENT
                    HEX        00,00,00,00,00,00    ; o
                    HEX        00,00,00,00,00,00
                    HEX        00,00,22,22,00,00
                    HEX        00,02,33,33,20,00
                    HEX        00,23,33,33,32,00
                    HEX        00,23,33,33,32,00
                    HEX        00,23,33,33,32,00
                    HEX        00,23,33,33,32,00
                    HEX        00,02,33,33,20,00
                    HEX        00,00,22,22,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00

Aplha_Diskette      ENT
                    HEX        02,22,22,22,22,00    ; Diskette
                    HEX        21,12,22,21,21,20
                    HEX        21,12,22,21,21,12
                    HEX        21,12,22,22,21,12
                    HEX        21,11,11,11,11,12
                    HEX        21,11,11,11,11,12
                    HEX        21,22,22,22,22,12
                    HEX        21,21,11,11,12,12
                    HEX        21,21,11,11,12,12
                    HEX        21,21,11,11,12,12
                    HEX        21,21,11,11,12,12
                    HEX        02,22,22,22,22,20

Aplha_Diskette_Mask HEX        F0,00,00,00,00,FF    ; Diskette Mask
                    HEX        00,00,00,00,00,0F
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        00,00,00,00,00,00
                    HEX        F0,00,00,00,00,0F


***********************************************
*  Body + Faces for Intro Screens animation.  *
***********************************************

Intro_Body_Face_Tab ENT                             ; Global Reference
                    DA         Intro_Body_1         ; 44(4*11)x65  @74,65=48C5
                    DA         Intro_Body_2         ; 40(4*10)x65  @190,65=48FF
                    DA         Intro_Body_3         ; 188(47*4)x70 @56,130=715C
                    DA         Intro_Face_1,Intro_Face_2  ; 72(4*18)x98 @118,32=343B
                    DA         Intro_Face_3,Intro_Face_4
                    DA         Intro_Face_5,Intro_Face_6
                    
Intro_Body_1        HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,01,11,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,11,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,11,11,11,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,11,11,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,11,11,11,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,11,11,11,11,11,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,11,11,11,11,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,11,11,11,11,11,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,11,11,11,11,11,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,01,11,11,11,11,11,11,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,10,11,11,11,11,11,11,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,01,11,11,11,11,11,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,01,11,11,11,11,11,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,10,11,11,11,11,11,11
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,11,01,11,11,11,11,10
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,11,01,11,11,11,10,05
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,11,10,11,11,10,05,55
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,11,10,11,10,05,55,55
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,00,00,00,05,55,55,55
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,10,00,22,22,05,55,55,55,55
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,02,22,22,22,20,55,55,55,55
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,02,22,22,22,22,22,05,55,55,55
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,02,22,22,22,22,22,22,20,55,55,55
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,02,22,22,22,22,22,22,22,22,05,55,55
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,00,22,22,22,22,22,22,22,22,22,20,55,55
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,F0,22,22,22,22,22,22,22,22,22,22,20,55,55
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,02,22,22,22,22,22,22,22,22,22,22,22,05,55
                    HEX        FF,FF,FF,FF,FF,FF,FF,F0,22,22,22,22,22,22,22,22,22,22,22,22,20,55
                    HEX        FF,FF,FF,FF,FF,FF,FF,02,22,22,22,22,22,22,22,22,22,22,22,22,20,55
                    HEX        FF,FF,FF,FF,FF,FF,F0,22,22,22,22,22,22,22,22,22,22,22,22,22,22,05
                    HEX        FF,FF,FF,FF,FF,FF,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,05
                    HEX        FF,FF,FF,FF,FF,F0,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,05
                    HEX        FF,FF,FF,FF,FF,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20
                    HEX        FF,FF,FF,FF,F0,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20
                    HEX        FF,FF,FF,FF,F0,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20
                    HEX        FF,FF,FF,FF,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,FF,FF,F0,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,FF,FF,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,FF,F0,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,FF,F0,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,FF,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,FF,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,F0,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,F0,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        F0,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        F0,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        F0,00,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,02,00,00,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,F0,22,22,00,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,FF,02,22,04,00,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,FF,02,20,44,44,40,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,FF,F0,20,44,44,44,40,02,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,FF,FF,04,44,44,44,44,40,22,22,22,22,22,22,22,22,22,22,22,22,22
                    HEX        FF,FF,FF,04,44,44,44,44,44,00,22,22,22,22,22,22,22,22,22,22,22,22

Intro_Body_2        HEX        FF,FF,FF,00,00,00,00,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,00,00,22,22,22,22,22,00,00,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        00,22,22,22,22,22,22,22,22,22,00,0F,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        22,22,22,22,22,22,22,22,22,22,22,20,0F,FF,FF,FF,FF,FF,FF,FF
                    HEX        22,22,22,22,22,22,22,22,22,22,22,22,20,FF,FF,FF,FF,FF,FF,FF
                    HEX        22,22,22,22,22,22,22,22,22,22,22,22,22,00,FF,FF,FF,FF,FF,FF
                    HEX        22,22,22,22,22,22,22,22,22,22,22,22,22,22,0F,FF,FF,FF,FF,FF
                    HEX        22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,FF,FF,FF,FF,FF
                    HEX        22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,0F,FF,FF,FF,FF
                    HEX        02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,FF,FF,FF,FF
                    HEX        50,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,0F,FF,FF,FF
                    HEX        55,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,0F,FF,FF,FF
                    HEX        55,50,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,FF,FF,FF
                    HEX        55,55,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,0F,FF,FF
                    HEX        55,55,50,22,22,22,22,22,22,22,22,22,22,22,22,22,22,0F,FF,FF
                    HEX        55,55,55,00,22,22,22,22,22,22,22,22,22,22,22,22,22,20,FF,FF
                    HEX        55,55,55,55,02,22,22,22,22,22,22,22,22,22,22,22,22,20,FF,FF
                    HEX        55,55,55,55,50,22,22,22,22,22,22,22,22,22,22,22,22,20,FF,FF
                    HEX        55,55,55,55,55,02,22,22,22,22,22,22,22,22,22,22,22,22,0F,FF
                    HEX        55,55,55,55,55,50,22,22,22,22,22,22,22,22,22,22,22,22,0F,FF
                    HEX        55,55,55,55,55,55,02,22,22,22,22,22,22,22,22,22,22,22,0F,FF
                    HEX        55,55,55,55,55,55,50,22,22,22,22,22,20,22,22,22,22,22,0F,FF
                    HEX        55,55,55,55,55,55,55,02,22,22,22,22,20,22,22,22,22,22,0F,FF
                    HEX        55,55,55,55,55,55,55,50,22,22,22,22,20,22,22,22,22,22,0F,FF
                    HEX        55,55,55,55,55,55,55,55,02,22,22,22,20,22,22,22,22,22,0F,FF
                    HEX        55,55,55,55,55,55,55,55,50,22,22,22,20,22,22,22,22,22,0F,FF
                    HEX        55,55,55,55,55,55,55,55,55,02,22,22,02,22,22,22,22,22,0F,FF
                    HEX        55,55,55,55,55,55,55,55,55,02,22,22,02,22,22,22,22,22,0F,FF
                    HEX        55,55,55,55,55,55,55,55,55,50,22,22,02,22,22,22,22,22,0F,FF
                    HEX        55,55,55,55,55,55,55,55,55,50,22,22,02,22,22,22,22,22,0F,FF
                    HEX        55,55,55,55,55,55,55,55,55,50,22,22,02,22,22,22,22,20,FF,FF
                    HEX        55,55,55,55,55,55,55,55,55,50,22,20,22,22,22,22,22,20,FF,FF
                    HEX        55,55,55,55,55,55,55,55,55,50,22,20,22,22,22,22,22,20,FF,FF
                    HEX        55,55,55,55,55,55,55,55,55,50,22,20,22,22,22,22,22,20,FF,FF
                    HEX        55,55,55,55,55,55,55,55,55,50,22,20,22,22,22,22,22,0F,FF,FF
                    HEX        55,55,55,55,55,55,55,55,55,50,22,02,22,22,22,22,22,0F,FF,FF
                    HEX        55,55,55,55,55,55,55,55,55,50,22,02,22,22,22,22,22,0F,FF,FF
                    HEX        55,55,55,55,55,55,55,55,55,50,22,02,22,22,22,22,20,FF,FF,FF
                    HEX        55,55,55,55,55,55,55,55,55,50,20,22,22,22,22,22,20,FF,FF,FF
                    HEX        55,55,55,55,55,55,55,55,55,02,20,22,22,22,22,22,0F,FF,FF,FF
                    HEX        55,55,55,55,55,55,55,55,55,02,02,22,22,22,22,20,FF,FF,FF,FF
                    HEX        55,55,55,55,55,55,55,55,55,02,02,22,22,22,22,00,FF,FF,FF,FF
                    HEX        55,55,55,55,55,55,55,55,55,00,22,22,22,22,20,40,FF,FF,FF,FF
                    HEX        55,55,55,55,55,55,55,55,50,20,22,22,22,20,04,40,FF,FF,FF,FF
                    HEX        55,55,55,55,55,55,55,55,50,02,22,22,20,04,44,40,FF,FF,FF,FF
                    HEX        55,55,55,55,55,55,55,55,50,00,00,00,04,44,44,44,0F,FF,FF,FF
                    HEX        55,55,55,55,55,55,55,55,00,44,44,44,44,44,44,44,0F,FF,FF,FF
                    HEX        55,55,55,55,55,55,55,55,04,44,44,44,44,44,44,44,0F,FF,FF,FF
                    HEX        55,55,55,55,55,55,55,55,04,44,44,44,44,44,44,44,0F,FF,FF,FF
                    HEX        55,55,55,55,55,55,55,55,04,44,44,44,44,44,44,44,00,0F,FF,FF
                    HEX        55,55,55,55,55,55,55,50,44,44,44,44,44,44,44,44,04,40,FF,FF
                    HEX        55,55,55,55,55,55,55,50,44,44,44,44,44,44,44,44,04,44,0F,FF
                    HEX        55,55,55,55,55,55,55,50,44,44,44,44,44,44,44,40,44,44,40,FF
                    HEX        55,55,55,55,55,55,55,04,44,44,44,44,44,44,44,04,44,44,40,FF
                    HEX        55,55,55,55,55,55,55,04,44,44,44,44,44,44,44,44,44,44,40,FF
                    HEX        55,55,55,55,55,55,55,04,44,44,44,44,44,44,44,44,44,44,44,0F
                    HEX        55,55,55,55,55,55,50,44,44,44,44,44,44,40,44,44,44,44,44,0F
                    HEX        55,55,55,55,55,55,50,44,00,44,44,44,44,04,44,44,44,44,44,0F
                    HEX        55,55,55,55,55,55,50,44,44,00,04,04,44,44,44,44,44,44,44,0F
                    HEX        55,55,55,55,55,55,50,44,44,44,44,44,44,44,44,44,44,44,44,0F
                    HEX        55,55,55,55,55,55,04,44,44,44,44,44,44,44,44,44,44,44,44,0F
                    HEX        55,55,55,55,55,55,04,44,44,44,44,44,44,44,44,44,44,44,44,0F
                    HEX        55,55,55,55,55,55,04,44,44,44,44,44,44,44,44,44,44,44,44,0F
                    HEX        55,55,55,55,55,50,44,44,44,44,44,44,44,44,44,44,44,44,44,0F
                    HEX        55,55,55,55,55,50,44,44,44,44,44,44,44,44,44,44,44,44,44,0F

Intro_Body_3        HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,00,22,22,22,22,22,22,22,22,22,22,22,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,02,22,22,22,22,22,22,22,22,22,22,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,40,02,22,22,22,22,22,22,22,22,22,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,04,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,40,22,22,22,22,22,22,22,22,20,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,04,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,02,22,22,22,22,22,22,22,20,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,04,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,40,02,22,22,22,22,22,22,20,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,40,22,22,22,22,22,22,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,02,22,22,22,22,22,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,40,22,22,22,22,20,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,02,22,22,22,20,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,22,22,22,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,04,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,02,22,22,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,22,20,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,04,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,22,20,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,0F,F0,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,02,00,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,FF,F0,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,00,FF,00,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,0F,FF,FF,04,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,00,FF,FF,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,F0,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,0F,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,0F,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,F0,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,F0,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,F0,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,0F,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,F0,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,0F,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,0F,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,00,00,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,0F,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,00,04,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,0F,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,40,04,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,0F,FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,40,00,04,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,FF,FF,FF,FF,FF,FF,FF,FF,FF,04,44,40,03,33,30,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,04,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,FF,FF,FF,FF,FF,FF,FF,FF,FF,04,44,03,33,33,33,04,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,FF,FF,FF,FF,FF,FF,FF,FF,FF,04,40,33,33,33,33,30,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,00,FF,FF,FF,FF,FF,FF,00,00,00,04,03,33,33,33,33,33,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,20,FF,FF,FF,FF,FF,F0,02,22,22,00,33,33,33,33,33,33,30,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,02,0F,FF,FF,FF,FF,FF,02,22,22,22,20,33,33,33,33,33,33,30,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,F0,44,44,44,44,40,00,00,04,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,02,0F,FF,FF,FF,FF,F0,22,22,22,22,22,03,33,33,33,33,33,30,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,F0,44,44,44,00,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,00,22,0F,FF,FF,FF,FF,F0,22,22,22,22,22,20,33,33,33,33,33,30,FF,FF,FF,FF,FF,FF,F0,00,FF,FF
                    HEX        FF,04,44,44,00,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,22,22,0F,FF,FF,FF,FF,F0,22,22,22,22,22,20,33,33,33,33,33,30,FF,FF,FF,FF,FF,FF,02,22,00,0F
                    HEX        FF,04,44,40,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,30,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,02,22,22,0F,FF,FF,FF,FF,F0,22,22,22,22,22,03,33,33,33,33,33,33,0F,FF,FF,FF,FF,F0,22,22,22,20
                    HEX        F0,44,44,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,02,00,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,00,22,22,22,0F,FF,FF,FF,FF,F0,22,22,22,22,22,03,33,33,33,33,33,33,0F,FF,FF,FF,FF,02,22,22,22,20
                    HEX        F0,44,40,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,02,22,00,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,00,22,22,22,22,20,FF,FF,FF,FF,FF,02,22,22,22,20,20,33,33,33,33,33,33,30,FF,FF,FF,FF,02,22,22,22,0F
                    HEX        F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,02,22,22,00,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,00,22,22,22,22,22,20,FF,FF,FF,FF,FF,F0,22,22,20,02,20,33,33,33,33,33,33,33,0F,FF,FF,F0,22,22,22,22,0F
                    HEX        FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,02,22,22,22,00,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,00,22,22,22,22,22,22,20,FF,FF,FF,FF,FF,FF,00,00,02,22,22,03,33,33,33,33,33,33,30,FF,FF,02,22,22,22,20,FF
                    HEX        FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,22,22,22,22,20,00,55,55,55,55,55,55,55,55,55,55,55,55,55,00,02,22,22,22,22,22,22,22,20,0F,FF,FF,FF,FF,FF,FF,F0,22,22,22,20,33,33,33,33,33,33,30,FF,FF,02,22,22,22,0F,FF
                    HEX        FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,22,22,22,22,22,22,00,00,55,55,55,55,55,55,55,55,55,00,00,22,22,22,22,22,22,22,22,22,20,50,FF,FF,FF,FF,FF,FF,F0,22,22,22,20,33,33,33,33,33,33,33,0F,F0,22,22,22,20,FF,FF
                    HEX        FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,22,22,22,22,22,22,22,22,00,00,00,00,00,00,00,00,00,22,22,22,22,22,22,22,22,22,22,22,05,55,00,FF,FF,FF,FF,F0,03,02,22,22,20,00,00,03,33,33,33,33,0F,02,22,22,22,20,FF,FF
                    HEX        FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,00,55,55,55,0F,FF,FF,FF,03,33,02,22,22,22,03,33,30,33,33,33,33,00,02,22,22,22,0F,FF,FF
                    HEX        FF,FF,FF,00,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,02,00,55,55,50,FF,FF,F0,33,33,02,22,22,22,20,33,33,03,33,33,33,30,22,22,22,20,FF,FF,FF
                    HEX        FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,22,22,00,05,55,0F,FF,F0,33,33,30,22,22,22,22,03,30,33,33,33,33,30,22,22,22,0F,FF,FF,FF
                    HEX        FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,05,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,00,22,22,22,20,00,50,FF,03,33,33,30,22,22,22,22,03,03,33,33,33,33,30,02,22,22,0F,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,00,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,55,50,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,05,55,00,02,22,22,22,00,FF,03,33,33,30,02,22,22,22,20,03,33,33,33,33,33,30,22,20,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,05,55,55,50,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,00,55,55,55,50,00,22,22,22,00,03,33,33,03,02,22,22,22,22,03,33,33,33,33,33,30,22,0F,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,05,55,55,55,55,50,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,00,55,55,55,55,55,55,00,02,22,22,03,33,33,03,02,22,22,22,20,33,33,33,33,33,33,33,00,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,00,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,F0,55,55,55,55,55,55,50,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55,55,50,00,22,20,33,33,30,30,22,22,22,03,33,33,33,33,33,33,33,0F,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,FF,FF,FF,05,55,55,55,55,55,55,55,50,00,02,22,22,22,22,22,22,22,22,22,22,22,22,20,00,05,55,55,55,55,55,55,55,55,55,55,55,02,20,33,33,30,00,22,22,00,33,33,33,33,33,33,33,33,0F,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,F0,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,F0,55,55,55,55,55,55,55,55,55,55,50,00,02,22,22,22,22,22,22,22,22,20,00,05,55,55,55,55,55,55,55,55,55,55,55,55,55,50,02,03,33,30,30,00,00,33,33,33,33,33,33,33,33,33,0F,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,FF,FF,FF,05,55,55,55,55,55,55,55,55,55,55,55,55,50,00,00,00,00,00,00,00,00,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,03,33,03,33,33,33,33,33,33,33,33,30,03,33,33,30,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,F0,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,00,33,03,33,33,33,33,33,33,33,30,03,33,33,33,30,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,0F,FF,FF,FF,FF,FF,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,00,00,33,33,33,33,33,33,30,03,33,33,33,33,33,0F,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,F0,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,03,33,33,33,33,30,03,33,33,33,33,33,33,0F,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,40,FF,FF,FF,FF,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,33,33,33,30,03,33,33,33,33,33,33,33,0F,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,00,FF,FF,F0,55,55,55,55,55,50,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,03,33,30,03,33,33,33,33,33,33,33,33,0F,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,00,FF,05,55,55,55,55,50,03,30,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,33,0F,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,00,05,55,55,55,50,03,33,33,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,30,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,05,55,55,00,03,33,33,33,30,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,00,03,33,33,33,33,33,33,00,00,0F,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,44,44,44,40,00,00,00,00,55,00,00,33,33,33,33,33,30,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,03,33,33,33,33,00,20,0F,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,44,44,40,03,33,33,33,30,00,33,33,33,33,33,33,33,33,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,03,33,33,33,00,22,22,0F,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,04,44,44,44,44,44,44,44,40,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,33,33,00,22,22,22,00,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,44,44,44,44,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,03,33,33,30,05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,00,00,22,22,22,22,20,FF,FF,FF,FF,FF,8F

Intro_Face_1        HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,00,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,01,01,11,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,01,11,01,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,01,11,10,11,11,11,10,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,01,11,11,10,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,11,11,01,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,11,11,01,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,F0,01,11,11,11,10,11,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,01,11,11,11,11,01,11,11,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,F0,11,11,11,11,11,01,11,11,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,01,11,11,11,11,11,01,11,11,11,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,F0,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,01,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,F0,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,01,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,F0,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,01,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,01,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,F0,11,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,F0,11,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,01,11,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,01,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,01,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,01,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,01,00,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,10,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,01,11,00,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,10,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,01,11,11,00,01,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,10,01,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,01,11,11,11,10,00,01,10,11,11,11,11,11,11,11,11,11,11,11,10,01,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,F0,11,11,11,11,11,11,10,00,00,01,11,11,11,11,11,11,11,00,00,01,11,11,11,11,00,00,00,0F,FF,FF,FF,FF,FF,FF
                    HEX        FF,F0,11,11,11,11,11,11,11,11,11,10,00,00,00,00,00,00,00,11,11,11,11,11,11,11,01,11,11,10,00,00,00,FF,FF,FF
                    HEX        FF,F0,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,0F,FF,FF
                    HEX        FF,F0,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,10,FF,FF
                    HEX        FF,F0,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,11,0F,FF
                    HEX        FF,F0,01,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,0F,FF
                    HEX        FF,F0,40,01,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,00,11,11,11,11,11,11,11,11,10,FF
                    HEX        FF,F0,44,40,01,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,04,40,11,11,11,11,11,11,11,11,11,0F
                    HEX        FF,F0,44,44,00,01,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,00,04,44,40,11,11,11,11,11,11,11,11,11,10
                    HEX        FF,00,44,44,03,30,00,01,11,11,11,11,11,11,11,11,11,11,11,11,10,00,44,44,44,40,11,11,11,11,11,11,11,11,11,10

                    HEX        00,04,44,40,33,33,33,30,00,00,11,11,11,11,11,11,11,10,00,00,00,44,44,44,44,44,01,11,11,11,11,11,11,11,10,02
                    HEX        11,04,44,40,33,33,33,33,33,33,00,00,00,00,00,00,00,03,33,33,30,44,44,44,40,04,01,11,11,11,11,11,11,11,02,22
                    HEX        11,04,44,40,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,44,44,40,00,00,00,00,00,00,00,00,50,22
                    HEX        11,00,00,00,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,44,44,44,02,22,22,20,55,55,55,55,55,02
                    HEX        11,04,44,00,00,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,44,44,44,40,22,22,22,05,55,55,55,55,50
                    HEX        11,04,44,03,30,00,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,44,44,40,22,22,22,05,55,55,55,55,55
                    HEX        11,04,44,03,33,30,00,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,44,44,44,02,22,22,05,55,55,55,55,55
                    HEX        11,04,40,33,33,01,10,03,33,33,33,03,33,33,33,33,33,33,33,33,33,33,04,44,44,44,44,02,22,22,20,55,55,55,55,55
                    HEX        11,04,40,03,30,01,11,00,33,33,33,03,33,33,33,33,33,33,33,33,33,33,04,44,44,44,44,02,22,22,20,55,55,55,55,55
                    HEX        11,04,40,30,00,01,11,10,03,33,33,03,33,33,33,00,00,00,00,00,03,33,30,44,44,44,44,00,22,22,20,55,55,55,55,55
                    HEX        10,44,03,33,33,00,00,00,03,33,33,03,30,00,00,00,00,00,00,33,33,33,30,44,44,44,44,00,22,22,22,05,55,55,55,55
                    HEX        10,44,03,33,33,33,33,33,33,33,33,00,03,00,01,11,11,11,03,33,33,33,33,04,44,44,40,44,02,22,22,00,05,55,55,55
                    HEX        10,44,03,33,33,33,33,33,33,33,30,33,33,30,01,11,11,10,33,33,33,33,33,04,44,44,40,44,02,22,22,22,20,55,55,55
                    HEX        10,44,03,33,33,33,33,33,33,33,30,33,33,33,30,00,00,03,33,33,33,33,33,04,44,44,04,00,40,22,22,22,20,55,55,55
                    HEX        10,44,03,33,33,33,33,33,30,00,03,33,33,30,00,03,33,33,33,33,33,33,33,30,44,40,44,44,00,22,22,22,22,05,55,55
                    HEX        10,44,03,33,33,33,33,00,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,04,44,44,40,02,22,22,22,05,55,55
                    HEX        04,44,40,33,33,33,30,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,04,44,44,44,02,22,22,22,05,55,55
                    HEX        04,44,40,33,33,33,03,33,33,33,33,33,03,33,33,33,33,30,33,33,33,33,33,04,40,44,44,44,44,02,22,22,22,20,55,55
                    HEX        00,44,44,03,33,33,03,33,33,33,33,33,30,33,33,33,33,33,03,33,33,33,30,44,40,44,44,44,44,40,22,22,22,20,55,55
                    HEX        50,44,44,03,33,33,03,33,33,33,33,33,33,03,33,33,33,33,33,33,33,33,30,44,40,44,44,44,44,44,02,22,22,22,05,55
                    HEX        55,04,44,03,33,33,30,33,33,33,33,33,33,03,33,33,33,33,33,33,33,33,04,44,04,44,44,44,44,44,02,22,22,22,05,55
                    HEX        55,04,44,40,33,33,33,33,33,33,33,33,30,33,33,33,33,33,33,33,33,33,04,44,04,44,44,44,44,44,02,22,22,22,05,55
                    HEX        55,50,44,40,33,33,33,33,00,03,00,00,03,33,33,33,33,33,33,33,33,30,44,44,04,44,44,44,44,44,40,22,22,22,20,55
                    HEX        55,50,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,40,44,44,44,44,44,44,40,22,22,22,20,55
                    HEX        55,55,04,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,40,44,44,44,44,44,44,40,22,22,22,20,55
                    HEX        55,55,04,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,04,44,44,44,44,44,44,40,22,22,22,20,55
                    HEX        55,55,50,44,40,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,04,44,44,44,44,44,44,40,22,22,22,20,55
                    HEX        55,55,50,44,40,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,04,44,44,44,44,44,44,40,22,22,22,22,05
                    HEX        55,55,55,04,40,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,40,44,44,44,44,44,44,44,40,22,22,22,22,05
                    HEX        55,55,55,04,40,33,33,33,00,00,00,03,33,33,33,33,33,33,33,33,04,44,40,44,44,44,44,44,44,44,40,22,22,22,22,05
                    HEX        55,55,55,50,40,33,33,00,33,33,33,30,03,33,33,33,33,33,33,33,04,44,40,44,44,44,44,44,44,44,04,02,22,22,22,05
                    HEX        55,55,55,50,40,33,30,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,04,44,44,44,44,44,44,44,04,02,22,22,22,05
                    HEX        55,55,55,55,00,33,30,03,33,33,33,33,33,33,33,33,33,33,33,30,44,44,04,44,44,40,44,44,44,44,04,02,22,22,22,05
                    HEX        55,55,55,55,00,33,33,30,00,03,33,33,33,33,33,33,33,33,33,04,44,40,44,44,40,04,44,44,44,40,44,02,22,22,22,05
                    HEX        55,55,55,55,50,33,33,33,33,30,00,00,33,33,33,33,33,33,33,04,44,40,44,44,04,44,44,44,44,04,44,02,22,22,22,05
                    HEX        55,55,55,55,50,03,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,04,44,40,44,44,44,44,44,04,44,02,22,22,22,05
                    HEX        55,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,33,33,30,44,40,44,40,04,44,44,44,44,40,44,44,02,22,22,22,05
                    HEX        55,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,33,33,30,44,04,44,04,44,44,44,44,44,04,44,40,22,22,22,22,05
                    HEX        55,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,33,33,04,44,04,00,44,44,44,44,44,40,44,44,40,22,22,22,22,05
                    HEX        05,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,33,33,04,40,40,44,44,44,44,44,40,04,44,44,40,22,22,22,22,05
                    HEX        05,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,33,30,44,40,04,44,44,44,44,44,04,44,44,44,00,22,22,22,22,05
                    HEX        05,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,33,30,44,04,44,44,44,44,44,00,44,44,44,40,02,22,22,22,20,55
                    HEX        05,55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,33,04,44,44,44,44,44,44,00,44,44,44,44,00,22,22,22,22,20,55
                    HEX        05,55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,33,04,44,44,44,44,44,00,44,44,44,44,44,02,22,22,22,22,05,55
                    HEX        20,55,55,55,55,55,00,33,33,33,33,33,33,33,33,33,00,44,44,44,44,40,00,44,44,44,44,44,00,22,22,22,22,22,05,55
                    HEX        20,55,55,55,55,55,55,00,03,33,33,33,33,33,30,00,44,44,44,44,00,04,44,44,44,44,44,40,22,22,22,22,22,20,55,55
                    HEX        20,55,55,55,55,55,55,55,50,00,00,00,00,00,04,44,44,44,00,00,44,44,44,44,44,44,40,02,22,22,22,22,22,05,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,02,22,22,22,00,00,00,00,44,44,44,44,44,44,44,40,02,22,22,22,22,22,22,05,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,02,22,22,22,22,20,44,44,44,44,44,44,44,44,40,02,22,22,22,22,22,22,20,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,50,22,22,22,22,22,00,44,44,44,44,44,44,00,02,22,22,22,22,22,22,20,05,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,02,22,22,22,22,22,00,00,00,00,00,00,22,22,22,22,22,22,22,22,05,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,50,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,50,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,50,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,02,22,22,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,50,02,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,55,50,00,22,22,22,22,22,22,22,20,00,05,55,55,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,00,00,02,22,00,00,00,05,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,00,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
 
Intro_Face_2        HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,10,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,10,11,10,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,01,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,01,11,11,11,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,11,10,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,10,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,11,01,11,11,11,11,11,11,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,00,11,11,11,01,11,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,F0,11,11,11,10,11,11,11,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,01,11,11,11,10,11,11,11,11,11,11,11,11,01,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,F0,11,11,11,11,10,11,11,11,11,11,11,11,11,10,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,01,11,11,11,11,01,11,11,11,11,11,11,11,11,11,01,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,F0,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,01,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,01,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,10,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,F0,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,01,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,01,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,01,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,F0,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,10,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,F0,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,11,10,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,01,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,11,11,01,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,F0,11,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,11,11,10,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,F0,11,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,11,11,10,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,01,11,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,11,11,11,01,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,01,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,01,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,01,10,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,00,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,F0,11,11,00,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,00,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,F0,11,11,11,00,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,00,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,F0,11,11,11,11,00,01,11,10,11,11,11,11,11,11,11,11,11,10,00,11,11,11,11,10,00,00,00,0F,FF,FF,FF,FF,FF,FF
                    HEX        FF,F0,11,11,11,11,11,10,00,00,11,11,11,11,11,11,10,00,00,01,11,11,11,11,11,10,11,11,11,10,00,00,00,FF,FF,FF
                    HEX        FF,F0,11,11,11,11,11,11,11,10,00,00,00,00,00,00,01,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,0F,FF,FF
                    HEX        FF,F0,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,10,FF,FF
                    HEX        FF,F0,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,0F,FF
                    HEX        FF,F0,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,0F,FF
                    HEX        FF,F0,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,10,FF
                    HEX        FF,F0,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,00,11,11,11,11,11,11,11,11,11,0F
                    HEX        FF,F0,00,01,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,00,40,11,11,11,11,11,11,11,11,11,10
                    HEX        FF,00,33,30,01,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,00,44,40,11,11,11,11,11,11,11,11,11,10

                    HEX        00,10,33,33,30,00,01,11,11,11,11,11,11,11,11,11,11,11,11,11,11,00,04,44,44,40,11,11,11,11,11,11,11,11,10,02
                    HEX        11,03,33,33,33,33,30,00,00,01,11,11,11,11,11,11,11,11,00,00,00,44,44,44,44,00,00,11,11,11,11,11,11,11,02,22
                    HEX        10,03,00,03,33,33,33,33,33,30,00,00,00,00,00,00,00,00,33,33,30,44,44,44,40,44,44,00,00,00,00,00,00,00,50,22
                    HEX        10,00,00,00,00,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,44,40,44,44,02,22,20,55,55,55,55,55,02
                    HEX        10,03,33,30,00,00,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,44,40,44,44,02,22,22,05,55,55,55,55,50
                    HEX        10,03,33,33,30,00,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,44,04,44,44,02,22,22,05,55,55,55,55,55
                    HEX        10,40,33,33,33,10,00,33,33,33,33,30,33,33,33,33,33,33,33,33,33,30,44,44,04,44,44,02,22,22,05,55,55,55,55,55
                    HEX        10,40,33,33,30,01,10,03,33,33,33,30,33,33,33,33,33,33,33,33,33,30,44,44,04,44,44,02,22,22,20,55,55,55,55,55
                    HEX        11,00,33,33,00,01,11,10,33,33,33,30,33,33,33,33,33,33,33,33,33,30,44,44,04,44,44,02,22,22,20,55,55,55,55,55
                    HEX        11,00,33,33,30,01,11,11,03,33,33,03,33,33,33,33,33,00,00,00,03,33,04,40,44,44,40,40,22,22,20,55,55,55,55,55
                    HEX        11,00,33,33,33,30,00,00,00,30,03,03,33,33,00,00,00,11,03,33,33,33,04,40,44,44,40,44,02,22,22,05,55,55,55,55
                    HEX        11,00,33,33,33,33,33,33,33,03,33,03,33,00,00,11,11,10,33,33,33,33,30,40,44,44,40,44,40,22,22,00,05,55,55,55
                    HEX        11,04,03,33,33,33,33,33,30,33,30,00,33,33,00,11,11,03,33,33,33,33,30,44,44,44,40,44,40,22,22,22,20,55,55,55
                    HEX        11,04,03,33,33,33,33,33,03,33,03,33,33,33,30,00,00,33,33,33,33,33,30,44,44,44,04,44,44,02,22,22,20,55,55,55
                    HEX        11,04,40,33,33,33,33,30,33,30,33,33,33,03,33,33,33,33,33,33,33,33,30,44,44,44,04,44,44,02,22,22,22,05,55,55
                    HEX        11,00,40,33,33,33,33,33,30,03,33,33,33,30,03,33,33,33,33,33,33,33,30,44,44,40,44,44,44,40,22,22,22,05,55,55
                    HEX        11,10,44,03,33,33,33,03,33,33,33,30,33,33,30,03,33,33,33,33,33,33,30,44,44,40,44,44,44,40,22,22,22,05,55,55
                    HEX        10,02,04,03,33,33,03,33,33,33,33,33,03,33,33,30,03,33,33,33,33,33,30,44,44,04,44,44,44,40,22,22,22,20,55,55
                    HEX        05,02,20,03,33,30,33,33,33,33,33,33,30,33,33,33,30,00,00,33,33,33,30,44,44,04,44,44,44,44,02,22,22,20,55,55
                    HEX        55,02,20,03,33,30,33,33,33,33,33,33,30,33,33,33,33,33,33,33,33,33,00,44,44,04,44,44,44,44,02,22,22,22,05,55
                    HEX        55,50,22,00,33,30,33,33,33,33,33,33,30,33,33,33,33,33,33,33,33,30,44,44,40,44,44,44,44,44,02,22,22,22,05,55
                    HEX        55,50,22,20,33,33,03,33,33,33,33,00,03,33,33,33,33,33,33,33,33,30,44,44,40,44,44,44,44,44,02,22,22,22,05,55
                    HEX        55,50,22,20,33,33,30,33,33,33,00,33,33,33,33,33,33,33,33,33,33,04,44,44,40,44,44,44,44,44,02,22,22,22,20,55
                    HEX        55,50,22,20,03,33,33,00,33,30,33,33,33,33,33,33,33,33,33,33,33,04,44,44,04,44,44,44,44,44,40,22,22,22,20,55
                    HEX        55,55,02,20,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,44,04,44,44,44,44,44,40,22,22,22,20,55
                    HEX        55,55,02,20,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,44,04,44,44,44,44,44,40,22,22,22,20,55
                    HEX        55,55,50,22,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,44,40,44,44,44,44,44,44,40,22,22,22,20,55
                    HEX        55,55,50,22,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,44,40,44,44,44,44,44,44,40,22,22,22,22,05
                    HEX        55,55,55,02,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,44,04,44,44,44,44,44,44,40,22,22,22,22,05
                    HEX        55,55,55,50,03,33,33,33,33,30,00,33,33,33,33,33,33,33,33,30,44,44,44,04,44,44,44,44,44,44,40,22,22,22,22,05
                    HEX        55,55,55,50,03,33,33,33,00,03,33,00,03,33,33,33,33,33,33,30,44,44,40,44,44,44,44,44,44,44,40,02,22,22,22,05
                    HEX        55,55,55,55,03,33,33,00,33,33,33,33,30,03,33,33,33,33,33,04,44,44,04,44,44,44,44,40,44,44,04,02,22,22,22,05
                    HEX        55,55,55,55,50,33,00,33,33,33,33,33,33,33,33,33,33,33,33,04,44,44,04,44,44,44,44,04,44,44,04,02,22,22,22,05
                    HEX        55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,04,40,44,44,44,44,40,44,44,44,04,02,22,22,22,05
                    HEX        55,55,55,55,50,33,33,00,33,33,30,33,33,33,33,33,33,33,30,44,40,04,44,44,44,40,04,44,44,44,04,02,22,22,22,05
                    HEX        55,55,55,55,50,33,33,33,00,00,03,33,33,33,33,33,33,33,30,44,44,44,44,44,44,04,44,44,44,44,04,02,22,22,22,05
                    HEX        55,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,44,44,44,00,44,44,44,44,40,44,02,22,22,22,05
                    HEX        55,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,33,33,04,44,44,44,44,40,44,44,44,44,44,40,40,22,22,22,22,05
                    HEX        55,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,33,33,04,44,44,44,40,04,44,44,44,44,44,04,40,22,22,22,22,05
                    HEX        05,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,33,33,04,44,44,44,04,44,44,44,44,44,44,04,40,22,22,22,22,05
                    HEX        05,55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,33,30,44,44,44,00,44,44,44,44,44,44,40,44,00,22,22,22,22,05
                    HEX        05,55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,33,30,44,40,00,44,44,44,44,44,44,44,04,40,02,22,22,22,20,55
                    HEX        05,55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,33,04,00,04,44,44,44,44,44,44,44,40,44,00,22,22,22,22,20,55
                    HEX        05,55,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,04,44,44,44,44,44,44,44,44,44,04,44,02,22,22,22,22,05,55
                    HEX        20,55,55,55,55,55,03,33,33,33,33,33,33,33,33,33,30,44,44,44,44,44,44,44,44,44,40,44,00,22,22,22,22,22,05,55
                    HEX        20,55,55,55,55,55,50,33,33,33,33,33,33,33,33,33,04,44,44,44,44,44,44,44,44,40,04,40,22,22,22,22,22,20,55,55
                    HEX        20,55,55,55,55,55,55,00,33,33,33,33,33,33,33,00,44,44,44,44,44,44,44,44,40,04,40,02,22,22,22,22,22,05,55,55
                    HEX        20,55,55,55,55,55,55,55,03,33,33,33,33,33,30,44,44,44,44,44,44,44,44,40,04,40,02,22,22,22,22,22,22,05,55,55
                    HEX        20,55,55,55,55,55,55,55,50,03,33,33,33,00,04,44,44,44,44,44,44,44,40,04,40,02,22,22,22,22,22,22,20,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,00,00,00,00,44,44,44,44,44,44,44,44,00,04,00,02,22,22,22,22,22,22,20,05,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,02,22,00,04,44,44,44,40,00,00,00,00,22,22,22,22,22,22,22,22,05,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,50,22,22,20,00,00,00,02,22,22,22,22,22,22,22,22,22,22,22,20,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,50,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,02,22,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,50,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,02,22,22,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,50,02,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,55,50,00,22,22,22,22,22,22,22,20,00,05,55,55,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,00,00,02,22,00,00,00,05,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,00,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55

Intro_Face_3        HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,10,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,01,11,10,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,11,01,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,11,10,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,10,11,11,11,11,10,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,01,11,11,01,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,11,11,01,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,11,11,01,11,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,F0,01,11,11,11,10,11,11,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,01,11,11,11,11,10,11,11,11,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,F0,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,01,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,F0,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,01,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,F0,11,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,01,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,01,11,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,01,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,F0,11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,10,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,01,11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,11,01,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,F0,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,01,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,F0,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,01,10,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,01,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,10,10,00,00,0F,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,01,10,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,10,11,01,11,10,00,00,00,FF,FF,FF
                    HEX        FF,FF,FF,01,11,00,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,10,11,01,11,11,11,11,11,0F,FF,FF
                    HEX        FF,FF,F0,11,11,11,00,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,00,11,10,11,11,11,11,11,10,FF,FF
                    HEX        FF,FF,F0,11,11,11,11,00,01,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,10,01,11,10,11,11,11,11,11,11,0F,FF
                    HEX        FF,FF,F0,11,11,11,11,11,10,00,11,11,11,01,11,11,11,11,11,11,11,11,11,00,01,11,11,10,11,11,11,11,11,11,0F,FF
                    HEX        FF,FF,F0,11,11,11,11,11,11,11,00,00,01,01,11,11,11,11,11,11,10,00,00,11,11,11,11,10,11,11,11,11,11,11,10,FF
                    HEX        FF,FF,F0,11,11,11,11,11,11,11,11,11,10,00,00,00,00,00,00,00,01,11,11,11,11,11,11,10,11,11,11,11,11,11,11,0F
                    HEX        FF,F0,00,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,10
                    HEX        FF,01,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,11,10

                    HEX        00,11,04,01,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,10,02
                    HEX        11,11,04,40,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,02,22
                    HEX        11,11,04,44,00,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,00,00,00,00,00,00,00,50,22
                    HEX        11,11,04,44,44,01,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,02,22,20,55,55,55,55,55,02
                    HEX        11,11,04,44,44,00,01,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,02,22,22,05,55,55,55,55,50
                    HEX        11,11,04,44,40,33,30,00,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,00,04,40,22,22,05,55,55,55,55,55
                    HEX        11,11,04,44,40,33,33,33,00,01,11,11,11,11,11,11,11,11,11,11,11,11,11,10,00,44,44,00,22,22,05,55,55,55,55,55
                    HEX        11,11,04,44,40,33,33,33,33,30,00,00,01,11,11,11,11,11,11,11,11,00,00,00,44,44,40,44,02,22,20,55,55,55,55,55
                    HEX        11,11,04,44,00,00,03,33,33,33,33,33,30,00,00,00,00,00,00,00,00,33,33,30,44,44,40,44,02,22,20,55,55,55,55,55
                    HEX        11,11,04,44,03,30,00,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,04,44,02,22,20,55,55,55,55,55
                    HEX        11,10,44,44,03,30,10,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,04,44,40,22,22,05,55,55,55,55
                    HEX        11,10,44,40,33,30,11,00,33,33,33,33,03,33,33,33,33,33,33,33,33,33,33,33,04,44,04,44,40,22,22,00,05,55,55,55
                    HEX        11,10,44,40,33,33,01,10,03,33,33,33,03,33,33,33,33,33,33,33,33,33,33,33,04,44,04,44,40,02,22,22,20,55,55,55
                    HEX        11,10,44,40,33,33,00,11,00,33,33,33,03,33,33,33,33,33,33,33,33,33,33,33,04,44,04,44,40,40,22,22,20,55,55,55
                    HEX        11,10,44,03,33,33,00,01,11,00,33,33,03,33,33,33,33,33,33,33,33,33,33,33,30,44,04,44,40,40,22,22,22,05,55,55
                    HEX        11,10,44,03,33,33,30,01,11,11,00,33,03,03,33,33,00,00,00,03,33,33,33,33,30,44,04,44,04,44,02,22,22,05,55,55
                    HEX        11,10,04,03,33,33,33,30,11,00,00,03,03,30,00,00,11,11,11,10,00,00,00,33,30,44,04,44,04,44,02,22,22,05,55,55
                    HEX        10,02,04,03,33,33,33,33,00,33,33,00,33,33,30,00,11,11,11,11,03,33,33,33,30,44,04,44,04,44,02,22,22,20,55,55
                    HEX        05,02,20,03,33,33,33,00,33,33,33,03,33,33,33,01,11,11,11,00,33,33,33,33,30,44,04,44,04,44,40,22,22,20,55,55
                    HEX        55,02,20,40,33,33,33,03,33,33,30,33,33,33,33,30,00,00,00,33,33,33,33,33,30,44,04,40,44,44,40,22,22,22,05,55
                    HEX        55,50,20,40,33,33,33,33,33,33,03,33,33,33,33,33,30,33,33,33,33,33,33,33,30,44,04,40,44,44,40,22,22,22,05,55
                    HEX        55,50,22,00,33,33,33,33,30,00,33,33,33,33,33,33,33,00,03,33,33,33,33,33,30,44,40,40,44,44,40,22,22,22,05,55
                    HEX        55,50,22,00,33,33,33,33,03,33,33,33,33,33,03,33,33,33,33,33,33,33,33,33,30,44,44,04,44,44,44,02,22,22,20,55
                    HEX        55,50,22,20,33,33,33,30,33,33,33,33,33,33,03,33,33,33,33,33,33,33,33,33,30,44,44,04,44,44,44,02,22,22,20,55
                    HEX        55,55,02,20,03,33,33,30,33,33,33,33,33,30,30,33,33,33,33,33,33,33,33,33,04,44,44,04,44,44,44,02,22,22,20,55
                    HEX        55,55,02,22,03,33,33,30,33,33,33,33,33,30,33,03,33,33,33,33,33,33,33,33,04,44,40,44,44,44,44,02,22,22,20,55
                    HEX        55,55,50,22,00,33,33,33,03,33,33,33,33,30,00,33,33,33,33,33,33,33,33,30,44,44,40,44,44,44,44,02,22,22,20,55
                    HEX        55,55,50,22,20,33,33,33,30,33,33,33,30,03,33,33,33,33,33,33,33,33,33,30,44,44,04,44,44,44,44,02,22,22,22,05
                    HEX        55,55,55,02,22,03,33,33,33,33,00,00,03,33,33,33,33,33,33,33,33,33,33,04,44,44,04,44,44,44,44,40,22,22,22,05
                    HEX        55,55,55,50,22,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,40,44,44,44,44,44,40,22,22,22,05
                    HEX        55,55,55,50,22,20,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,04,44,44,44,44,44,40,22,22,22,05
                    HEX        55,55,55,55,02,20,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,40,04,44,44,44,44,44,00,22,22,22,05
                    HEX        55,55,55,55,50,22,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,00,44,44,44,44,44,44,02,22,22,22,05
                    HEX        55,55,55,55,55,02,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,40,04,44,44,44,44,44,44,02,22,22,22,05
                    HEX        55,55,55,55,55,02,03,33,33,33,30,00,00,03,33,33,33,33,33,33,33,30,44,04,04,44,44,44,40,44,44,02,22,22,22,05
                    HEX        55,55,55,55,55,50,03,33,33,30,03,33,33,30,03,33,33,33,33,33,33,30,44,44,04,44,44,44,04,44,44,02,22,22,22,05
                    HEX        55,55,55,55,55,55,03,33,33,03,33,33,33,33,30,33,33,33,33,33,33,30,44,40,44,44,44,40,44,44,40,02,22,22,22,05
                    HEX        55,55,55,55,55,55,03,33,30,33,33,33,33,33,33,03,33,33,33,33,33,04,44,40,44,44,44,04,44,44,40,22,22,22,22,05
                    HEX        55,55,55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,04,44,44,40,44,44,44,40,22,22,22,22,05
                    HEX        05,55,55,55,55,55,50,33,00,00,00,00,03,33,33,33,33,33,33,33,33,04,44,04,44,44,04,44,44,44,00,22,22,22,22,05
                    HEX        05,55,55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,40,44,44,40,44,44,44,40,00,22,22,22,22,05
                    HEX        05,55,55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,40,44,40,04,44,44,44,40,02,22,22,22,20,55
                    HEX        05,55,55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,44,04,44,44,44,44,00,22,22,22,22,20,55
                    HEX        05,55,55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,00,44,44,44,44,40,02,22,22,22,22,05,55
                    HEX        20,55,55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,40,44,44,44,44,44,00,22,22,22,22,22,05,55
                    HEX        20,55,55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,33,33,33,04,00,04,44,44,44,44,40,22,22,22,22,22,20,55,55
                    HEX        20,55,55,55,55,55,50,33,33,33,33,33,33,33,33,33,33,33,33,30,00,44,44,44,44,44,44,02,22,22,22,22,22,05,55,55
                    HEX        20,55,55,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,33,30,44,44,44,44,44,44,00,22,22,22,22,22,22,05,55,55
                    HEX        20,55,55,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,33,04,44,44,44,44,44,40,22,22,22,22,22,22,20,55,55,55
                    HEX        20,55,55,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,30,44,44,44,44,44,40,02,22,22,22,22,22,20,05,55,55,55
                    HEX        20,55,55,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,04,44,44,44,44,40,02,22,22,22,22,22,22,05,55,55,55,55
                    HEX        20,55,55,55,55,55,55,03,33,33,33,33,33,33,33,33,33,33,04,44,44,40,00,02,22,22,22,22,22,22,20,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,50,33,33,33,33,33,33,33,33,33,30,00,00,00,02,22,22,22,22,22,22,22,20,05,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,00,33,33,33,33,33,33,33,00,02,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,00,00,00,00,00,00,00,22,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,02,22,22,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,50,02,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,55,50,00,22,22,22,22,22,22,22,20,00,05,55,55,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,00,00,02,22,00,00,00,05,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,00,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55

Intro_Face_4        HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,11,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,11,11,11,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,11,11,11,01,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,11,11,01,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,01,11,11,11,11,01,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,11,11,11,11,01,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,11,11,11,11,01,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,01,11,11,11,11,11,11,01,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,11,11,11,11,11,11,01,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,11,11,11,11,11,11,01,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,01,11,11,11,11,11,11,01,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,10,11,11,11,11,11,11,11,01,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,01,11,11,11,11,11,11,11,01,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,01,10,11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,F0,11,01,11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,10,00,00,0F,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,01,10,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,01,11,10,00,00,00,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,01,10,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,10,11,11,11,11,11,0F,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,F0,11,01,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,10,11,11,11,11,11,10,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,F0,11,01,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,01,11,11,11,11,11,0F,FF
                    HEX        FF,FF,FF,FF,FF,F0,00,01,10,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,01,11,11,11,11,11,0F,FF
                    HEX        FF,FF,FF,FF,00,01,11,01,10,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,10,11,11,11,11,11,10,FF
                    HEX        FF,FF,F0,00,11,11,10,11,10,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,0F
                    HEX        FF,F0,01,11,11,11,10,11,10,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,01,11,11,11,11,11,10
                    HEX        FF,01,11,11,11,11,10,11,10,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,10,01,11,11,11,11,11,10

                    HEX        00,11,11,11,11,11,10,11,11,00,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,10,01,10,11,11,11,11,10,02
                    HEX        11,11,11,11,11,11,01,11,11,11,00,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,10,01,11,10,11,11,11,11,02,22
                    HEX        11,11,11,11,11,11,01,11,11,11,11,00,01,11,11,11,11,11,11,10,11,11,11,11,11,00,01,11,11,10,00,00,00,00,50,22
                    HEX        11,11,11,11,11,11,01,11,11,11,11,11,10,00,00,11,11,11,11,10,11,11,10,00,00,11,11,11,11,10,55,55,55,55,55,02
                    HEX        11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,00,00,00,00,00,00,00,01,11,11,11,11,11,11,10,05,55,55,55,55,50
                    HEX        11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,05,55,55,55,55,55
                    HEX        11,11,11,11,11,11,00,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,05,55,55,55,55,55
                    HEX        11,11,11,11,11,11,04,01,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,20,55,55,55,55,55
                    HEX        11,11,11,11,11,11,04,00,01,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,20,55,55,55,55,55
                    HEX        11,11,11,11,11,11,04,04,40,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,00,20,55,55,55,55,55
                    HEX        11,11,11,11,11,10,44,04,44,00,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,00,40,22,05,55,55,55,55
                    HEX        11,11,11,11,11,10,44,40,44,44,00,01,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,00,44,40,22,00,05,55,55,55
                    HEX        11,11,11,11,10,00,44,40,44,44,44,40,00,11,11,11,11,11,11,11,11,11,11,11,11,10,00,30,44,44,02,22,20,55,55,55
                    HEX        11,11,11,11,00,40,44,40,44,44,44,03,33,00,00,01,11,11,11,11,11,11,11,00,00,03,33,30,44,44,02,22,20,55,55,55
                    HEX        11,11,11,00,22,00,44,40,44,44,40,33,33,33,33,30,00,00,00,00,00,00,00,33,33,33,33,30,44,44,02,22,22,05,55,55
                    HEX        11,11,00,22,22,00,44,40,44,44,40,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,02,22,22,05,55,55
                    HEX        11,10,22,22,22,00,44,44,04,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,00,22,22,05,55,55
                    HEX        10,02,22,22,22,20,44,44,04,44,03,33,33,33,33,33,33,33,33,03,33,33,33,33,33,33,33,33,04,44,40,02,22,20,55,55
                    HEX        05,02,22,22,22,20,44,44,04,40,33,33,33,33,33,33,33,33,33,30,33,33,33,33,33,33,33,33,04,44,40,02,22,20,55,55
                    HEX        55,02,22,22,22,20,44,44,04,40,33,33,33,33,33,33,33,33,33,30,33,33,33,33,33,33,33,33,04,44,40,40,22,22,05,55
                    HEX        55,50,22,22,22,20,44,44,04,03,33,00,00,00,00,03,33,33,33,30,33,33,33,33,00,00,00,00,00,44,40,44,02,22,05,55
                    HEX        55,50,22,22,22,20,44,44,04,03,00,00,00,00,00,00,00,03,33,30,33,33,33,00,11,00,00,00,30,44,40,44,02,22,05,55
                    HEX        55,50,22,22,22,20,44,44,00,33,33,33,30,11,11,00,00,00,00,33,33,33,30,11,11,00,00,03,30,44,04,44,40,22,20,55
                    HEX        55,50,22,22,22,20,44,44,00,33,33,33,33,00,00,00,00,03,33,33,33,33,00,00,00,00,03,33,30,44,04,44,40,22,20,55
                    HEX        55,55,02,22,22,20,44,44,03,33,33,33,33,33,33,33,00,33,33,33,33,33,33,33,33,33,33,33,33,04,04,44,40,22,20,55
                    HEX        55,55,02,22,22,22,04,44,03,33,33,33,33,33,33,30,03,33,33,33,33,33,33,33,33,33,33,33,33,04,04,44,40,22,20,55
                    HEX        55,55,50,22,22,22,04,44,03,33,33,33,33,30,00,03,33,33,33,33,30,03,33,33,33,33,33,33,33,04,04,44,40,22,20,55
                    HEX        55,55,50,22,22,22,20,44,03,33,33,33,33,33,33,33,33,33,30,33,33,30,03,33,33,33,33,33,33,00,44,44,40,22,22,05
                    HEX        55,55,55,02,22,22,20,44,03,33,33,33,33,33,33,33,33,33,03,33,33,33,30,33,33,33,33,33,33,00,44,44,44,02,22,05
                    HEX        55,55,55,50,22,22,22,04,40,33,33,33,33,33,33,33,33,30,33,33,33,33,33,03,33,33,33,33,33,30,44,44,44,02,22,05
                    HEX        55,55,55,50,22,22,22,04,40,33,33,33,33,33,33,33,33,30,33,33,33,33,33,03,33,33,33,33,33,30,44,44,44,00,22,05
                    HEX        55,55,55,55,02,22,22,20,40,33,33,33,33,33,33,33,33,30,33,33,33,33,33,00,33,33,33,33,33,30,44,44,44,40,22,05
                    HEX        55,55,55,55,50,22,22,20,40,33,33,33,33,33,33,33,33,33,03,33,33,33,30,03,33,33,33,33,33,04,44,44,44,40,22,05
                    HEX        55,55,55,55,55,02,22,20,40,33,33,33,33,33,33,33,33,33,33,33,33,30,03,33,33,33,33,33,33,04,44,40,00,40,22,05
                    HEX        55,55,55,55,55,02,22,20,44,03,33,33,33,33,33,33,33,33,33,33,30,03,33,33,33,33,33,33,30,44,44,40,44,40,22,05
                    HEX        55,55,55,55,55,50,22,22,04,40,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,04,44,40,22,05
                    HEX        55,55,55,55,55,55,02,22,04,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,44,04,44,40,22,05
                    HEX        55,55,55,55,55,55,02,22,04,44,40,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,40,44,44,40,22,05
                    HEX        55,55,55,55,55,55,50,22,20,44,40,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,40,44,44,02,22,05
                    HEX        05,55,55,55,55,55,55,02,20,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,44,44,04,44,44,02,22,05
                    HEX        05,55,55,55,55,55,55,02,22,04,44,03,33,33,33,33,33,33,33,30,00,00,00,00,33,33,33,04,44,44,04,44,40,22,22,05
                    HEX        05,55,55,55,55,55,55,50,22,04,44,03,33,33,33,33,33,33,30,03,33,33,33,33,33,33,33,04,44,40,44,44,02,22,20,55
                    HEX        05,55,55,55,55,55,55,55,02,20,44,40,33,33,33,33,33,33,03,33,33,33,33,33,33,33,30,44,44,04,44,44,02,22,20,55
                    HEX        05,55,55,55,55,55,55,55,50,22,04,40,33,33,33,33,33,33,03,33,33,33,33,33,33,33,30,44,40,44,44,40,22,22,05,55
                    HEX        20,55,55,55,55,55,55,55,50,22,20,40,33,33,33,33,33,33,33,33,33,00,33,33,33,33,30,44,04,44,44,02,22,22,05,55
                    HEX        20,55,55,55,55,55,55,55,55,02,20,44,03,33,33,33,33,33,33,33,00,33,33,33,33,33,30,40,44,44,40,22,22,20,55,55
                    HEX        20,55,55,55,55,55,55,55,55,50,22,04,03,33,33,33,33,33,33,33,33,33,33,33,33,33,00,04,44,44,02,22,22,05,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,02,20,00,33,33,33,33,33,33,33,33,33,33,33,33,33,00,44,44,40,22,22,22,05,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,02,22,00,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,44,02,22,22,20,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,50,22,00,33,33,33,33,33,33,33,33,33,33,33,33,33,04,44,40,22,22,20,05,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,02,20,03,33,33,33,33,33,33,33,33,33,33,33,30,44,40,02,22,22,05,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,50,22,03,33,33,33,33,33,33,33,33,33,33,33,30,40,02,22,22,20,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,50,22,03,33,33,33,33,33,33,33,33,33,33,33,30,02,22,22,20,05,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,02,20,33,33,33,33,33,33,33,33,33,33,33,30,22,22,20,05,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,50,20,33,33,33,33,33,33,33,33,33,33,33,02,22,20,05,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,02,03,33,33,33,33,33,33,33,33,33,33,02,20,05,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,50,00,33,33,33,33,33,33,33,33,33,30,20,05,55,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,55,50,00,33,33,33,33,33,33,33,33,30,05,55,55,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,00,33,33,33,33,33,33,30,05,55,55,55,55,55,55,55,55,55,55,55
                    HEX        05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,00,03,33,33,33,30,00,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,00,00,00,05,55,55,55,55,55,55,55,55,55,55,55,55,55

Intro_Face_5        HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,11,11,00,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,11,11,01,11,11,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,11,10,11,11,11,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,11,11,11,01,11,11,11,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,11,11,11,11,11,10,11,11,11,11,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,10,11,11,11,11,11,01,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,01,11,11,11,11,11,10,11,11,11,11,10,0F,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,10,11,11,11,11,11,11,10,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,01,11,11,11,11,11,11,11,01,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,10,11,11,11,11,11,11,11,11,10,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,10,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,01,11,11,11,11,11,11,11,11,11,01,11,11,11,11,11,10,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,01,11,11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,0F,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,10,11,11,11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,10,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,01,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,10,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,01,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,0F,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,11,01,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,10,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,10,11,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,10,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,01,11,11,11,11,11,10,11,00,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,01,11,11,11,11,10,01,11,01,0F,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,01,11,11,11,11,11,11,11,11,11,11,11,11,01,11,11,11,10,01,11,11,01,10,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,F0,01,11,11,01,11,11,11,11,11,11,11,11,11,11,11,11,01,11,10,00,01,11,11,11,10,11,0F,FF
                    HEX        FF,FF,FF,FF,FF,F0,00,01,01,11,11,01,11,11,11,11,11,11,11,11,11,11,11,10,00,00,01,11,11,11,11,11,10,11,0F,FF
                    HEX        FF,FF,FF,FF,00,01,11,11,01,11,11,01,11,11,11,11,11,11,11,11,11,00,00,00,11,11,11,11,11,11,11,11,10,11,10,FF
                    HEX        FF,FF,F0,00,11,11,11,10,11,11,11,01,11,11,11,11,10,00,00,00,00,11,11,11,11,11,11,11,11,11,11,11,10,11,11,0F
                    HEX        FF,F0,01,11,11,11,11,10,11,00,00,00,00,00,00,00,01,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,11,11,10
                    HEX        FF,01,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,11,11,10

                    HEX        00,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,11,10,02
                    HEX        11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,00,01,02,22
                    HEX        11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,00,44,00,50,22
                    HEX        11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,00,04,44,05,55,02
                    HEX        11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,00,03,33,04,44,40,55,50
                    HEX        11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,00,03,33,33,33,04,44,40,55,55
                    HEX        11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,00,00,03,33,33,33,33,33,30,44,40,55,55
                    HEX        11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,10,00,00,00,33,33,33,33,33,33,33,33,30,44,40,55,55
                    HEX        11,11,11,11,11,11,11,11,00,00,00,00,00,00,00,00,00,00,00,03,33,33,33,33,33,33,33,33,33,33,30,00,44,40,55,55
                    HEX        11,11,11,11,11,11,11,11,04,44,44,44,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,30,00,33,04,40,55,55
                    HEX        11,11,11,11,11,11,11,10,00,44,04,44,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,00,03,33,30,44,05,55
                    HEX        11,11,11,11,11,11,00,04,40,40,40,44,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,30,00,10,33,33,04,05,55
                    HEX        11,11,11,11,10,00,44,40,04,04,44,04,44,44,44,03,33,33,33,33,33,33,33,33,30,03,33,33,00,01,00,03,33,30,05,55
                    HEX        11,11,11,11,00,44,44,04,40,44,44,04,44,44,40,33,33,33,33,33,33,33,33,33,33,30,03,33,00,11,00,03,33,30,55,55
                    HEX        11,11,11,00,22,04,40,44,40,44,44,40,44,44,40,33,33,33,33,33,33,33,33,33,33,33,30,30,01,11,00,03,33,33,05,55
                    HEX        11,11,00,22,22,04,40,44,40,44,44,44,04,44,40,33,33,33,33,00,00,00,00,03,33,33,33,30,30,01,10,33,33,33,00,55
                    HEX        11,10,22,22,22,04,40,44,40,44,44,44,40,44,40,33,33,33,00,00,00,00,00,00,00,00,03,33,33,30,00,33,33,33,30,55
                    HEX        10,02,22,22,22,20,44,04,40,44,44,44,44,44,40,33,00,00,30,01,11,11,00,00,03,33,33,33,03,33,33,00,33,33,30,55
                    HEX        05,02,22,22,22,20,44,04,40,44,44,44,44,44,40,33,33,33,33,30,00,00,00,33,03,33,33,33,30,33,33,33,33,33,30,55
                    HEX        55,02,22,22,22,22,04,04,40,44,44,44,44,44,40,33,33,33,33,33,33,33,33,33,03,33,33,33,33,03,33,33,33,33,30,55
                    HEX        55,50,22,22,22,22,04,40,44,04,44,44,44,44,40,33,33,33,33,33,33,33,33,30,33,33,33,33,33,30,00,33,33,33,05,55
                    HEX        55,50,22,22,22,22,20,40,44,04,44,44,44,44,40,33,33,33,33,33,33,33,30,03,33,33,33,33,33,33,33,03,33,33,05,55
                    HEX        55,50,22,22,22,22,20,40,44,40,44,44,44,44,40,33,33,33,33,30,00,00,03,33,33,33,33,33,33,33,33,03,33,33,00,55
                    HEX        55,50,22,22,22,22,20,40,44,40,44,44,44,44,40,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,03,33,33,00,55
                    HEX        55,55,02,22,22,22,22,04,04,44,04,44,44,04,44,03,33,33,33,33,33,33,33,33,30,33,33,33,33,33,30,33,33,33,00,55
                    HEX        55,55,02,22,22,22,22,04,04,44,40,44,44,04,44,03,33,33,33,33,33,33,33,33,30,33,33,33,33,33,30,33,33,33,00,55
                    HEX        55,55,50,22,22,22,22,20,04,44,44,04,44,04,44,40,33,33,33,33,33,33,33,33,30,33,33,33,33,00,03,33,33,30,20,55
                    HEX        55,55,50,22,22,22,22,22,00,44,44,40,40,04,44,44,03,33,33,33,33,33,33,33,33,00,33,33,00,33,33,33,33,30,22,05
                    HEX        55,55,55,02,22,22,22,22,00,44,44,44,04,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,22,05
                    HEX        55,55,55,50,22,22,22,22,20,44,44,44,44,44,44,44,40,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,22,05
                    HEX        55,55,55,50,22,22,22,22,20,04,44,44,44,44,44,44,40,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,22,05
                    HEX        55,55,55,55,02,22,22,22,22,04,44,44,44,44,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,22,05
                    HEX        55,55,55,55,50,22,22,22,22,04,44,44,44,44,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,22,05
                    HEX        55,55,55,55,55,02,22,22,22,04,44,44,44,44,04,44,44,40,33,33,33,33,33,33,33,33,33,33,30,00,33,33,33,30,22,05
                    HEX        55,55,55,55,55,02,22,22,22,20,44,44,44,44,40,44,44,40,33,33,33,33,33,33,33,33,33,00,03,33,33,33,33,30,22,05
                    HEX        55,55,55,55,55,50,22,22,22,20,44,44,44,44,44,04,44,44,03,33,33,33,33,33,33,33,30,33,33,33,33,33,33,30,22,05
                    HEX        55,55,55,55,55,55,02,22,22,20,44,44,04,44,44,40,44,44,03,33,33,33,33,33,33,33,03,33,33,33,03,33,33,30,22,05
                    HEX        55,55,55,55,55,55,02,22,22,22,04,44,40,44,44,44,04,44,40,33,33,33,33,33,33,30,33,33,33,30,33,33,33,30,22,05
                    HEX        55,55,55,55,55,55,50,22,22,22,04,44,44,04,44,44,40,44,40,33,33,33,33,33,33,30,33,33,00,03,33,33,33,30,22,05
                    HEX        05,55,55,55,55,55,55,02,22,22,04,44,44,04,44,44,44,04,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,30,22,05
                    HEX        05,55,55,55,55,55,55,02,22,22,20,44,44,40,44,44,44,40,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,30,22,05
                    HEX        05,55,55,55,55,55,55,50,22,22,20,44,44,44,04,44,44,44,04,40,33,33,33,33,33,33,33,33,33,33,33,33,33,30,20,55
                    HEX        05,55,55,55,55,55,55,55,02,22,22,04,44,44,40,04,44,44,40,40,33,33,33,33,33,33,33,33,33,33,33,33,33,30,20,55
                    HEX        05,55,55,55,55,55,55,55,50,22,22,20,44,44,44,40,44,44,44,04,03,33,33,33,33,33,33,33,33,33,33,33,33,30,05,55
                    HEX        20,55,55,55,55,55,55,55,50,22,22,20,44,44,44,44,04,44,44,40,03,33,33,33,33,33,33,33,33,33,33,33,33,30,05,55
                    HEX        20,55,55,55,55,55,55,55,55,02,22,22,04,44,44,44,40,04,44,44,40,33,33,33,33,33,33,33,33,33,33,33,33,30,55,55
                    HEX        20,55,55,55,55,55,55,55,55,50,22,22,20,44,44,44,44,40,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,30,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,02,22,22,04,44,44,44,44,00,04,44,40,33,33,33,33,33,33,33,33,33,33,33,30,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,02,22,22,20,44,44,44,44,44,40,00,04,03,33,33,33,33,33,33,33,33,33,33,05,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,50,22,22,22,04,44,44,44,44,44,44,44,40,33,33,33,33,33,33,33,33,33,30,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,02,22,22,20,44,44,44,44,44,44,44,44,03,33,33,33,33,33,33,33,33,30,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,50,22,22,22,04,44,44,44,44,44,44,44,40,33,33,33,33,33,33,33,33,05,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,50,22,22,22,20,04,44,44,44,44,44,44,44,00,33,33,33,30,00,00,00,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,02,22,22,22,20,00,00,00,00,00,00,00,02,00,00,00,00,05,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,50,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,02,22,22,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,50,02,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,55,50,00,22,22,22,22,22,22,22,20,00,05,55,55,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,00,00,02,22,00,00,00,05,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,00,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55

Intro_Face_6        HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,11,10,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,11,01,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,01,11,11,11,01,11,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,00,11,11,11,11,10,11,11,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,01,11,11,11,11,10,11,11,11,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,01,11,11,11,11,11,01,11,11,10,00,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,10,11,11,11,11,11,11,01,11,11,11,10,0F,FF,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,01,11,11,11,11,11,11,10,11,11,11,11,00,0F,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,01,11,11,11,11,11,11,10,11,11,11,11,11,00,FF,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,10,11,11,11,11,11,11,11,10,11,11,11,11,11,10,0F,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,10,11,11,11,11,11,11,11,11,01,11,11,11,11,11,00,FF,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,01,11,11,11,11,11,11,11,11,01,11,11,11,11,11,10,0F,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,01,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,00,FF,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,10,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,10,0F,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,10,11,11,11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,00,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,10,11,11,11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,10,FF,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,01,11,11,11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,10,0F,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,00,11,11,01,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,00,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,FF,01,11,11,01,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,10,FF,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,F0,01,11,10,11,11,11,11,11,11,11,11,11,11,11,10,11,11,11,11,11,11,10,00,00,FF,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,01,11,11,11,11,10,01,11,00,0F,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,FF,F0,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,01,11,11,11,00,01,11,11,10,10,FF,FF
                    HEX        FF,FF,FF,FF,FF,FF,FF,F0,01,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,01,11,00,00,11,11,11,11,10,11,0F,FF
                    HEX        FF,FF,FF,FF,FF,F0,00,01,01,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,00,00,11,11,11,11,11,11,10,11,0F,FF
                    HEX        FF,FF,FF,FF,00,01,11,11,01,11,11,10,11,11,11,11,11,11,11,11,11,10,00,00,11,11,11,11,11,11,11,11,11,01,10,FF
                    HEX        FF,FF,F0,00,11,11,11,10,11,11,11,10,11,11,11,11,11,11,00,00,00,01,11,11,11,11,11,11,11,11,11,11,11,01,11,0F
                    HEX        FF,F0,01,11,11,11,11,10,10,00,00,00,00,00,00,00,00,00,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,01,11,10
                    HEX        FF,01,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,01,11,10

                    HEX        00,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,01,10,02
                    HEX        11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,04,01,02,22
                    HEX        11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,00,04,44,40,50,22
                    HEX        11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,00,33,30,44,40,55,02
                    HEX        11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,10,00,03,33,33,30,44,40,55,50
                    HEX        11,11,11,11,11,11,11,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,00,00,03,33,33,33,33,33,04,40,55,55
                    HEX        11,11,11,11,11,11,11,11,01,11,11,11,11,11,11,11,11,11,11,11,11,00,00,00,33,33,33,33,33,33,33,33,30,40,55,55
                    HEX        11,11,11,11,11,11,11,11,00,00,00,01,11,11,11,00,00,00,00,00,00,33,33,33,33,33,33,33,33,33,33,33,30,44,05,55
                    HEX        11,11,11,11,11,11,11,11,00,44,44,40,00,00,00,44,40,33,33,33,33,33,33,33,33,33,33,33,33,33,00,00,00,04,05,55
                    HEX        11,11,11,11,11,11,11,11,04,44,44,44,44,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,30,00,00,33,04,05,55
                    HEX        11,11,11,11,11,11,11,10,00,40,04,44,44,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,00,03,33,33,04,05,55
                    HEX        11,11,11,11,11,11,00,04,40,04,40,44,44,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,00,10,33,33,30,05,55
                    HEX        11,11,11,11,10,00,44,44,40,44,44,04,44,44,44,40,33,33,33,33,33,33,33,33,33,30,33,33,30,11,10,03,33,30,05,55
                    HEX        11,11,11,11,00,44,44,44,40,44,44,40,44,44,44,40,33,33,33,33,33,33,33,33,33,33,03,33,30,11,00,03,33,33,05,55
                    HEX        11,11,11,00,22,04,44,40,00,44,44,44,44,44,44,03,33,33,33,33,33,33,33,33,33,33,03,33,01,11,00,00,03,33,05,55
                    HEX        11,11,00,22,22,04,44,04,40,44,44,44,44,44,44,03,33,33,30,00,00,00,00,03,33,33,30,00,00,00,33,33,33,33,05,55
                    HEX        11,10,22,22,22,04,40,44,40,44,44,44,44,44,44,03,33,00,00,00,00,00,00,00,00,03,30,03,33,33,33,33,33,33,05,55
                    HEX        10,02,22,22,22,20,44,04,40,44,44,44,44,44,40,33,33,03,00,01,11,11,10,00,00,33,33,03,33,33,33,33,33,33,05,55
                    HEX        05,02,22,22,22,20,44,04,44,04,44,44,44,44,40,33,33,33,33,30,00,00,00,00,03,33,33,30,03,33,33,33,33,33,05,55
                    HEX        55,02,22,22,22,22,04,04,44,04,44,44,44,44,40,33,33,33,33,33,33,33,33,30,33,33,33,33,30,00,33,33,33,33,05,55
                    HEX        55,50,22,22,22,22,04,40,44,04,44,44,44,44,44,03,33,33,33,33,33,33,33,03,33,33,33,33,33,33,00,33,33,30,05,55
                    HEX        55,50,22,22,22,22,20,40,44,40,44,44,44,44,44,03,33,33,33,33,33,33,00,33,33,33,33,33,33,33,30,00,33,30,05,55
                    HEX        55,50,22,22,22,22,20,40,44,40,44,44,44,44,44,40,33,33,33,33,33,33,33,33,33,30,33,33,33,33,30,33,03,30,20,55
                    HEX        55,50,22,22,22,22,20,40,44,44,04,44,44,44,44,40,33,33,33,33,33,33,33,33,33,03,33,33,33,33,30,33,33,30,20,55
                    HEX        55,55,02,22,22,22,22,04,04,44,04,44,44,44,44,44,03,33,33,33,33,33,33,33,33,03,33,33,33,33,30,33,33,30,20,55
                    HEX        55,55,02,22,22,22,22,04,04,44,40,44,44,44,44,44,03,33,33,33,33,33,33,33,33,03,33,33,33,33,30,33,33,30,20,55
                    HEX        55,55,50,22,22,22,22,20,04,44,44,04,44,44,44,44,40,33,33,33,33,33,33,33,33,03,33,30,03,33,03,33,33,30,20,55
                    HEX        55,55,50,22,22,22,22,22,00,44,44,40,04,40,44,44,44,03,33,33,33,33,33,33,33,30,00,33,30,00,33,33,33,30,22,05
                    HEX        55,55,55,02,22,22,22,22,00,44,44,44,40,04,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,02,22,05
                    HEX        55,55,55,50,22,22,22,22,20,44,44,44,44,44,44,44,44,40,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,02,22,05
                    HEX        55,55,55,50,22,22,22,22,20,04,44,44,44,44,44,44,44,40,33,33,33,33,33,33,33,33,33,33,33,33,33,33,33,02,22,05
                    HEX        55,55,55,55,02,22,22,22,22,04,44,44,44,04,44,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,02,22,05
                    HEX        55,55,55,55,50,22,22,22,22,04,44,44,44,40,44,44,44,44,03,33,33,33,33,33,33,33,33,33,33,00,00,03,33,02,22,05
                    HEX        55,55,55,55,55,02,22,22,22,04,44,44,44,44,00,44,44,40,33,33,33,33,33,33,33,33,33,30,00,33,33,30,33,30,22,05
                    HEX        55,55,55,55,55,02,22,22,22,20,44,44,44,44,44,04,44,40,33,33,33,33,33,33,33,33,33,30,33,33,33,30,03,30,22,05
                    HEX        55,55,55,55,55,50,22,22,22,20,44,44,44,44,44,40,44,40,33,33,33,33,33,33,33,33,00,03,33,33,30,03,33,30,22,05
                    HEX        55,55,55,55,55,55,02,22,22,22,04,44,40,44,44,44,04,03,33,33,33,33,33,33,33,33,33,33,33,33,03,33,33,30,22,05
                    HEX        55,55,55,55,55,55,02,22,22,22,04,44,40,44,44,44,40,03,33,33,33,33,33,33,33,33,33,33,33,30,33,33,33,30,22,05
                    HEX        55,55,55,55,55,55,50,22,22,22,20,44,44,04,44,44,44,40,33,33,33,33,33,33,33,33,33,33,00,03,33,33,33,30,22,05
                    HEX        05,55,55,55,55,55,55,02,22,22,20,44,44,04,44,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,22,05
                    HEX        05,55,55,55,55,55,55,02,22,22,22,04,44,40,44,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,22,05
                    HEX        05,55,55,55,55,55,55,50,22,22,22,04,44,44,04,44,44,44,40,33,33,33,33,33,33,33,33,33,33,33,33,33,33,30,20,55
                    HEX        05,55,55,55,55,55,55,55,02,22,22,20,44,44,40,04,44,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,33,00,55
                    HEX        05,55,55,55,55,55,55,55,50,22,22,20,44,44,44,40,44,44,44,40,33,33,33,33,33,33,33,33,33,33,33,33,33,30,05,55
                    HEX        20,55,55,55,55,55,55,55,50,22,22,22,04,44,44,44,00,44,44,40,33,33,33,33,33,33,33,33,33,33,33,33,33,30,05,55
                    HEX        20,55,55,55,55,55,55,55,55,02,22,22,04,44,44,44,44,00,44,44,03,33,33,33,33,33,33,33,33,33,33,33,33,33,05,55
                    HEX        20,55,55,55,55,55,55,55,55,50,22,22,20,44,44,44,44,44,00,04,40,33,33,33,33,33,33,33,33,33,33,33,33,30,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,02,22,20,44,44,44,44,44,44,40,00,00,33,33,33,33,33,33,33,33,33,33,33,30,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,02,22,22,04,44,44,44,44,44,44,44,44,03,33,33,33,33,33,33,33,33,33,33,05,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,50,22,22,04,44,44,44,44,44,44,44,44,40,33,33,33,33,33,33,33,33,33,00,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,02,22,20,04,44,44,44,44,44,44,44,44,00,33,33,33,33,33,33,33,00,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,50,22,22,20,00,44,44,44,44,44,44,44,44,03,33,33,33,33,30,00,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,50,22,22,22,22,00,04,44,44,44,00,00,00,00,00,00,00,00,00,05,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,02,22,22,22,22,20,00,00,00,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,50,22,22,22,22,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,02,22,22,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,50,02,22,22,22,22,22,22,22,22,22,22,20,05,55,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,55,50,00,22,22,22,22,22,22,22,20,00,05,55,55,55,55,55,55,55,55,55,55
                    HEX        20,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,00,00,02,22,00,00,00,05,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,50,00,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55
                    HEX        05,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55

                    
************************************************
*  Black & White colors in Background Screens. *
************************************************

ColorSceneTab       HEX        0100,0200,0300,0400,0500,0600,0601,0700  ; Scene ID | Screen ID
                    HEX        0701,0800,0801,0802,0900,0901,0A00,0A01
                    HEX        0A02,0A03,0A04,0A05,0A06,0A07,0B00,0B01
                    HEX        0B02,0C00,0C01,0D00,0D01,0D02,0D03,0D04
                    HEX        0D05,0D06,0E00,0E01,0F00,0F01,0F02,0F03
                    HEX        0F04,0F05,1000,1001,1002,1003,1004,1005
                    HEX        0000

ColorBWTab          HEX        1100,1100,1100,1100,1100,1100,1100,1100  ; W B  
                    HEX        1100,1100,1100,1100,1100,1100,1100,1100
                    HEX        1100,1100,1100,1100,1100,1100,1100,1100
                    HEX        1100,1100,1100,1100,0022,77FF,77FF,77FF
                    HEX        77FF,77FF,1100,1100,1100,1100,1100,1100
                    HEX        1100,1100,1100,1100,1100,1100,1100,1100

*------------------------ Other Source Files -----------------------------------

*--  Sound Functions
                    PUT        DragonsLair.Sound    ; Sound Functions
                
********************************************************************************

